package com.gx.obe.common.file.adapter;

import java.io.File;
import java.text.MessageFormat;
import java.util.List;
import java.util.Optional;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.CallBack;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.FileDialogUtils;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.common.file.entity.BidFile;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.widget.dialog.MessageViewDialog;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.util.utils.RuntimeUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.evaluation.EvaluationStepService;

/**
 * @Description: 导出投标文件
 * @author chenxw
 */
public class ExportBidFileAdapter  {
	
	private EvaluationStepService evaluationStepService = new EvaluationStepService();
	
	public void exportFils(Shell parentShell, TenderProjectEntity tenderProject, List<Supplier> supplierList) {
		File file = new File(FileConstants.getBidFilePath(tenderProject.getId()));
		if (!file.exists() || file.listFiles().length <= 0) {
			MessageDialog.openError(parentShell, Messages.Tips_noBidFilePresent);
			return;
		}
//		final List<Supplier> selectionSupplierList = getSelectionSupplier();
//		if (selectionSupplierList.size() == 0) {
//			MessageDialog.openShortMessage("请选择要导出标书的" + supplierName);
//			return;
//		}
		final String selectDir = FileDialogUtils.getDirectorySavePath(Messages.Tips_selectExportDic, "[" + tenderProject.getTenderNo() + "]" + tenderProject.getTenderName());
		if (!StringUtils.isNotEmpty(selectDir)) {
			return;
		}
		// 判断当前是否选择上传投标文件文件
		final MessageViewDialog dialog = new MessageViewDialog(parentShell, SWT.NONE);
		final CallBack callBack = dialog.getCallBack();
		new Thread(new Runnable() {
			public void run() {
				final String saveDir = selectDir + File.separator + FileUtils.filterFileName("[" + tenderProject.getTenderNo() + "  ]  "+ tenderProject.getTenderName()) ;
				if (null != supplierList && supplierList.size() > 0) {
					boolean isSealed = evaluationStepService.isSealed(tenderProject);
					for (Supplier supplier : supplierList) {
						List<BidFile> bidFileList = BidFile.getSupplierBidFileList(tenderProject.getId(), supplier.getId(), null);
						if (null != bidFileList && bidFileList.size() > 0) {
							for (BidFile bidFile : bidFileList) {
								callBack.callBackString("正在导出" + supplier.getSupplierName() + "标书...");
								String bidFileName = bidFile.getName().substring(bidFile.getName().lastIndexOf("."));
								String target = null;
								if (isSealed && bidFile.isNotPublicBid()) {
									target = saveDir.concat(File.separator).concat(supplier.getId()).concat(File.separator).concat(bidFile.getSimpleName()).concat(bidFileName);
								} else {
									target = saveDir.concat(File.separator).concat(supplier.getSupplierName().trim()).concat(File.separator).concat(bidFile.getSimpleName()).concat(bidFileName);
								}
								target = Optional.ofNullable(target)
										.map(File::new)
										.filter(File::exists)
										.filter(File::isFile)
										.map(File::getAbsolutePath)
										.map(t -> t
												.substring(0, t.lastIndexOf("."))
												.concat("-")
												.concat(String.valueOf(System.currentTimeMillis()))
												.concat(bidFileName))
										.orElse(target);
								FileUtils.copy(bidFile.getAbsolutePath(), target);
								callBack.callBackString("   " + MessageFormat.format(Messages.Tips_exportSuccessFormat, bidFile.getSimpleName()));
							}
						} else {
							if (isSealed) {
								callBack.callBackString(supplier.getId() + Messages.Tips_noBidFileCanBeExport);
							} else {
								callBack.callBackString(supplier.getSupplierName() + Messages.Tips_noBidFileCanBeExport);
							}
						}
					}
				}
				File file = new File(saveDir);
				if (file.exists()) {
					Display.getDefault().syncExec(new Runnable() {
						public void run() {
							dialog.showMessage(Messages.Tips_exportSuccess);
							dialog.close();
							if (MessageDialog.openConfirm(parentShell, Messages.Button_openDirectory, Messages.Tips_exportAndOpenDirectory, new String[] { Messages.Button_open, Messages.Button_close })) {
								RuntimeUtils.openDirector(saveDir);
							}
						}
					});
				} else {
					Display.getDefault().syncExec(new Runnable() {
						public void run() {
							MessageDialog.openError(parentShell, Messages.Tips_noBidFileCanBeExport);
							dialog.close();
						}
					});
				}
			}
		}).start();
		dialog.open();
	}
	
}
