package com.gx.obe.common.file.composite;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ExecutorService;

import org.dom4j.Element;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.common.file.inter.IFileDownloadProject;
import com.gx.obe.common.file.listener.DownloadListener;
import com.gx.obe.common.file.listener.IFileDownload;
import com.gx.obe.common.file.shell.FileDownloadShell;
import com.gx.obe.common.file.shell.FileDownloadShell.DownListener;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.sun.media.Log;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 包件下载
 * @author chenxw
 */
public class FileDownloadProjectComposite extends Composite implements IFileDownloadProject {

	private Composite downloadComposite;
	private Map<String, IFileDownload> downloadCompositeMap;
	private TenderProjectEntity tenderProject;
	private Label label;
	private DownListener mainDownListener;

	/**
	 * Create the composite.
	 * 
	 * @param parent
	 * @param style
	 * @param tenderProjectEntity
	 */
	public FileDownloadProjectComposite(Composite parent, int style) {
		super(parent, style);
		setBackground(SWTResourceManager.getColor(235, 240, 244));
		setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.marginHeight = 1;
		gridLayout.marginWidth = 1;
		setLayout(gridLayout);
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent arg0) {
				dispose();
			}
		});

		Composite composite = new Composite(this, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1));
		GridLayout gl_composite = new GridLayout(1, false);
		composite.setLayout(gl_composite);

		label = new Label(composite, SWT.NONE | SWT.WRAP);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		label.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		label.setForeground(SWTResourceManager.getColor(20, 20, 20));
		downloadComposite = new Composite(this, SWT.NONE);
		downloadComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_composite_1 = new GridLayout(1, false);
		gl_composite_1.marginLeft = 0;
		gl_composite_1.verticalSpacing = 1;
		gl_composite_1.marginWidth = 0;
		gl_composite_1.marginHeight = 0;
		downloadComposite.setLayout(gl_composite_1);
		downloadComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		init();
	}

	/**
	 * @Description: 初始化
	 * @author chenxw
	 */
	private void init() {
		downloadCompositeMap = new HashMap<String, IFileDownload>();
	}

	/**
	 * @Description: 添加正在下载文件项
	 * @author chenxw
	 * @param tenderProjectEntity
	 * @param supplierList
	 * @param BIDOPENING_STAGE
	 * @param downloadType
	 * @param executorService
	 * @param downloadListener
	 */
	public void addIFileDownloading(TenderProjectEntity tenderProject, Supplier supplier, String BIDOPENING_STAGE, String downloadType, ExecutorService executorService, DownloadListener downloadListener) {
		label.setText("[".concat(tenderProject.getTenderNo()).concat("]").concat(tenderProject.getTenderName()));
		this.tenderProject = tenderProject;
		if (downloadComposite.isDisposed()) return;
		if (DownloadListener.HTTP_BIDDING.equals(downloadType) || DownloadListener.SERVICE_BIDDING.equals(downloadType) ||DownloadListener.FTP_BIDDING.equals(downloadType)) {
			String key = Messages.BiddingFile;
			addFileDownloadingComposite(key, null, BIDOPENING_STAGE, downloadType, executorService, downloadListener);
		} else {
			String BIDOPENING = "";
			if (TenderProjectEnum.STAGE_BUS_TEC.equals(BIDOPENING_STAGE)) {
				BIDOPENING = "-商务标、技术标";
			}
			if (TenderProjectEnum.STAGE_PRICE.equals(BIDOPENING_STAGE)) {
				BIDOPENING = "-经济标";
			}
			String key = supplier.getSupplierName().concat(BIDOPENING);
			addFileDownloadingComposite(key, supplier, BIDOPENING_STAGE, downloadType, executorService, downloadListener);
		}
		downloadComposite.layout();
	}

	private void addFileDownloadingComposite(String key, Supplier supplier, String BIDOPENING_STAGE, String downloadType, ExecutorService executorService, DownloadListener downloadListener) {
		if (downloadCompositeMap.containsKey(key)) {
			Log.error("重复下载");
			return;
		}
		FileDownloadingComposite composite = new FileDownloadingComposite(downloadComposite, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		composite.addMainDownloadListener(mainDownListener);
		composite.refresh(tenderProject, supplier, BIDOPENING_STAGE, downloadType, executorService, downloadListener);
		downloadCompositeMap.put(key, composite);
	}

	/**
	 * @Description:添加已下载文件项
	 * @author chenxw
	 * @param tenderProjectEntity
	 * @param iFileDownload
	 */
	public void addIFileDownloaded(TenderProjectEntity tenderProject, IFileDownload fileDownload) {
		if (downloadComposite.isDisposed()) return;
		String supplierId = null != fileDownload.getSupplier() ? fileDownload.getSupplier().getId() : "";
		String bidopeningStage = null != fileDownload.getBidopeningStage() ? fileDownload.getBidopeningStage() : "";
		String fileName = null != fileDownload.getFileName() ? fileDownload.getFileName() : "";
		String fileSize = null != fileDownload.getFileSize() ? fileDownload.getFileSize() : "";
		String downloadTime = null != fileDownload.getDownloadTime() ? fileDownload.getDownloadTime() : "";
		String filePath = null != fileDownload.getBidFilePath() ? fileDownload.getBidFilePath() : "";
		label.setText("[".concat(tenderProject.getTenderNo()).concat("]").concat(tenderProject.getTenderName()));
		FileDownloadedComposite composite = null;
		if (downloadCompositeMap.containsKey(fileName)) {
			composite = (FileDownloadedComposite) downloadCompositeMap.get(fileName);
		} else {
			composite = getFileDownloadedComposite();
		}
		composite.refresh(tenderProject.getId(), supplierId, bidopeningStage, filePath, fileName, fileSize, downloadTime);
		downloadCompositeMap.put(fileName, composite);
		downloadComposite.layout();
	}

	/**
	 * @Description: 初始化已下载列表
	 * @author chenxw
	 * @param downloadedElement 
	 */
	public void initFileDownloaded(Element downloadedElement) {
		if (downloadComposite.isDisposed()) return;
		String tenderId = downloadedElement.elementTextTrim("tenderId");
		String tenderNo = downloadedElement.elementTextTrim("tenderNo");
		String tenderName = downloadedElement.elementTextTrim("tenderName");
		String supplierId = downloadedElement.elementTextTrim("supplierId");
		String bidopeningStage = downloadedElement.elementTextTrim("bidopeningStage");
		String fileName = downloadedElement.elementTextTrim("fileName");
		String fileSize = downloadedElement.elementTextTrim("fileSize");
		String downloadTime = downloadedElement.elementTextTrim("downloadTime");
		String filePath = downloadedElement.elementTextTrim("filePath");
		if (StringUtils.isNotEmpty(tenderNo) || StringUtils.isNotEmpty(tenderName)) {
			label.setText("[".concat(tenderNo).concat("]").concat(tenderName));
		}
		FileDownloadedComposite composite = getFileDownloadedComposite();
		composite.refresh(tenderId, supplierId, bidopeningStage, filePath, fileName, fileSize, downloadTime);
		downloadCompositeMap.put(fileName, composite);
		downloadComposite.layout();
	}
	
	private FileDownloadedComposite getFileDownloadedComposite() {
		final FileDownloadedComposite composite = new FileDownloadedComposite(downloadComposite, SWT.NONE);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		composite.addMainDownloadListener(mainDownListener);
		return composite;
	}
	
	/**
	 * @Description: 删除下载文件
	 * @author chenxw
	 * @param iFileDownload
	 */
	public void removeIFileDownload(IFileDownload fileDownload) {
		downloadCompositeMap.remove(fileDownload.getFileName());
		fileDownload.dispose();
		if (!downloadComposite.isDisposed()) {
			downloadComposite.layout();
			layout();
		}
	}

	/**
	 * @Description: 下载完成监听
	 * @author chenxw
	 * @param downloadComplete
	 */
	public void addMainDownloadListener(FileDownloadShell.DownListener mainDownListener) {
		this.mainDownListener = mainDownListener;
	}

	/**
	 * @Description: 是否关闭
	 * @author chenxw
	 * @return
	 */
	public boolean isStop() {
		if (downloadCompositeMap.size() == 0) {
			dispose();
		}
		return downloadCompositeMap.size() == 0;
	}

	/** 
	 * @Description: 关闭所有下载任务
	 * @author chenxw 
	 */
	public void closeDownloadTask() {
		for (String key : downloadCompositeMap.keySet()) {
			IFileDownload download = downloadCompositeMap.get(key);
			download.closeDownloadTask();
		}
	}
	
	public TenderProjectEntity getTenderProject() {
		return tenderProject;
	}
	
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

 

}
