package com.gx.obe.common.file.composite;

import java.io.File;
import java.util.List;

import org.dom4j.Element;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.listener.IFileDownload;
import com.gx.obe.common.file.shell.FileDownloadShell.DownListener;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.RuntimeUtils;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.util.utils.XmlUtils.UpdateXml;
import com.gx.obe.web.entity.Supplier;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class FileDownloadedComposite extends Composite implements IFileDownload{

	private Composite contentComposite;
	private Label fileName_label;
	private Label fileSize_label;
	private Label time_label;
	private String fileName;
	private String fileSize;
	private String downloadTime;
	private String filePath;
	private String tenderId;
	private String supplierId;
	private String bidopeningStage;
	private File xmlFile = new File(FileConstants.SYS_USER_ROOT.concat("downloaded").concat(File.separator).concat("downloaded.xml"));
	private DownListener mainDownListener;
	
	/**
	 * Create the composite.
	 * @param parent
	 * @param style
	 */
	public FileDownloadedComposite(Composite parent, int style) {
		super(parent, style);
		setBackgroundMode(SWT.INHERIT_FORCE);
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginWidth = 0;
		gridLayout.verticalSpacing = 0;
		gridLayout.marginHeight = 0;
		setLayout(gridLayout);
		
		ESepator topSep = new ESepator(this, SWT.NONE);
		topSep.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		topSep.setLineStyle(SWT.LINE_DOT);
		topSep.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 4, 1));
		
		contentComposite = new Composite(this, SWT.NONE);
		contentComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		GridLayout gl_contentComposite = new GridLayout(5, false);
		contentComposite.setLayout(gl_contentComposite);
		
		fileName_label = new Label(contentComposite, SWT.WRAP);
		fileName_label.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		fileName_label.setForeground(ColorConstants.FONT_BLACK_COLOR);
		GridData gd_messageLabel = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
//		gd_messageLabel.widthHint = 200;
		fileName_label.setLayoutData(gd_messageLabel);
		
		fileSize_label = new Label(contentComposite, SWT.NONE);
		fileSize_label.setForeground(SWTResourceManager.getColor(100, 100, 100));
		fileSize_label.setText("100.00GB");
		
		time_label = new Label(contentComposite, SWT.NONE);
		GridData gd_label = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_label.widthHint = 135;
		time_label.setLayoutData(gd_label);
		time_label.setForeground(SWTResourceManager.getColor(100, 100, 100));
		time_label.setText("用时：1小时11分钟11秒");

		ELink openFilePath = new ELink(contentComposite, SWT.NONE);
		openFilePath.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/location.png"));
		openFilePath.setText(Messages.Button_openDirectory);
		openFilePath.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				File bidFile = new File(filePath);
				if (!bidFile.exists()) {
					bidFile.mkdirs();
				}
				RuntimeUtils.openDirector(filePath);
			}
		});
		
		ELink deleteFileRecord = new ELink(contentComposite, SWT.NONE);
		deleteFileRecord.setText(Messages.Button_delete);
		deleteFileRecord.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/delete.png"));
		deleteFileRecord.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				removeDownloadXml();
			}
		});
		init();
	}

	private void init() {
		
	}

	/** 
	 * @Description: 添加主监听
	 * @author chenxw
	 * @param mainDownListener 
	 */
	public void addMainDownloadListener(DownListener mainDownListener) {
		this.mainDownListener = mainDownListener;
	}
	
	/**
	 * @Description: 删除XML文件已下载信息
	 * @author chenxw 
	 */
	private void removeDownloadXml() {
		XmlUtils.updateXmlFile(xmlFile, new UpdateXml() {
			public void update(Element rootElement) {
				Element removeElement = null;
				List<?> downloadeds = rootElement.elements("downloaded");
				for (Object object : downloadeds) {
					Element downloadedElement = (Element) object;
					if (tenderId.equals(downloadedElement.elementTextTrim("tenderId")) && fileName.equals(downloadedElement.elementTextTrim("fileName"))) {
						removeElement = downloadedElement;
					}
				}
				if (null != removeElement) {
					rootElement.remove(removeElement);
				}
			}
		});
		mainDownListener.closeDownloaded(tenderId, this);
	}

	/**
	 * @Description: 已下载列表刷新
	 * @author chenxw
	 * @param filePath
	 * @param fileName
	 * @param fileSize 
	 * @param downloadTime 
	 */
	public void refresh(String tenderId, String supplierId, String bidopeningStage, String filePath, String fileName, String fileSize, String downloadTime) {
		this.tenderId = tenderId;
		this.supplierId = supplierId;
		this.bidopeningStage = bidopeningStage;
		this.filePath = filePath;
		this.fileName = fileName;
		this.fileSize = fileSize;
		this.downloadTime = downloadTime;
		fileName_label.setText(fileName);
		fileSize_label.setText(fileSize);
		time_label.setText(Messages.Tips_timeUsed +"：".concat(downloadTime));
	}

	public String getBidFilePath() {
		return filePath;
	}

	public String getFileName() {
		return fileName;
	}

	public String getFileSize() {
		return fileSize;
	}

	public String getDownloadTime() {
		return downloadTime;
	}

	public void closeDownloadTask() {
		this.dispose();
	}

	public Supplier getSupplier() {
		Supplier supplier = new Supplier();
		supplier.setId(supplierId);
		return supplier;
	}

	public String getBidopeningStage() {
		return bidopeningStage;
	}
	
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}

}
