package com.gx.obe.common.file.dialog;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.nebula.widgets.grid.listener.GridSelectionListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.ImageResolver;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.EMenu;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.business.entity.MenuFunction;
import com.gx.obe.business.enumeration.MenuFunctionEnum;
import com.gx.obe.common.file.adapter.ExportBidFileAdapter;
import com.gx.obe.common.file.entity.BidFile;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.composite.TipsComposite;
import com.gx.obe.common.widget.utils.MenuFunctionUtils;
import com.gx.obe.components.core.enumeration.CommonEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.RuntimeUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.entity.tender.TenderProjectRuleEntity;
import com.gx.obe.web.service.ProjectRuleService;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 评标时标书本地加载
 * @author guoyr
 */
public class BidFileLocalBorwseDialog extends EDialog {

	private Grid grid;
	private ImageResolver imageResolver = new ImageResolver();
	private TenderProjectEntity tenderProject;
	private Logger LOG = Logger.getLogger(BidFileLocalBorwseDialog.class);
	private List<BidFile> bidFileList;
	private Supplier curSupplier;
	private MenuFunctionUtils menuFunctionUtils;
	private ProjectRuleService projectRuleService;
	{
		LogUtils.logClass(BidFileLocalBorwseDialog.class);
	}

	public BidFileLocalBorwseDialog(Shell parentShell, TenderProjectEntity tenderProject, Supplier curSupplier, List<BidFile> bidFileList, MenuFunctionUtils menuFunctionUtils) {
		super(parentShell);
		this.tenderProject = tenderProject;
		this.bidFileList = bidFileList;
		this.curSupplier = curSupplier;
		this.menuFunctionUtils = menuFunctionUtils;
		this.setTitle(Messages.Button_viewLocalFiles);
		this.setShellStyle(SWT.MAX | SWT.CLOSE);
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 0;
		gl_container.horizontalSpacing = 0;
		gl_container.marginHeight = 0;
		gl_container.marginWidth = 0;
		container.setLayout(gl_container);
		
		Composite toolComposite = new Composite(container,SWT.None);
		GridLayout gl_toolComposite = new GridLayout(1, false);
		gl_toolComposite.verticalSpacing = 0;
		gl_toolComposite.horizontalSpacing = 0;
		gl_toolComposite.marginHeight = 5;
		gl_toolComposite.marginWidth = 5;
		toolComposite.setLayout(gl_toolComposite);
		toolComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		TipsComposite tipsLabel = new TipsComposite(toolComposite, SWT.NONE|SWT.WRAP);
		tipsLabel.setForeground(ColorConstants.FONT_BLACK_COLOR);
		tipsLabel.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.BOLD));
//		tipsLabel.setText(Messages.Tips_openClarifyFiles);
		tipsLabel.addTips(Messages.Tips_openClarifyFiles);
		tipsLabel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		grid = new Grid(container, SWT.V_SCROLL);
		grid.setHeaderVisible(true);
		grid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		GridColumn columnNo = new GridColumn(grid, SWT.CENTER);
		columnNo.setWidth(50);
		columnNo.setMaxmumWidth(50);
		columnNo.setText(Messages.Num);
		GridColumn column = new GridColumn(grid, SWT.NONE);
		column.setWidth(300);
		column.setText(Messages.FileName);
		GridColumn columnSize = new GridColumn(grid, SWT.RIGHT);
		columnSize.setWidth(100);
		columnSize.setMaxmumWidth(150);
		columnSize.setText(Messages.FileSize);

		init();
		return container;
	}

	private void init() {
		projectRuleService = new ProjectRuleService();
		grid.setAutoWidth(true);
		grid.setItemHeight(35);
		
		// 右键菜单事件
		grid.addListener(SWT.Selection, new GridSelectionListener() {
			public void rightMouseButtonDown(GridItem item) {
				gridMenuDetectAction(item);
			}
		});
		
		// 双击事件
		grid.addSelectionListener(new SelectionAdapter() {
			public void widgetDefaultSelected(SelectionEvent e) {
				GridItem item = (GridItem) e.item;
				File file = (File) item.getData();
				if (Program.launch(file.getAbsolutePath())) {} else {
					RuntimeUtils.openFIle(file);
				}
			}
		});
		
		TenderProjectRuleEntity tenderProjectRuleEntity = projectRuleService.getProjectRule(tenderProject.getId());
		boolean viewBidPrice = true;
		// 根据规则设置的查看价格相权限
		if(ObjectUtils.equals(tenderProjectRuleEntity.getViewBidPrice() , CommonEnum.NO)){
			viewBidPrice = false;
		}
		if(null != bidFileList && bidFileList.size() > 0){
			int index = 1;
			for (BidFile bidFile : bidFileList) {
				if (!viewBidPrice && BidFile.TYPE_BIDPRICE.equals(bidFile.getFileType())) {
					continue;
				}
				GridItem item = new GridItem(grid, SWT.NULL);
				item.setText(0, (index++) + "");
				item.setText(1, bidFile.getSimpleName());
				item.setImage(1, imageResolver.getSysIcon(bidFile, 24));
				item.setText(2, FileUtils.getFileSize(bidFile.length()));
				item.setData(bidFile);
			}
		}

	}
	
	private void exportFilesAction() {
//		List<Supplier> supplierList = supplierService.getEvaluationStepSupplierList(tenderProject.getId(), evaluationStep);
		List<Supplier> supplierList = new ArrayList<Supplier>();
		supplierList.add(curSupplier);
		// 只导出当前投标人
		new ExportBidFileAdapter().exportFils(getShell(), tenderProject, supplierList);
	}
	
	/**
	 * @Description: 右建菜单
	 * @author guoyr
	 */
	private void gridMenuDetectAction(final GridItem item){
		EMenu gmenu = new EMenu(getShell());
		final File file = (File)item.getData();
		if(null != menuFunctionUtils && MenuFunction.SHOW.equals(menuFunctionUtils.getAuthority(MenuFunctionEnum.OPEN_BID_FILES_DIR))){
			gmenu.addMenuItem(Messages.Button_openDirectory, ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/location.png"), null != file && file.isFile(), new EMenu.ClickListener() {
				public void click() { 
					RuntimeUtils.openDirectorAndSelect(file.getAbsolutePath());
				};
			});
		}
		
		gmenu.addMenuItem(Messages.Button_openFile, ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/open.png"), null != file && file.isFile(), new EMenu.ClickListener() {
			public void click() { 
				if (Program.launch(file.getAbsolutePath())) {
					LOG.debug("File has been launched: " + file);
				} else {
					LOG.error("Unable to launch file: " + file);
					RuntimeUtils.openFIle(file);
				}
			};
		});
		if(null != menuFunctionUtils && MenuFunction.SHOW.equals(menuFunctionUtils.getAuthority(MenuFunctionEnum.EXPORT_BID_FILES))){
			gmenu.addMenuItem(Messages.Button_exportAll, ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/open.png"), null != file && file.isFile(), new EMenu.ClickListener() {
				public void click() { 
					exportFilesAction();
				};
			});
		}
		gmenu.open();
	}
	
	@Override
	public boolean close() {
		imageResolver.disposite();
		return super.close();
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(701, 464);
	}

}
