package com.gx.obe.common.file.dialog;

import java.util.HashMap;
import java.util.Map;
import java.util.Optional;
import java.util.function.Function;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.SwtConstants;
import org.eclipse.swt.core.utils.SwtUtils;
import org.eclipse.swt.core.widgets.BigButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.enumeration.FileTypeEnum;
import com.gx.obe.common.file.listener.ClearProjectFileListener;
import com.gx.obe.common.file.thread.ClearProjectFileThread;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.button.ClearCacheButton;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class ClearProjectBidFileDialog extends EDialog {

	private TenderProjectEntity tenderProject;
	
	private Map<FileTypeEnum, ClearCacheButton> clearCacheButtonMap = new HashMap<>();
	
	private BigButton clearBtn;
	
	private Map<FileTypeEnum, Function<String, String>> filePathFunMap;
	
	private SelectionAdapter selectionAdapter = new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
			cacheBtnSelectedAction();
		}
	};
	
	/**
	 * Create the dialog.
	 * @param parentShell
	 */
	public ClearProjectBidFileDialog(Shell parentShell, TenderProjectEntity tenderProject) {
		super(parentShell);
		this.setTitle(Messages.ClearBidFile);
		this.tenderProject = tenderProject;
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout(1, false));
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setLayout(new RowLayout(SWT.HORIZONTAL));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1));
		
		ClearCacheButton biddingCacheBtn = new ClearCacheButton(composite_1, SWT.BOTTOM);
		biddingCacheBtn.addSelectionListener(selectionAdapter);
		biddingCacheBtn.setText(Messages.BiddingFile);
		biddingCacheBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/book-open_green.png"));
//		biddingCacheBtn.setImageTextForeground(SWTResourceManager.getColor(77, 163, 19));
//		biddingCacheBtn.setImageText("招");
//		biddingCacheBtn.setImageTextFont(SWTResourceManager.getFont("宋体", 40, SWT.NORMAL));
		clearCacheButtonMap.put(FileTypeEnum.BIDDING, biddingCacheBtn);
		
		ClearCacheButton bidCacheBtn = new ClearCacheButton(composite_1, SWT.BOTTOM);
		bidCacheBtn.addSelectionListener(selectionAdapter);
		bidCacheBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/tender_pager.png"));
		bidCacheBtn.setText(Messages.BidFile);
		clearCacheButtonMap.put(FileTypeEnum.BID, bidCacheBtn);
		
		ClearCacheButton answerCacheBtn = new ClearCacheButton(composite_1, SWT.BOTTOM);
		answerCacheBtn.addSelectionListener(selectionAdapter);
		answerCacheBtn.setText(Messages.ClarificationAnswer);
		answerCacheBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/search_hource.png"));
		clearCacheButtonMap.put(FileTypeEnum.ANSWER, answerCacheBtn);
		
		ClearCacheButton reportCacheBtn = new ClearCacheButton(composite_1, SWT.BOTTOM);
		reportCacheBtn.addSelectionListener(selectionAdapter);
		reportCacheBtn.setText(Messages.Report);
		reportCacheBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/word_64.png"), 48, 48);
		clearCacheButtonMap.put(FileTypeEnum.REPORT, reportCacheBtn);
		
		clearBtn = new BigButton(container, SWT.NONE);
		clearBtn.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				clearAction();
			}
		});
		clearBtn.setArcWidth(SwtConstants.BIG_BUTTON_HEIGHT);
		clearBtn.setBorderColor(null);
		clearBtn.setBackgroundColor(SWTResourceManager.getColor(77, 163, 19));
//		clearBtn.setActiveBackgroundColor(SWTResourceManager.getColor(77, 163, 19));
		clearBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/white/delete.png"));
		clearBtn.setText(Messages.Button_quickCleaning);
		clearBtn.setGAP(10);
		clearBtn.setLayoutData(new GridData(SWT.CENTER, SWT.FILL, false, false, 1, 1));
		
		init();
		return container;
	}
	
	private void init() {
		filePathFunMap = new HashMap<>();
		filePathFunMap.put(FileTypeEnum.BIDDING, FileConstants::getBiddingFilePath);
		filePathFunMap.put(FileTypeEnum.BID, FileConstants::getBidFilePath);
		filePathFunMap.put(FileTypeEnum.ANSWER, FileConstants::getAnswerQuestionFilePath);
		filePathFunMap.put(FileTypeEnum.REPORT, FileConstants::getReportFilePath);
		
		refreshProjectFileSize();
		initDefaultSelected();
		cacheBtnSelectedAction();
		
		getShell().setMinimumSize(400, 280);
		getShell().pack();
		SwtUtils.setCenter(getShell());
	}
	
	/**
	 * @Description: 初始化默认选择方式
	 * @author chenxw 
	 */
	private void initDefaultSelected() {
		clearCacheButtonMap.entrySet().stream().map(Map.Entry::getValue).filter(ClearCacheButton::getEnabled).forEach(t -> t.setSelection(true));
	}
	
	/**
	 * @Description: 复选框选择事件
	 * @author chenxw
	 */
	private void cacheBtnSelectedAction() {
		clearBtn.setEnabled(clearCacheButtonMap.entrySet().stream().map(Map.Entry::getValue).anyMatch(ClearCacheButton::getSelection));
	}
	
	/**
	 * @Description: 刷新文件大小
	 * @author chenxw 
	 */
	private void refreshProjectFileSize() {
		clearCacheButtonMap.entrySet().stream().forEach(e -> Optional.of(filePathFunMap)
				.map(m -> m.get(e.getKey()))
				.map(f -> f.apply(tenderProject.getId()))
				.map(FileUtils::getDirectorySize)
				.ifPresent(l -> refreshClearCacheButton(l, e.getValue())));
	}
	
	/**
	 * @Description: 刷新文件勾选按钮
	 * @author chenxw
	 * @param directorySize
	 * @param clearCacheButton 
	 */
	private void refreshClearCacheButton(long directorySize, ClearCacheButton clearCacheButton) {
		Optional.of(directorySize).map(FileUtils::getFileSize).ifPresent(clearCacheButton::setSubText);
		Optional.of(directorySize).map(t -> t > 0).ifPresent(clearCacheButton::setEnabled);
	}
	
	/**
	 * @Description: 清理事件
	 * @author chenxw
	 */
	private void clearAction() {
		if (!clearBtn.getEnabled()) {
			return;
		}
		int fileType = clearCacheButtonMap.entrySet().stream()
				.filter(t -> t.getValue().getSelection())
				.reduce(0, (a, b) -> a | b.getKey().getKey(), (a, b) -> a | b);
		if (fileType == 0) {
			return;
		}
		clearCacheButtonMap.entrySet().stream().map(Map.Entry::getValue).filter(ClearCacheButton::getSelection).forEach(t -> t.setSubText("正在清理中"));
		ClearProjectFileThread thread = new ClearProjectFileThread(tenderProject, fileType);
		thread.addClearProjectFileListener(new ClearProjectFileListener() {
			public void finish() {
				Display.getDefault().syncExec(() -> {
					if (getShell().isDisposed()) {
						return;
					}
					refreshProjectFileSize();
					clearCacheButtonMap.entrySet().stream().map(Map.Entry::getValue).forEach(t -> t.setSelection(false));
					cacheBtnSelectedAction();
				});
			}
		});
		new Thread(thread).start();
	}
	
	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(500, 300);
	}
}
