package com.gx.obe.common.file.download;

 
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;

import com.gx.obe.common.file.download.infer.DownloadFileListener;
import com.gx.obe.components.core.Constants;

/** 
 * @Description: 文件分割器提取
 * @author mazc 
 */
public class FileSplitterFetch extends Thread {
    
	/**
	 * 文件资源URL
	 */
	private String sURL; 
    /**
     * 文件片段开始位置
     */
	 long nStartPos; 
	
    /**
     * 文件片段结束位置
     */
	long nEndPos;
	
	String fileSplitterFetchMD5 = " ";
	
	/**
	 * 线程id
	 */
	private int nThreadID;  
	
    /**
     * 是否下载结束
     */
	 boolean bDownOver = false;
    
    /**
     * 是否停止下载
     */
	private boolean bStop = false;
	
    /**
     * 文件访问接口
     */
	private FileAccessI fileAccessI = null; 
	
	/**
	 * 该线程是否有错误信息
	 */
	private String strErrorsMessage;

	private DownloadFileListener listener;
	
	private boolean pause = false;
	
    /** 
     * @author mazc
     * @param sURL文件资源URL
     * @param sName 要保存的文件名(完整路径,绝对路径)
     * @param nStart 文件指针开始位置
     * @param nEnd 文件指针结束位置
     * @param id 线程ID
     * @throws IOException
     * @throws IOException 
     */
    public FileSplitterFetch(String sURL, String filePaht, long nStart, long nEnd,int id,String fileMD5,DownloadFileListener listener) throws IOException, IOException {
        this.sURL = sURL;
        this.nStartPos = nStart;
        this.nEndPos = nEnd;
        this.listener = listener;
        if(null != fileMD5){
        	this.fileSplitterFetchMD5 = fileMD5;        	
        }
        nThreadID = id;
        File file = new File(filePaht).getParentFile();
        if(!file.exists()){
        	file.mkdirs();
        }
        fileAccessI = new FileAccessI(filePaht, nStartPos);
    }

    public void run(){
    	while (nStartPos < nEndPos && !bStop){
    		checkPause();
    		InputStream input = null;
    		try{
            	URL url = new URL(sURL);
                HttpURLConnection httpConnection = (HttpURLConnection) url.openConnection();
                httpConnection.setRequestProperty("Charsert", "UTF-8"); //设置请求编码
                httpConnection.setRequestMethod("POST");
                httpConnection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
                httpConnection.setRequestProperty("User-Agent", "NetFox");
                httpConnection.setRequestProperty("ATUOKTHEN", Constants.TOKEN);
                String sProperty = "bytes=" + nStartPos + "-";
                httpConnection.setRequestProperty("bytesRange", sProperty);
                input = httpConnection.getInputStream();
                byte[] b = new byte[1024];
                int nRead;
                //不用再判断 nRead+nStartPos<nEndPos,只需要 nStartPos<nEndPos就可以,
                //因为是前面几个下载线程读取的内容超出了nEndPos,也会被它后面的子线程读取内容覆盖掉,
                //最后一个子下载子线程最后读取到的字节个数小于1024的,所以总的结束位置不超过就可以
                while ((nRead = input.read(b, 0, 1024)) > 0 && nStartPos < nEndPos && !bStop){
                	checkPause();
                	if (listener != null) {
                		listener.writeSize(nRead);
					}
                    nStartPos += fileAccessI.write(b, 0, nRead);
                }
                bDownOver = true;
            }catch (Exception e){
            	strErrorsMessage = e.getMessage();
            	e.printStackTrace();
            }finally{
            	if(null != input){
            		try {
						input.close();
					} catch (IOException e) {
						e.printStackTrace();
					}            		
            	}
            }
    	}
    	fileAccessI.close();
     }

    public void splitterStop(){
    	bStop = true;
    }
    
    public void setPause(boolean pause) {
    	this.pause = pause;
    }

    public String getErrors(){
    	return strErrorsMessage;
    }
    private void checkPause() {
    	// 暂停下载
    	while (pause) {
    		try {
    			Thread.sleep(1000);
    		} catch (InterruptedException e) {
    				e.printStackTrace();
    		}
    	}
    }
}
