package com.gx.obe.common.file.download;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URL;

import com.gx.obe.common.file.download.infer.DownloadFileListener;
import com.gx.obe.common.file.utils.ComputeUtill;
import com.gx.obe.common.file.utils.FileMd5Util;
import com.gx.obe.components.core.Constants;
import com.gx.obe.util.utils.StringUtil;

/** 
 * @Description: 本地文件
 * @author mazc 
 */
public class SiteFileFetch extends Thread{
	
	//当前下载大小
	private long downLoadTotalSize = 0;
	
	//上一次下载大小
	private long oldDownLoadTotalSize = 0;
	
	//当前下载速度
	double speed = 0; 
	
	//当前下载进度
	int percent = 0; 
	
	// 剩余时间
	long remainderTime = 0;
	
	//计算频率
	long sepTime = 1000;
	
	/**
	 * 文件Md5值
	 */
	String fileMD5 = "";
	
//	long startTime;
    /**
     * 文件信息
     */
    InfoBean infoBean = null;
    
    /**
     * 文件指针开始位置数组
     */
    long[] nStartPos;
 
    /**
     * 文件指针结束位置数组
     */
    long[] nEndPos;
    
    /**
     * 文件指针结束位置数组
     */
    String[] nFileMD5;
 
    /**
     * 子下载线程数组
     */
    FileSplitterFetch[] fileSplitterFetch;

    /**
     * 文件长度 (字节byte)
     */
    long nFileLength;
    /**
     * 是否第一次取文件
     */
    boolean bFirst = true;

    /**
     * 停止标志
     */
    boolean bStop = false;

    /**
     * 暂停标志
     */
    boolean pause = false;
    /**
     * 临时文件用于记录文件下载信息(下载线程个数,每个下载线程的当前文件开始指针,文件结束指针)
     */
    private File tmpFile;

    // 输出到文件的输出流
    DataOutputStream output;

    public boolean isLoading;
    
    private DownloadFileListener listener;
    
    public SiteFileFetch(InfoBean bean,DownloadFileListener listener){
    	this.listener = listener;
        this.infoBean = bean;
//      adjustFileNameForDuplicate(bean);  根据后面业务需求决定是否放开
        //创建文件
        this.tmpFile = new File(bean.getsFilePath() + File.separator + bean.getsFileName() + ".info");
    }  
 
    /**
     * @Description: 读取之前下载保存下来的文件指针位置
     * @author mazc 
     */
    private void readPos(){
    	DataInputStream input = null;
    	try {
    		input = new DataInputStream(new FileInputStream(tmpFile));
    		//获取整个文件的MD5
     		fileMD5 = input.readUTF();
    		// 记录了文件被划分成几个子文件(子任务)
    		int nCount = input.readInt();
    		nStartPos = new long[nCount];
    		nEndPos = new long[nCount];
    		nFileMD5 = new String[nCount];
    		for (int i = 0; i < nStartPos.length; i++){
    			nStartPos[i] = input.readLong();
    			nEndPos[i] = input.readLong();
    			nFileMD5[i] = input.readUTF();
    		}
    	} catch (FileNotFoundException e) {
    		e.printStackTrace();
		} catch (IOException e) {
 			e.printStackTrace();
		}finally{
			try {
				input.close();
			} catch (IOException e) {
 				e.printStackTrace();
			}
		}
    }
    
    /**
     * @Description: 保存下载信息 文件指针位置
     * @author mazc 
     */
    private void writePos(){
    	try{
    		output = new DataOutputStream(new FileOutputStream(tmpFile));
    		output.writeUTF(fileMD5);
    		output.writeInt(nStartPos.length);
    		for (int i = 0; i < nStartPos.length; i++){
    			output.writeLong(fileSplitterFetch[i].nStartPos);
    			output.writeLong(fileSplitterFetch[i].nEndPos);
    			output.writeUTF(fileSplitterFetch[i].fileSplitterFetchMD5);
        	}
        }catch (IOException e){
        	e.printStackTrace();
        }catch (Exception e){
        	e.printStackTrace();
        }finally{
        	try {
				output.close();
			} catch (IOException e) {
 				e.printStackTrace();
			}
        }
    }
    
    
    
    public void run(){
    	//获得文件长度
        this.nFileLength = getFileSize();
        if (tmpFile.exists()){
            // 临时文件存在,则认为不是第一次下载,之前有下载过,但是没下载完成(断点续传下载)
            bFirst = false;
            readPos();
        }else{
        	int iSplitter = infoBean.getnSplitter();
        	if(iSplitter == 0){
        		iSplitter = ComputeUtill.setContentLength(nFileLength);     
        	}	
            // 文件指针开始位置数组个数取决于文件被分割成子文件的个数
            nStartPos = new long[iSplitter];
            nEndPos = new long[iSplitter];
            nFileMD5 =  new String[iSplitter];
        }
    	try{
            if (nFileLength == -1){
                isLoading = false;
                bStop = true;
                System.err.println("获取文件长度失败");
                return;
            }else if (nFileLength == -2){
                isLoading = false;
                bStop = true;
                System.err.println("无法访问文件");
                return;
            }
            // 如果是第一次下载
            if (bFirst){
            	File file = new File(infoBean.getsFilePath() + File.separator + infoBean.getsFileName());
            	if(file.exists()){
            		file.delete();
            	}
                // 分配文件指针数组的起始结束位置
                for (int i = 0; i < nStartPos.length; i++){
                	nStartPos[i] = (long) (i * (nFileLength / nStartPos.length));
                }
                for (int i = 0; i < nEndPos.length - 1; i++){
                    nEndPos[i] = nStartPos[i + 1];
                }
                nEndPos[nEndPos.length - 1] = nFileLength;
            }
            // 创建 启动子线程数组
            fileSplitterFetch = new FileSplitterFetch[nStartPos.length];
            isLoading = true;
            checkPause();
            for (int i = 0; i < nStartPos.length; i++) {
            	checkPause();
                fileSplitterFetch[i] = new FileSplitterFetch( infoBean.getsSiteURL(), infoBean.getsFilePath() + File.separator + infoBean.getsFileName(),nStartPos[i], nEndPos[i], i,nFileMD5[i],listener);
                fileSplitterFetch[i].setPause(pause);
                fileSplitterFetch[i].start();
            }
            boolean breakWhile = false;
         // 如果下载没有停止,则每隔1s去保存一次文件指针信息到临时文件
            while (!bStop){
            	//将指针信息写入临时文件
            	writePos();
            	compute(sepTime);
                breakWhile = true;
                for (int i = 0; i < nStartPos.length; i++){
                	// 只要其中有一个没下载完成,
                	if (!fileSplitterFetch[i].bDownOver){
                		breakWhile = false;
                		break;
                	}
                }
                if (breakWhile){
                	break;
                }
                try{
                	Thread.sleep(sepTime);
                }catch (Exception e){
                	e.printStackTrace();
                }
            }
            compute(sepTime);
            MD5check();
            percent = 100;
            isLoading = false;
            if(null != listener){
            	for (int i = 0; i < nStartPos.length; i++){
            		// 只要其中有一个没下载完成,
            		if (!StringUtil.empty(fileSplitterFetch[i].getErrors())){
            			listener.complate(false);
             			return ;
            		}
            	}
            	listener.complate(true); 
            	if(tmpFile != null && tmpFile.exists()){
        			//全部下载完成,则删除临时文件,
        			tmpFile.delete();
        		}
            }
        }catch (Exception e){
            isLoading = false;
            e.printStackTrace();
        }
    }

    /**
     * @Description: 进行相应的计算
     * @author mazc
     * @param sepTime 
     */
    private void compute(long sepTime){
    	computeherLoadProgress();
    	computeDownloadSpeed(sepTime);
    	computeRemainingTimeCurrent();
    } 
 
    /**
     * @Description: 获得文件长度
     * @author mazc
     * @return 
     */
    public long getFileSize(){
    	int nFileLength = -1;
        try{
        	URL url = new URL(infoBean.getsSiteURL());
            HttpURLConnection httpConnection = (HttpURLConnection) url.openConnection();
            httpConnection.setRequestProperty("Charsert", "UTF-8"); //设置请求编码
            httpConnection.setRequestMethod("POST");
            httpConnection.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
            httpConnection.setRequestProperty("GetContent-MD5", "MD5");
            httpConnection.setRequestProperty("ATUOKTHEN", Constants.TOKEN);
            int responseCode = httpConnection.getResponseCode();
            if (responseCode >= 400){
            	processErrorCode(responseCode);
                return -2; 
            }
            String sHeader;
            for (int i = 1;; i++){
            	sHeader = httpConnection.getHeaderFieldKey(i);
                if (sHeader != null){
                    if (sHeader.equals("Content-Length")){
                        nFileLength = Integer.parseInt(httpConnection.getHeaderField(sHeader));
                    }
                    if(sHeader.equals("Content-MD5")){
                    	fileMD5 = httpConnection.getHeaderField(sHeader);
                    }
                }else{
                	break;
                }
            }
        }catch (IOException e){
        	e.printStackTrace();
        }catch (Exception e){
        	e.printStackTrace();
        }
        return nFileLength;
    }

    /**
     * 计算下载进度
     */
    public void computeherLoadProgress(){
    	if(null == fileSplitterFetch){
    		percent = 0;	
    	}
    	// 剩余的字节数
    	long laveLength = 0;
    	for (int i = 0; i < nStartPos.length; i++){
    		laveLength += (fileSplitterFetch[i].nEndPos - fileSplitterFetch[i].nStartPos);
    	}
    	try {
    		percent = (int) ((nFileLength - laveLength) * 100 / nFileLength);
		} catch (Exception e) {
			percent = 100;
 		}
    
    	if(percent == 100){
    		if(tmpFile != null && tmpFile.exists()){
    			//全部下载完成,则删除临时文件,
    			tmpFile.delete();
    		}
    		isLoading = false;
        	bStop = true;
        }
    	if(null != listener){
    		listener.progress(percent);    		
    	}
     }
	
    /**
     * @Description: 计算下载速度
     * @author mazc
     * @return 
     */
    private void computeDownloadSpeed(long minsecond){
    	// 剩余的字节数
        long laveLength = 0;
        for (int i = 0; i < nStartPos.length; i++){
        	laveLength += (fileSplitterFetch[i].nEndPos - fileSplitterFetch[i].nStartPos);
        }
        //文件长度减剩余长度
        downLoadTotalSize = nFileLength - laveLength;
        speed = (downLoadTotalSize - oldDownLoadTotalSize) / minsecond * 1000;
  		oldDownLoadTotalSize = downLoadTotalSize;
  		
  		if(null != listener){
    		listener.speed(ComputeUtill.computeSize(speed).concat("/S")); 		
    	}
    }
    
    /**
     * @Description: 计算时间
     * @author mazc
     * @return 
     */
    public void computeRemainingTimeCurrent(){
    	// 剩余的字节数
    	long laveLength = 0;
    	for (int i = 0; i < nStartPos.length; i++){
        	laveLength += (fileSplitterFetch[i].nEndPos - fileSplitterFetch[i].nStartPos);
        }
    	if(speed == 0){
    		return ;
    	}
    	remainderTime = laveLength / (long)speed;
    	if(null != listener){
    		listener.remainingTime(ComputeUtill.getTime(remainderTime * 1000)); 		
    	}
     }
         
    /**
     * @Description: 判断完整文件是一致
     * @author mazc
     * @return 
     */
    public boolean MD5check(){
		if(bStop){
			boolean isUnanimous = false;
			File tmpFile = new File(infoBean.getsFilePath() + File.separator + infoBean.getsFileName());
			if(fileMD5.equals(FileMd5Util.getFileMD5(tmpFile))){
				isUnanimous = true;
			}else{
				isUnanimous = false;
			}
			if(null != listener){
	    		listener.checkMD5(isUnanimous); 		
	    	}
		}
		return false;
 	}
     
    /**
     * @Description: 获取剩余时间
     * @author mazc
     * @return 
     */
    public String getRemainingTimeCurrent(){
    	return ComputeUtill.getTime(remainderTime * 1000);
    }

    /**
     * @Description: 获取下载速度
     * @author mazc
     * @return 
     */
    public String getDownloadSpeed(){
     	String strDownloadSpeed = ComputeUtill.computeSize(speed).concat("/S");
		return strDownloadSpeed;
    }
    
    
    /**
     * @Description: 获取当前进度
     * @author mazc
     * @return 
     */
    public int gatherLoadProgress(){
    	return percent;
    }
   
    
    private void processErrorCode(int nErrorCode){
        System.err.println("Error Code : " + nErrorCode);
    }

    public boolean isLoading(){
    	return isLoading;
    }
    public void setPause(boolean pause) {
    	this.pause = pause;
    	if(null != nStartPos) {
    		for (int i = 0; i < nStartPos.length; i++){
    			if(null != fileSplitterFetch[i]) {
    				fileSplitterFetch[i].setPause(pause);
    			}
    		}    		
    	}
    }
    /**
     * @Description: 停止文件下载
     * @author mazc 
     */
    public void siteStop(){
        bStop = true;
        isLoading = false;
        if(null != nStartPos) {
        	for (int i = 0; i < nStartPos.length; i++){
        		fileSplitterFetch[i].splitterStop();
        	}
        }
        //数据初始化
        downLoadTotalSize = 0; 
        oldDownLoadTotalSize = 0;
        speed = 0; 
    	percent = 0; 
    	//删除记录文件
//        tmpFile.delete();
        //删除下载文件
        File downLoadFile = new File(infoBean.getsFilePath() + File.separator + infoBean.getsFileName());
        downLoadFile.delete();
    }

	
   
   


    public interface LoadProgressListener{
    	void onstartLoad();
    	void onProgressUpdate(int percent);
    	void onCompleteLoad();
    	void onStopLoad();
    }
    
    /**
     * @Description: 文件名重复则重新命名
     * @author mazc
     * @param bean 
     */
    private void adjustFileNameForDuplicate(InfoBean bean){
        if (bean != null && bean.getsFileName() != null && bean.getsFilePath() != null){
            File file = new File(bean.getsFilePath() + File.separator + bean.getsFileName());
            int lastDotIdx = bean.getsFileName().lastIndexOf(".");
            String prefix = bean.getsFileName().substring(0, lastDotIdx);
            String suffix = bean.getsFileName().substring(lastDotIdx + 1);
            int count = 1;
            if(file.exists()){
            	File loadInfoFile = new File(bean.getsFilePath() + File.separator + bean.getsFileName() + ".info");
            	//如果临时文件存在,则认为是上次没有下载完成的,这是不用重新命名
            	if (loadInfoFile.exists()){
            		return ;
            	}
                String newPrefix = prefix + "(" + count + ")";
                bean.setsFileName(newPrefix + "." + suffix);
                file = new File(bean.getsFilePath() + File.separator + bean.getsFileName());
                count++;
            }
        }
    }
    
    private void checkPause() {
    	// 暂停下载
    	while (pause) {
    		try {
    			Thread.sleep(1000);
    		} catch (InterruptedException e) {
    				e.printStackTrace();
    		}
    	}
    }
    
}
