package com.gx.obe.common.file.entity;

import java.io.File;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.Element;

import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.components.core.enumeration.ExpertEnum;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.util.utils.XmlUtils;

public class BidFile extends File {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	public static final String TYPE_SIGN = "sign";// 签到文件
	public static final String TYPE_SYSTEM = "system";// 系统文件
	public static final String TYPE_SYSTEM_FILES = "system;SystemAttachment";
	public static final String TYPE_NONE = "None";//商务技术标类型
	public static final String TYPE_BUSINESS = "Business";//商务标类型
	public static final String TYPE_TECHNOLOGY = "Technology";//技术标类型
	public static final String TYPE_BIDPRICE = "BidPrice";//价格标类型
	public static final String TYPE_BILL = "Bill";//工程量清单文件
	
	public static final String BID_PRICE_XML = "BidPrice.xml";//投标报价文件
	
	public static final String CA_TYPE = "caType";
	public static final String SIGN_CERTID = "certID";
	public static final String ENC_CERTID = "encCertId";
	
	public static final String PUBLIC_BID_FILE = "public_bid_file";
	public static final String PRIVATE_BID_FILE = "private_bid_file";
	
	private static Logger LOG = Logger.getLogger(BidFile.class);
	
	private String simpleName;
	private String suffix;
	private String fileId;
	private String fileType;
	private String bidFileType;
	private String bookmark;
	private int page = 0;
	private boolean isEncFile = true;
	private boolean notPublicBid = false;
	
	public BidFile(String pathname) {
		super(pathname);
	}
	
	public BidFile(File file) {
		super(file.getAbsolutePath());
	}
	
	public String getSimpleName() {
		if (null == simpleName) {
			int index = this.getName().lastIndexOf(".");
			if (index > 0) {
				if (null == suffix) {
					suffix = this.getName().substring(index + 1);
				}
				simpleName = this.getName().substring(0, index);
			} else {
				if (null == suffix) {
					suffix = "";
				}
				simpleName = this.getName();
			}
		}
		return simpleName;
	}
	
	public void setSimpleName(String simpleName) {
		this.simpleName = simpleName;
	}
	
	public String getSuffix() {
		if (null == suffix) {
			int index = this.getName().lastIndexOf(".");
			if (index > 0) {
				if (null == simpleName) {
					simpleName = this.getName().substring(0, index);
				}
				suffix = this.getName().substring(index + 1);
			} else {
				if (null == simpleName) {
					this.getName();
				}
				suffix = "";
			}
		}
		return suffix;
	}
	
	public void setSuffix(String suffix) {
		this.suffix = suffix;
	}
	
	public int getPage() {
		return page;
	}
	
	public void setPage(int page) {
		this.page = page;
	}
	
	public String getBookmark() {
		return bookmark;
	}
	
	public void setBookmark(String bookmark) {
		this.bookmark = bookmark;
	}
	
	public String getFileId() {
		return fileId;
	}
	
	public void setFileId(String fileId) {
		this.fileId = fileId;
	}
	
	public boolean isEncFile() {
		return isEncFile;
	}
	
	public void setEncFile(boolean isEncFile) {
		this.isEncFile = isEncFile;
	}
	
	public void setNotPublicBid(boolean notPublicBid) {
		this.notPublicBid = notPublicBid;
	}

	public boolean isNotPublicBid() {
		return notPublicBid;
	}
	
	public String getFileType() {
		return null != fileType ? fileType : BidFile.TYPE_NONE;
	}
	
	public void setFileType(String fileType) {
		this.fileType = fileType;
	}
	
	public void setBidFileType(String bidFileType) {
		this.bidFileType = bidFileType;
	}

	public String getBidFileType() {
		return null != bidFileType ? bidFileType : BidFile.TYPE_NONE;
//		return bidFileType;
	}
	
	/**
	 * @Description: 获得待解密的标书
	 * @author chenxw
	 * @param bidFilePath
	 * @param decBusinessBid
	 * @param decTechnologyBid
	 * @param decPriceBid
	 * @return
	 */
	public static List<BidFile> getBidFileList(String bidFilePath, boolean decBusinessBid, boolean decTechnologyBid, boolean decPriceBid) {
		List<BidFile> bidFileList = new ArrayList<BidFile>();
		try {
			File bidFileIndexFile = new File(bidFilePath + FileConstants.BID_FILE_INDEX_XML);
			if (!bidFileIndexFile.exists()) {
				return bidFileList;
			}
			Document bidFileIndexFileDocument = XmlUtils.getXmlDocument(bidFileIndexFile.getAbsolutePath());
			if (null == bidFileIndexFileDocument) {
				return bidFileList;
			}
			Element rootElement = bidFileIndexFileDocument.getRootElement();
			List<?> fileItemElementList = rootElement.elements("fileItem");
			if (null == fileItemElementList || fileItemElementList.size() == 0) {
				return bidFileList;
			}
			for (Object fileItemObject : fileItemElementList) {
				Element fileItemElement = (Element) fileItemObject;
				String bidFileType = getBidFileType(fileItemElement);
				if (null == bidFileType || bidFileType.length() == 0) {
					continue;
				}
				if (BidFile.TYPE_SYSTEM.equals(bidFileType)) {
					continue;
				} else if (BidFile.TYPE_BIDPRICE.equals(bidFileType) && !decPriceBid) {
					continue;
				} else if (BidFile.TYPE_NONE.equals(bidFileType) && (!decBusinessBid || !decTechnologyBid)) {
					continue;
				} else if (BidFile.TYPE_BUSINESS.equals(bidFileType) && !decBusinessBid) {
					continue;
				} else if (BidFile.TYPE_TECHNOLOGY.equals(bidFileType) && !decTechnologyBid) {
					continue;
				}
				
				String bidFileName = getBidFileName(fileItemElement);
				if (null == bidFileName || bidFileName.length() == 0) {
					continue;
				}
				File encryptFile = new File(bidFilePath.concat(bidFileName).concat(FileConstants.ECN));//加密文件 3186b4f38d024375b1afbb3400f56527.pdf.enc
				File decryptFile = new File(bidFilePath.concat(bidFileName));// 解密后的文件3186b4f38d024375b1afbb3400f56527.pdf
				
				BidFile bidFile = null;
				if (encryptFile.exists()) {
					bidFile = new BidFile(encryptFile);
					bidFile.setEncFile(true);
				} else if (decryptFile.exists()) {
					bidFile = new BidFile(decryptFile);
					bidFile.setEncFile(false);
				}
				if (null != bidFile) {
					bidFile.setFileId(fileItemElement.elementTextTrim("fileId"));
					bidFile.setFileType(fileItemElement.elementTextTrim("type"));
					bidFileList.add(bidFile);
				}
			}
		} catch (Exception e) {}
		return bidFileList;
	}
	
	
	/**
	 * @Description: 获得工程量投标清单
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 * @return
	 */
	public static File getSupplierBidBillFile(String tenderId, String supplierId, String suffix) {
		File bidFileIndexFile = new File(FileConstants.getBidFilePath(tenderId, supplierId).concat(FileConstants.BID_FILE_INDEX_XML));
		if (null != bidFileIndexFile && bidFileIndexFile.exists()) {
			Document bidFileIndexFileDocument = XmlUtils.getXmlDocument(bidFileIndexFile.getAbsolutePath());
			if (null != bidFileIndexFileDocument) {
				// 投标信封
				Element rootElement = bidFileIndexFileDocument.getRootElement();
				List<?> fileItemElementList = rootElement.elements("fileItem");
				if (null != fileItemElementList && fileItemElementList.size() > 0) {
					for (Object fileNameObject : fileItemElementList) {
						Element fileNameElement = (Element) fileNameObject;
						String type = "";
						Element fileTypeElement = fileNameElement.element("type");
						if (null != fileTypeElement) {
							type = fileTypeElement.getText();
						}
						if (!type.startsWith(TYPE_BILL)) {
							continue;
						}
						String fileName = fileNameElement.elementTextTrim("fileName");//加密投标文件.pdf.enc
						String fileId = fileNameElement.elementTextTrim("fileId");
						if (fileName.endsWith(FileConstants.ECN)) {
							fileName = fileName.substring(0, fileName.lastIndexOf(FileConstants.ECN));
						}
						if (fileName.endsWith(suffix)) {
							if (fileId.indexOf(".") > 0) {
								// 如果fileId有后缀，则不再取fileName的后缀
								suffix = "";
							} else {
								if (fileName.indexOf(".") > 0) {
									suffix = fileName.substring(fileName.lastIndexOf("."));//加密文件的后缀（为了避免文件中存在多个.）
								}
							}
							return new File(bidFileIndexFile.getParent().concat(File.separator).concat(fileId).concat(suffix));
						}
					}
				}
			}
		}
		return null;
	}
	
	/**
	 * @Description: 获得评审点列表
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @param evaluationFactorCode
	 * @return
	 */
	public static List<EvaluationPoint> getBidFileEvaluationPointList(String evalPointFilePath, String evaluationFactorCode) {
		List<EvaluationPoint> evaluationPointList = new ArrayList<EvaluationPoint>();
		
		File evalPointFile = new File(evalPointFilePath);
		if (null != evalPointFile && evalPointFile.exists()) {
			Document bidFileIndexFileDocument = XmlUtils.getXmlDocument(evalPointFile.getAbsolutePath());
			if (null != bidFileIndexFileDocument) {
				String bidFileDir = evalPointFile.getParent();
				// 投标信封
				Element rootElement = bidFileIndexFileDocument.getRootElement();
				List<?> respItemElementList = rootElement.elements("RespItem");
				if (null != respItemElementList && respItemElementList.size() > 0) {
					for (Object fileNameObject : respItemElementList) {
						Element respItemElement = (Element) fileNameObject;
						
						String evalFactorCode = respItemElement.attributeValue("evalFactorId");
						// 获得指定评审指标的评审点，为空则列出所有的评审点
						if (null != evaluationFactorCode && evaluationFactorCode.length() > 0) {
							if (null != evalFactorCode && !evalFactorCode.equals(evaluationFactorCode)) {
								continue;
							}
						}
						
						String fileName = respItemElement.attributeValue("fileName");//加密投标文件.pdf.enc
						String fileId = respItemElement.attributeValue("fileId");//3186b4f38d024375b1afbb3400f56527
						String suffix = "";
						if (fileName.endsWith(FileConstants.ECN)) {
							fileName = fileName.substring(0, fileName.lastIndexOf(FileConstants.ECN));
						}
						if (fileId.indexOf(".") > 0) {
							// 如果fileId有后缀，则不再取fileName的后缀
							suffix = "";
						} else {
							if (fileName.indexOf(".") > 0) {
								suffix = fileName.substring(fileName.lastIndexOf("."));//加密文件的后缀（为了避免文件中存在多个.）
							}
						}
						File decryptFile = new File(bidFileDir.concat(File.separator).concat(fileId).concat(suffix));// 解密后的文件3186b4f38d024375b1afbb3400f56527.pdf
						
						String bookmark = respItemElement.elementText("bookmark");// 书签
						String pageNumber = respItemElement.elementText("pageNumber"); // pdf页码
						EvaluationPoint point = new EvaluationPoint();
						point.setName(respItemElement.elementText("name"));
						point.setBookmark(bookmark);
						if (null != pageNumber && !"".equals(pageNumber)) {
							point.setPageNo(Integer.parseInt(pageNumber));
						}
						BidFile bidFile = new BidFile(decryptFile.getAbsolutePath());
						bidFile.setFileId(fileId);
						point.setBidFile(bidFile);
						evaluationPointList.add(point);
					}
				}
			}
		}
		return evaluationPointList;
	}
	
	/**
	 * @Description: 获得报价明细文件名称
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 * @param bidPriceIndex
	 * @return
	 */
	public static String getBidPriceDetailFileName(String tenderId, String supplierId, int bidPriceIndex) {
		File xmlFile = new File(FileConstants.getBidFilePath(tenderId, supplierId) + FileConstants.BID_PRICE_XML);
		if (null == xmlFile || !xmlFile.exists()) {
			return null;
		}
		Document document = XmlUtils.getXmlDocument(xmlFile.getAbsolutePath());
		if (null == document) {
			LOG.error(xmlFile.getAbsoluteFile() + "报价文件读取错误");
			return null;
		}
		List<?> bidPriceElementList = document.getRootElement().elements("bidPrice");
		if (bidPriceElementList.size() <= bidPriceIndex) {
			return null;
		}
		Element bidPriceElement = (Element) bidPriceElementList.get(bidPriceIndex);
		return bidPriceElement.attributeValue("priceDetailFileName");
	}
	
	/**
	 * @Description: 获得供应商的质疑文件
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @return
	 */
	public static File getSupplierQuestionFile(String tenderId, String supplierId) {
		File supplierQuestinFileDir = new File(FileConstants.getBidFileQuestionPath(tenderId, supplierId));
		if (supplierQuestinFileDir.exists()) {
			for (File questionFile : supplierQuestinFileDir.listFiles()) {
				if (questionFile.isFile()) {
					return questionFile;
				}
			}
		}
		return null;
	}
	
	/**
	 * @Description: 检查是否存在投标文件
	 * @author chenxw
	 * @param filePath
	 * @param BIDOPENING_STAGE
	 * @return
	 */
	public static boolean checkBidFileExists(String bidFilePath, String BIDOPENING_STAGE) {
		if (null == bidFilePath || bidFilePath.length() == 0) {
			return false;
		}
		File file = new File(bidFilePath.concat(FileConstants.BID_FILE_INDEX_XML));
		if (file.exists()) {
			Document bidFileIndexFileDocument = XmlUtils.getXmlDocument(file.getAbsolutePath());
			if (null == bidFileIndexFileDocument) {
				return false;
			}
			List<?> fileItemElementList = bidFileIndexFileDocument.getRootElement().elements("fileItem");
			if (null == fileItemElementList || fileItemElementList.size() == 0) {
				return false;
			}
			for (Object fileItemObject : fileItemElementList) {
				Element fileItemElement = (Element) fileItemObject;
				
				String bidFileType = getBidFileType(fileItemElement);
				if (BidFile.TYPE_SYSTEM.equals(bidFileType)) {
					continue;
				}
				
				String bidFileName = getBidFileName(fileItemElement);
				if (StringUtils.isEmpty(bidFileName)) {
					continue;
				}
				
				File bidFile = new File(bidFilePath.concat(getBidFileName(fileItemElement)));
				if (bidFile.exists() && bidFile.isFile()) {
					continue;
				}
				
				if (TenderProjectEnum.STAGE_ALL.equals(BIDOPENING_STAGE)) {
					return false;
				} else if (TenderProjectEnum.STAGE_PRICE.equals(BIDOPENING_STAGE)) {
					if (BidFile.TYPE_BIDPRICE.equals(bidFileType) || BidFile.TYPE_BILL.equals(bidFileType)) {
						return false;
					}
				} else if (TenderProjectEnum.STAGE_BUS_TEC.equals(BIDOPENING_STAGE)) {
					if (BidFile.TYPE_NONE.equals(bidFileType) || BidFile.TYPE_BUSINESS.equals(bidFileType) || BidFile.TYPE_TECHNOLOGY.equals(bidFileType)) {
						return false;
					}
				}
			}
			return true;
		}else {
			File fileDir = new File(bidFilePath);
			if(fileDir.exists() && FileUtils.getDirectorySize(fileDir) > 100) {
				return true;
			}
			return false;
		}
	}
	
	/**
	 * @Description: 根据开标阶段获取投标文件
	 * @author chenxw
	 * @param filePath
	 * @param BIDOPENING_STAGE
	 * @return
	 */
	public static List<String> getBidFileListByBidopeningStage(String bidFilePath, String BIDOPENING_STAGE) {
		if (null == bidFilePath || bidFilePath.length() == 0) {
			return null;
		}
		File file = new File(bidFilePath.concat(FileConstants.BID_FILE_INDEX_XML));
		if (!file.exists()) {
			return null;
		}
		Document bidFileIndexFileDocument = XmlUtils.getXmlDocument(file.getAbsolutePath());
		if (null == bidFileIndexFileDocument) {
			return null;
		}
		List<?> fileItemElementList = bidFileIndexFileDocument.getRootElement().elements("fileItem");
		if (null == fileItemElementList || fileItemElementList.size() == 0) {
			return null;
		}
		List<String> fileNameList = new ArrayList<String>();
		for (Object fileItemObject : fileItemElementList) {
			Element fileItemElement = (Element) fileItemObject;
			
			String bidFileName = getBidFileName(fileItemElement);
			if (null == bidFileName || bidFileName.length() == 0) {
				continue;
			}
			
			String bidFileType = getBidFileType(fileItemElement);
			if (null == bidFileType || bidFileType.length() == 0) {
				continue;
			}
			if (BidFile.TYPE_SYSTEM.equals(bidFileType)) {
				fileNameList.add(bidFileName);
			} else if (TenderProjectEnum.STAGE_ALL.equals(BIDOPENING_STAGE)) {
				fileNameList.add(bidFileName);
			} else if (TenderProjectEnum.STAGE_PRICE.equals(BIDOPENING_STAGE) && BidFile.TYPE_BIDPRICE.equals(bidFileType)) {
				fileNameList.add(bidFileName);
			} else if (TenderProjectEnum.STAGE_BUS_TEC.equals(BIDOPENING_STAGE)) {
				if (BidFile.TYPE_BUSINESS.equals(bidFileType) || BidFile.TYPE_TECHNOLOGY.equals(bidFileType) || BidFile.TYPE_NONE.equals(bidFileType)) {
					fileNameList.add(bidFileName);
				}
			}
		}
		return fileNameList;
	}
	
	public static List<BidFile> getBidPriceFileList(String tenderId, String supplierId) {
		List<BidFile> bidFileList = getDecBidFileList(tenderId, supplierId);
		CollectionUtils.filter(bidFileList, t -> !t.getName().endsWith(FileConstants.XML) && BidFile.TYPE_BIDPRICE.equals(t.getBidFileType()));
		return bidFileList;
	}
	
	public static List<BidFile> getBidFileListByEvalBidType(String tenderId, String supplierId, final String evalBidType) {
		List<BidFile> bidFileList = getDecBidFileList(tenderId, supplierId);
		CollectionUtils.filter(bidFileList, t -> {
			if (t.getName().endsWith(FileConstants.XML)) {
				return false;
			}
			String fileType = t.getFileType();
			fileType = t.getBidFileType();
			if (evalBidType.equals(EvaluationFactorEnum.EVAL_BID_BUS) || evalBidType.equals(EvaluationFactorEnum.EVAL_BID_TEC) || evalBidType.equals(EvaluationFactorEnum.EVAL_BID_DETAIL)) {
				if (fileType.equals(BidFile.TYPE_NONE)) {
					return true;
				}
			}
			if (evalBidType.equals(EvaluationFactorEnum.EVAL_BID_BUS) && fileType.equals(BidFile.TYPE_BUSINESS)) {
				return true;
			}
			if (evalBidType.equals(EvaluationFactorEnum.EVAL_BID_TEC) && fileType.equals(BidFile.TYPE_TECHNOLOGY)) {
				return true;
			}
			if (evalBidType.equals(EvaluationFactorEnum.EVAL_BID_PRICE) && fileType.equals(BidFile.TYPE_BIDPRICE)) {
				return true;
			}
			return false;
		});
		return bidFileList;
	}
	
	/**
	 * @Description: 获取投标文件审查列表
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 * @return
	 */
	public static List<BidFile> getBidFileExamineList(String tenderId, String supplierId, final int evalBidType) {
		List<BidFile> bidFileList = getDecBidFileList(tenderId, supplierId);
		CollectionUtils.filter(bidFileList, t -> {
			if (t.getName().endsWith(FileConstants.XML) ) {
				return false;
			}
			
			String fileType = t.getFileType();
			// 把系统附件信息取掉
			if(BidFile.TYPE_SYSTEM.equals(fileType)){
				return false;
			}
			fileType = t.getBidFileType();
			
			if ((evalBidType & ExpertEnum.EXPERT_BUS) == 0 && fileType.equals(BidFile.TYPE_BUSINESS)) {
				return false;
			}
			if ((evalBidType & ExpertEnum.EXPERT_TEC) == 0 && fileType.equals(BidFile.TYPE_TECHNOLOGY)) {
				return false;
			}
//			if ((evalBidType & (ExpertEnum.EXPERT_BUS | ExpertEnum.EXPERT_TEC)) == 0 || fileType.equals(BidFile.TYPE_NONE)) {
//				return false;
//			}
			if ((evalBidType & ExpertEnum.EXPERT_PRICE) == 0) {
				if (BidFile.TYPE_BIDPRICE.equals(fileType) || BidFile.TYPE_BILL.equals(fileType)) {
					return false;
				}
			}
			return true;
		});
		return bidFileList;
	}
	
	/**
	 * @Description: 根据类型获得标书文件
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @param fileType
	 * @return
	 */
	public static List<BidFile> getSupplierBidFileList(String tenderId, String supplierId, String fileType) {
		List<BidFile> documentFileList = new ArrayList<BidFile>();
		List<BidFile> bidFileList = getDecBidFileList(tenderId, supplierId);
		if (null != bidFileList && bidFileList.size() > 0) {
			for (BidFile  bidFile : bidFileList) {
				if (bidFile.getName().endsWith(FileConstants.XML)) {
					continue;
				} else if (bidFile.getName().endsWith(".mdb")) {
					continue;
				}
				documentFileList.add(bidFile);
			}
		}
		return documentFileList;
	}
	
	
	public static List<BidFile> getLocalBidFileList(String tenderId, String supplierId) {
		File bidFileDir = new File(FileConstants.getBidFilePath(tenderId, supplierId));
		if(bidFileDir.exists()){
			List<BidFile> bidFileList = new ArrayList<BidFile>();
			List<String> fileList = new ArrayList<String>();
			FileUtils.getListFiles(bidFileDir.getAbsolutePath(), fileList, null, true);
			for(String filePath : fileList){
				File file = new File(filePath);
				if(file.exists()){
					if(file.getName().endsWith(".xml")){
						continue;
					}
					BidFile bidFile = new BidFile(file);
					bidFile.setFileType(BidFile.TYPE_NONE);
					bidFile.setFileId(file.getAbsolutePath());
					bidFileList.add(bidFile);
				}
			}
			return bidFileList;
		}
		return null;
	}
	
	/**
	 * @Description: 获取投标文件列表
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 * @return
	 */
	public static List<BidFile> getDecBidFileList(String tenderId, String supplierId) {
		File bidFileIndexFile = new File(FileConstants.getBidFilePath(tenderId, supplierId).concat(FileConstants.BID_FILE_INDEX_XML));
		if (bidFileIndexFile.exists() && bidFileIndexFile.isFile()) {
			Document document = XmlUtils.getXmlDocument(bidFileIndexFile.getAbsolutePath());
			if (null != document) {
				Element rootElement = document.getRootElement();
				List<?> objectList = rootElement.elements("fileItem");
				if (null != objectList && objectList.size() > 0) {
					List<BidFile> bidFileList = new ArrayList<BidFile>();
					for (Object object : objectList) {
						Element fileItem = (Element) object;
						String bidFileName = getBidFileName(fileItem);
						if (StringUtils.isNotEmpty(bidFileName)) {
							BidFile bidFile = new BidFile(FileConstants.getBidFilePath(tenderId, supplierId).concat(bidFileName));
							if (bidFile.exists() && bidFile.isFile()) {
								bidFile.setFileId(fileItem.elementTextTrim("fileId"));
								bidFile.setFileType(fileItem.elementTextTrim("type"));
								String isNotPublicBid = fileItem.elementTextTrim("isNotPublicBid");
								if (StringUtils.isEmpty(isNotPublicBid)) {
									isNotPublicBid = fileItem.elementTextTrim("IsNotPublicBid");
								}
								bidFile.setNotPublicBid("true".equalsIgnoreCase(isNotPublicBid));
								if (bidFile.isNotPublicBid()) {
									bidFile.setSimpleName("暗标文件");
								} else {
									bidFile.setSimpleName(fileItem.elementTextTrim("name"));
								}
								bidFile.setBidFileType(getBidFileType(fileItem));
								bidFileList.add(bidFile);
							}
						}
					}
					return bidFileList;
				}
			}
		}else {
			return getLocalBidFileList(tenderId, supplierId);
		}
		return null;
	}
	
	/**
	 * @Description: 获得投标文件文件名
	 * @author chenxw
	 * @param fileItem
	 * @param isEnc
	 * @return
	 */
	public static String getBidFileName(Element fileItem) {
		if (null == fileItem) {
			return null;
		}
		String fileId = fileItem.elementTextTrim("fileId");
		if (StringUtils.isEmpty(fileId)) {
			return null;
		}
		if (fileId.indexOf(".") != -1) {
			if (fileId.indexOf(".zip") > 0) {
				return fileId.substring(0, fileId.indexOf(".zip"));
			}
			return fileId;
		}
		String fileName = fileItem.elementTextTrim("fileName");
		if (StringUtils.isEmpty(fileName)) {
			return null;
		}
		fileName = fileName.endsWith(FileConstants.ECN) ? fileName.substring(0, fileName.lastIndexOf(FileConstants.ECN)) : fileName;
		if (fileName.indexOf(".") == -1) {
			return null;
		}
		return fileId.concat(fileName.substring(fileName.lastIndexOf(".")));
	}
	
	/**
	 * @Description: 获得投标文件文件类型
	 * @author chenxw
	 * @param fileItem
	 * @return
	 */
	private static String getBidFileType(Element fileItem) {
		String type = fileItem.elementTextTrim("type");
		if (StringUtils.isNotEmpty(type)) {
			if (type.startsWith(BidFile.TYPE_BIDPRICE)) {
				type = BidFile.TYPE_BIDPRICE;
			} else if (type.equals(BidFile.TYPE_SYSTEM) && BidFile.BID_PRICE_XML.equals(getBidFileName(fileItem))) {
				type = BidFile.TYPE_BIDPRICE;
			} else if (type.startsWith(BidFile.TYPE_BILL)) {
				type = BidFile.TYPE_BILL;
			} else if (TYPE_SYSTEM_FILES.indexOf(type) >= 0) {
				type = BidFile.TYPE_SYSTEM;
			}
			return type;
		}
		return null;
	}
	
	/**
	 * @Description: 删除所有投标文件
	 * @author guoyr
	 * @param tenderId 
	 */
	public static void deleteBidFileAction(String tenderId){
		File file = new File(FileConstants.getBidFilePath(tenderId));
		if(file.exists() && file.isDirectory()){
			FileUtils.delAllFile(file.getAbsolutePath());
		}
	}
	
	public static void main(String[] arges) {
		System.out.println(new BigDecimal("222234555555566666666666763334564324643227777777777777"));
	}

}
