package com.gx.obe.common.file.entity;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.Element;

import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.web.entity.tender.TenderProjectEntity;

import common.Logger;

public class BiddingFile extends File {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String simpleName;
	private String suffix;
	private String fileId;
	private String packCode;
	
	public static final String BIDDING_FILE_INDEX_XML = "BiddingFileIndex.xml";
	public static final String BIDDING_APPENDIX_INDEX_XML = "BiddingAppendixIndex.xml";
	public static final String BIDPRICE_TABLE_INDEX = "BidPriceTableIndex.xml";//招标文件中报价一览表
	public static final String TYPE_BIDDINGFILE = "BiddingFile";// 招标文件
	public static final String TECHNOLOGY_STANDARD_INDEX_XML = "TechnologyStandardIndex.xml";// 招标文件
	
	public static final String BID_OPENING_AGENDA_DIR = "bidOpeningAgenda";
	public static String BID_KBYC = "开标议程";
	private static Logger LOG = Logger.getLogger(BiddingFile.class);
	
	public BiddingFile(String pathname) {
		super(pathname);
	}
	
	/**
	 * @Description: 获取招标文件列表
	 * @author chenxw
	 * @param tenderProjectEntity
	 * @return
	 */
	public static List<BiddingFile> getBiddingFileList(TenderProjectEntity tenderProjectEntity) {
		List<BiddingFile> biddingFileList = new ArrayList<BiddingFile>();
		BiddingFile biddingFile = BiddingFile.getBiddingFile(tenderProjectEntity.getId());
		if (null != biddingFile) biddingFileList.add(biddingFile);
		List<BiddingFile> appendixList = getBiddingFileListByIndexFileName(tenderProjectEntity.getId(), BIDDING_APPENDIX_INDEX_XML);
		if (CollectionUtils.isNotNull(appendixList)) biddingFileList.addAll(appendixList);
		List<BiddingFile> technologyStandardFileList = getTechnologyStandardFileList(tenderProjectEntity.getId());
		if (CollectionUtils.isNotNull(technologyStandardFileList)) for (BiddingFile technologyStandardFile : technologyStandardFileList)
			if (StringUtils.equals(technologyStandardFile.getPackCode(), tenderProjectEntity.getTenderNo())) biddingFileList.add(technologyStandardFile);
		return biddingFileList;
	}
	
	/**
	 * @Description: 获取技术规范文件列表
	 * @author chenxw
	 * @param tenderId
	 * @return 
	 */
	public static List<BiddingFile> getTechnologyStandardFileList(String tenderId) {
		File file = new File(FileConstants.getBiddingFilePath(tenderId).concat(TECHNOLOGY_STANDARD_INDEX_XML));
		if (file.exists() && file.isFile()) {
			Document document = XmlUtils.getXmlDocument(file.getAbsolutePath());
			if (null != document) {
				Element rootElement = document.getRootElement();
				List<?> technologyStandardInfoList = rootElement.elements("technologyStandardInfo");
				if (CollectionUtils.isNotNull(technologyStandardInfoList)) {
					List<BiddingFile> biddingFileList = new ArrayList<BiddingFile>();
					for (Object object : technologyStandardInfoList) {
						Element technologyStandardInfo = (Element) object;
						String packCode = technologyStandardInfo.elementTextTrim("packCode");
//						String packName = technologyStandardInfo.elementTextTrim("packName");
						String fileName = technologyStandardInfo.elementTextTrim("fileName");
						String actualFileName = technologyStandardInfo.elementTextTrim("actualFileName");
						File actualFile = new File(FileConstants.getBiddingFilePath(tenderId).concat(actualFileName));
						if (actualFile.exists() && actualFile.isFile()) {
							BiddingFile biddingFile = new BiddingFile(actualFile.getAbsolutePath());
							biddingFile.setSimpleName(fileName);
							biddingFile.setPackCode(packCode);
							biddingFileList.add(biddingFile);
						}
					}
					return biddingFileList;
				}
			}
		}
		return null;
	}
	
	/**
	 * @Description: 根据索引文件名称获取招标文件列表
	 * @author chenxw
	 * @param tenderId
	 * @param fileName
	 * @return
	 */
	public static List<BiddingFile> getBiddingFileListByIndexFileName(String tenderId, String fileName) {
		File file = new File(FileConstants.getBiddingFilePath(tenderId).concat(fileName));
		if (file.exists() && file.isFile()) {
			Document document = XmlUtils.getXmlDocument(file.getAbsolutePath());
			if (null != document) {
				Element rootElement = document.getRootElement();
				List<?> objectList = rootElement.elements("fileItem");
				if (CollectionUtils.isNotNull(objectList)) {
					List<BiddingFile> biddingFileList = new ArrayList<BiddingFile>();
					for (Object object : objectList) {
						Element fileItem = (Element) object;
						String fileNameWithExt = fileItem.elementTextTrim("fileNameWithExt");
						if (StringUtils.isNotEmpty(fileNameWithExt)) {
							if (fileNameWithExt.endsWith(FileConstants.XML) || fileNameWithExt.endsWith(FileConstants.LOG)) {
								continue;
							}
							BiddingFile biddingFile = new BiddingFile(FileConstants.getBiddingFilePath(tenderId).concat(fileNameWithExt));
							if (biddingFile.exists() && biddingFile.isFile()) {
								String simpleName = fileItem.elementTextTrim("name");
								if (TYPE_BIDDINGFILE.equals(simpleName)) {
									simpleName = Messages.BiddingFile;
								}
								biddingFile.setSimpleName(simpleName);
								biddingFileList.add(biddingFile);
							}
						}
					}
					return biddingFileList;
				}
			}
		}else {
			File biddingFileIndexFile = new File(FileConstants.getBiddingFilePath(tenderId).concat(BIDDING_FILE_INDEX_XML));
			if(!biddingFileIndexFile.exists()) {
				return getLocalBidFileList(tenderId);
			}
		}
		return null;
	}
	
	public static List<BiddingFile> getLocalBidFileList(String tenderId) {
		File bidFileDir = new File(FileConstants.getBiddingFilePath(tenderId));
		if(bidFileDir.exists() && bidFileDir.isDirectory()){
			List<BiddingFile> bidFileList = new ArrayList<BiddingFile>();
			List<String> fileList = new ArrayList<String>();
			FileUtils.getListFiles(bidFileDir.getAbsolutePath(), fileList, null, true);
			for(String filePath : fileList){
				File file = new File(filePath);
				if(file.exists()){
					if(file.getName().endsWith(".xml") || file.getName().equals(FileConstants.PURCHASEFILE + ".pdf") || file.getName().equals(FileConstants.PURCHASEFILE + ".docx")|| file.getName().equals(FileConstants.PURCHASEFILE + ".doc")){
						continue;
					}
					BiddingFile bidFile = new BiddingFile(file.getAbsolutePath());
					bidFile.setFileId(file.getAbsolutePath());
					bidFileList.add(bidFile);
				}
			}
			return bidFileList;
		}
		return null;
	}
	
	public static BiddingFile getBiddingFile(String tenderId) {
		BiddingFile bidddingFile = null;
		// 勿删，暂时不从索引文件中读取
		boolean readBiddingFromIndex = false;
		if (readBiddingFromIndex) {
			File biddingFileIndexFile = new File(FileConstants.getBiddingFilePath(tenderId).concat(BIDDING_FILE_INDEX_XML));
			if (null != biddingFileIndexFile && biddingFileIndexFile.exists()) {
				Document biddingFileIndexFileDocument = XmlUtils.getXmlDocument(biddingFileIndexFile.getAbsolutePath());
				if (null != biddingFileIndexFileDocument) {
					// 投标信封
					Element rootElement = biddingFileIndexFileDocument.getRootElement();
					List<?> fileItemElementList = rootElement.elements("fileItem");
					if (null != fileItemElementList && fileItemElementList.size() > 0) {
						for (Object fileNameObject : fileItemElementList) {
							Element fileNameElement = (Element) fileNameObject;
							String name = fileNameElement.elementTextTrim("name");//加密投标文件.pdf.enc
							String fileName = fileNameElement.elementTextTrim("fileName");//加密投标文件.pdf.enc
							String fileId = fileNameElement.elementTextTrim("fileId");//3186b4f38d024375b1afbb3400f56527
							String suffix = "";
							String type = "";
							Element fileTypeElement = fileNameElement.element("type");
							if (null != fileTypeElement) {
								type = fileTypeElement.getText();
							}
							if (null != type && !type.equals(TYPE_BIDDINGFILE)) {
								continue;
							}
							File file = null;
							if (fileId.indexOf(".") > 0) {
								// 如果fileId有后缀，则不再取fileName的后缀
								suffix = "";
								file = new File(biddingFileIndexFile.getParent().concat(File.separator).concat(fileId));
							} else {
								if (fileName.indexOf(".") > 0) {
									suffix = fileName.substring(fileName.lastIndexOf("."));
								}
								file = new File(biddingFileIndexFile.getParent().concat(File.separator).concat(fileId).concat(suffix));
							}
							
							if (null != file && file.exists()) {
								bidddingFile = new BiddingFile(file.getAbsolutePath());
								bidddingFile.setSimpleName(name);
								return bidddingFile;
							}
						}
					}
				}
			}
		} else {
			String biddingFilePath = FileConstants.getBiddingFilePath(tenderId);
			if (new File(biddingFilePath + FileConstants.PURCHASEFILE + FileConstants.PDF).exists()) {
				bidddingFile = new BiddingFile(biddingFilePath + FileConstants.PURCHASEFILE + FileConstants.PDF);
			} else if (new File(biddingFilePath + FileConstants.PURCHASEFILE + FileConstants.DOCX).exists()) {
				bidddingFile = new BiddingFile(biddingFilePath + FileConstants.PURCHASEFILE + FileConstants.DOCX);
			} else if (new File(biddingFilePath + FileConstants.PURCHASEFILE + FileConstants.DOC).exists()) {
				bidddingFile = new BiddingFile(biddingFilePath + FileConstants.PURCHASEFILE + FileConstants.DOC);
			}
			if (null != bidddingFile) {
				bidddingFile.setSimpleName(FileConstants.ZBWJ_DOC);
			}
		}
		return bidddingFile;
		
	}
	
	public static List<BiddingFile> getAnswerQuestionFileList(String tenderId) {
		BiddingFile bidddingFile = null;
		List<BiddingFile> fileList = null;
		File biddingFileIndexFile = new File(FileConstants.getAnswerQuestionFilePath(tenderId).concat(FileConstants.BIDDING_FILE_FOLDER).concat(File.separator).concat("AnswerQuestionIndex.xml"));
		if (null != biddingFileIndexFile && biddingFileIndexFile.exists()) {
			Document biddingFileIndexFileDocument = XmlUtils.getXmlDocument(biddingFileIndexFile.getAbsolutePath());
			if (null != biddingFileIndexFileDocument) {
				// 投标信封
				Element rootElement = biddingFileIndexFileDocument.getRootElement();
				List<?> fileItemElementList = rootElement.elements("fileItem");
				if (null != fileItemElementList && fileItemElementList.size() > 0) {
					fileList = new ArrayList<BiddingFile>();
					for (Object fileNameObject : fileItemElementList) {
						Element fileNameElement = (Element) fileNameObject;
						String name = fileNameElement.elementTextTrim("name");//加密投标文件.pdf.enc
						String fileName = fileNameElement.elementTextTrim("fileName");//加密投标文件.pdf.enc
						String fileId = fileNameElement.elementTextTrim("fileId");//3186b4f38d024375b1afbb3400f56527
						String suffix = "";
//						String type = "";
//						Element fileTypeElement = fileNameElement.element("type");
//						if(null != fileTypeElement){
//							type = fileTypeElement.getText();
//						}
						File file = null;
						if (fileId.indexOf(".") > 0) {
							// 如果fileId有后缀，则不再取fileName的后缀
							suffix = "";
							file = new File(biddingFileIndexFile.getParent().concat(File.separator).concat(fileId));
						} else {
							if (fileName.indexOf(".") > 0) {
								suffix = fileName.substring(fileName.lastIndexOf("."));
							}
							file = new File(biddingFileIndexFile.getParent().concat(File.separator).concat(fileId).concat(suffix));
						}
						if (null != file && file.exists()) {
							bidddingFile = new BiddingFile(file.getAbsolutePath());
							bidddingFile.setSimpleName(name);
							fileList.add(bidddingFile);
						}
					}
				}
			}
		}
		return fileList;
	}
	
	/**
	 * @Description: 获得开标议程
	 * @author guoyr
	 * @param tenderId
	 * @return
	 */
	public static File getBidOpeningPlanFile(String tenderId) {
		File biddingFileDir = new File(FileConstants.getBiddingFilePath(tenderId));
		String[] openPlanFileType = new String[] { ".doc", ".docx", ".pdf" };
		File openPlanFile = null;
		if (null != biddingFileDir && biddingFileDir.exists()) {
			for (String fileEndWidth : openPlanFileType) {
				openPlanFile = new File(biddingFileDir.getAbsolutePath().concat(File.separator).concat(BiddingFile.BID_KBYC).concat(fileEndWidth));
				if (null != openPlanFile && openPlanFile.exists()) {
					return openPlanFile;
				}
			}
		}
		return null;
	}
	
	/**
	 * @Description: 根据类型获得招标文件附件
	 * @author chenxw
	 * @param tenderId
	 * @param fileType
	 * @return
	 */
	public static BiddingFile getBiddingFileBill(String tenderId) {
		File biddingAppendixIndexFile = new File(FileConstants.getBiddingFilePath(tenderId).concat(FileConstants.BIDDING_APPENDIX_INDEX_XML));
		if (biddingAppendixIndexFile.exists()) {
			Document biddingAppendixIndexFileDocument = XmlUtils.getXmlDocument(biddingAppendixIndexFile.getAbsolutePath());
			if (null != biddingAppendixIndexFileDocument) {
				String biddingFileDir = biddingAppendixIndexFile.getParent();
				// 投标信封
				Element rootElement = biddingAppendixIndexFileDocument.getRootElement();
				List<?> fileItemElementList = rootElement.elements("fileItem");
				if (null != fileItemElementList && fileItemElementList.size() > 0) {
					for (Object fileNameObject : fileItemElementList) {
						Element fileNameElement = (Element) fileNameObject;
						String fileId = fileNameElement.elementTextTrim("fileId");//3186b4f38d024375b1afbb3400f56527
						if (null == fileId) {
							continue;
						}
						String type = "";
						Element fileTypeElement = fileNameElement.element("type");
						if (null != fileTypeElement) {
							type = fileTypeElement.getText();
						}
						if (!type.startsWith("Bill")) {
							continue;
						}
						String fileName = fileNameElement.elementTextTrim("fileName");//加密投标文件.pdf.enc
						String suffix = "";
						if (fileName.endsWith(FileConstants.XML)) {
							if (fileId.indexOf(".") > 0) {
								// 如果fileId有后缀，则不再取fileName的后缀
								suffix = "";
							} else {
								if (fileName.indexOf(".") > 0) {
									suffix = fileName.substring(fileName.lastIndexOf("."));//加密文件的后缀（为了避免文件中存在多个.）
								}
							}
							File decryptFile = new File(biddingFileDir.concat(File.separator).concat(fileId).concat(suffix));// 解密后的文件3186b4f38d024375b1afbb3400f56527.pdf
							if (decryptFile.exists()) {
								return new BiddingFile(decryptFile.getAbsolutePath());
							}
						}
					}
				}
			}
		}
		return null;
	}
	
	/**
	 * @Description: 删除所有招标文件
	 * @author guoyr
	 * @param tenderId 
	 */
	public static void deleteBidddingFileAction(String tenderId){
		File file = new File(FileConstants.getBiddingFilePath(tenderId));
		if(file.exists() && file.isDirectory()){
			FileUtils.delAllFile(file.getAbsolutePath());
		}
		File answerQuestionFile = new File(FileConstants.getAnswerQuestionFilePath(tenderId));
		if(answerQuestionFile.exists() && answerQuestionFile.isDirectory()){
			FileUtils.delAllFile(answerQuestionFile.getAbsolutePath());
		}
	}
	
	public String getSimpleName() {
		return null != simpleName ? simpleName : getName();
	}
	
	public void setSimpleName(String simpleName) {
		this.simpleName = simpleName;
	}
	
	public String getSuffix() {
		return suffix;
	}
	
	public void setSuffix(String suffix) {
		this.suffix = suffix;
	}
	
	public String getFileId() {
		return fileId;
	}
	
	public void setFileId(String fileId) {
		this.fileId = fileId;
	}
	
	public void setPackCode(String packCode) {
		this.packCode = packCode;
	}
	
	public String getPackCode() {
		return packCode;
	}
	
}
