package com.gx.obe.common.file.enumeration;

import java.io.File;

import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import com.gx.obe.components.core.Constants;
import com.gx.obe.components.core.CoreActivator;
import com.gx.obe.config.utils.ProjectUtils;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;

public class FileConstants {

	/**
	 * 系统的当前路径
	 **/
	public static String PRODUCT_ROOT = getPluginPath(CoreActivator.getDefault());
	public static String getPluginPath(AbstractUIPlugin plugin){
		String root = ProjectUtils.toFullPath(plugin, "/");
		// 如果前面有/，读取文件时找不到
		if(root.charAt(0)=='/'){
			root = root.substring(1);
		}
		return root;
	}
	
	/**
	 * 系统用户所在目录
	 */
	public static final String SYS_USER_ROOT = System.getProperty("user.home").concat(File.separator).concat(Constants.PRODUCE_EN_NAME).concat(File.separator);
	
	/**
	 * 数据集存放目录
	 */
	public static final String DATASET_PATH = SYS_USER_ROOT.concat("dataset").concat(File.separator);

	/**
	 * 自定义模板目录
	 */
	public static final String CUSTOM_TEMPLATE_PATH = SYS_USER_ROOT.concat("template").concat(File.separator);

	public static final String SYSTEM_TEMPLATE_PATH = Platform.getInstallLocation().getURL().getPath().concat(File.separator).concat("template").concat(File.separator);
	
	/**
	 * 开标议程路径
	 */
	public static String BID_OPENING_AGENDA_PATH = PRODUCT_ROOT + File.separator + "bid_opening_agenda";
	
	/**
	 * @Description: 缓存目录
	 */
	public static String CACHE_PATH = getInstallPath() + "cache" + File.separator;
	
	public static String SOFTWARE_CACHE_PATH = CACHE_PATH.concat("software").concat(File.separator);
	
	/**
	 * 招标文件目录
	 **/
	public static String BIDDING_FILE_FOLDER = "biddingFile";
	/**
	 * 答疑文件的目录
	 **/
	public static String ANSWER_QUESTION_FOLDER = "answerQuestion";
	/**
	 * 再次报价的目录
	 **/
	public static String REBIDPRICE_FOLDER = "reBidPrice";
	
	/**
	 * 投标文件目录
	 **/
	public static String BID_FILE_FOLDER = "bidFile";
	/**
	 * 供应商质疑目录
	 **/
	private static String SUPPLIER_QUESTION_FOLDER = "bidFileQuestion";
	/**
	 * 政策法规目录
	 **/
	public static String POLICIES_AND_LAWS_FOLDER = "policiesAndLaws";
	
	private static String SIGN_FILE_FOLDER = "signFile";
	
	/**
	 * 临时目录
	 */
	public static String TEMP_FOLDER = System.getProperty("java.io.tmpdir").concat("OBE_TEMP").concat(File.separator);
	
	
	/**
	 * 视频录制目录
	 **/
	public static String BIDDING_VIDEO_BASE_PATH = "video";
	
	/**
	 * 标书文件路径
	 **/
	public static String BID_FILE_PATH = PropertiesUtils.getProperty(com.gx.obe.config.ConfigConstants.BID_FILE_PATH, PRODUCT_ROOT).concat(File.separator);
	/**
	 * 视频录制文件路径
	 **/
	public static String VIDEO_FILE_PATH = PropertiesUtils.getProperty(com.gx.obe.config.ConfigConstants.VIDEO_FILE_PATH, PRODUCT_ROOT).concat(File.separator);
	
	/**
	 * 区块链文件的目录
	 **/
	public static String QKL_FOLDER = "qklFiles";
	
	/**
	 * @Description: 招标文件路径
	 * @author guoyr
	 * 
	 * @param tenderId
	 * @return 
	 */
	public static String getBiddingFilePath(String tenderId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(BIDDING_FILE_FOLDER).concat(File.separator);
	}
	
	/**
	 * @Description: 澄清答疑文件路径
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	public static String getAnswerQuestionFilePath(String tenderId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(ANSWER_QUESTION_FOLDER).concat(File.separator);
	}
	
	/**
	 * @Description: 再次报价文件路径
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	public static String getReBidPriceFilePath(String tenderId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(REBIDPRICE_FOLDER).concat(File.separator);
	}
	/**
	 * @Description: 项目现场直播视频录制存放路径
	 * @author guoyr
	 * @param tenderId
	 * @return 
	 */
	public static String getVideoFilePath(String tenderId){
		return VIDEO_FILE_PATH.concat(tenderId).concat(File.separator).concat(BIDDING_VIDEO_BASE_PATH);
	}
	
	/**
	 * @Description: 获得项目的投标文件路径
	 * @author guoyr
	 * @param tenderId

	 */
	public static String getBidFilePath(String tenderId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(BID_FILE_FOLDER).concat(File.separator);
	}
	/**
	 * @Description: 获得投标人的投标文件路径
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	public static String getBidFilePath(String tenderId, String supplierId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(BID_FILE_FOLDER).concat(File.separator).concat(supplierId).concat(File.separator);
	}
	
	/**
	 * @Description: 获得投标人的质疑文件路径
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	public static String getBidFileQuestionPath(String tenderId, String supplierId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(SUPPLIER_QUESTION_FOLDER).concat(File.separator).concat(supplierId).concat(File.separator);
	}

	/**
	 * 投标文件路径临时目录
	 **/
	public static String TEMP_PATH = BID_FILE_PATH+"file_temp".concat(File.separator);
	
	/**
	 * 模板存放路径
	 **/
	public static String TEMPLATE_PATH = PropertiesUtils.getProperty(com.gx.obe.config.ConfigConstants.TEMPLATE_PATH, PRODUCT_ROOT).concat(File.separator) + "template".concat(File.separator);
	
	public static String getReportFilePath(String tenderId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(REPORT_FILE_FOLDER).concat(File.separator);
	}
	public static String getOpeningReportFilePath(String tenderId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(REPORT_FILE_FOLDER).concat(File.separator).concat("opening").concat(File.separator);
	}
	
	public static String getUploadBiddingFilePath(String tenderId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(REPORT_FILE_FOLDER).concat(File.separator).concat(UPLOAD_BIDDING_FILE);
	}
	
	/**
	 * 招标文件
	 **/
	public static String BIDDING_FILE = "biddingFile";
	public static String UPLOAD_BIDDING_FILE = "uploadBiddingFile";

	/**
	 * 投标文件
	 **/
	public static String BID_FILE = "bidFile";
	
	public static String REPORT_FILE_FOLDER = "reportFile";
	
	public static String REPORT_CODE_INQUIRE = "inquireFile";
	
	public static String TEMPLATE_FILE_FOLDER = "templateFile";
	
	public static String VIDEO_FILE_FOLDER = "videoFile";
	
	// 评审报告的模板名称
	public static final String REVIEW_REPORT_NAME = "reviewReport.ftl";
	
	/**
	 * 开标一览表模板
	 **/
	public static final String BID_OPENING_TABLE_TEMPLATE = "PRICE.xml";
	
	/**
	 * 谈判内容模板
	 **/
	public static final String NEGOTIATION_TEMPLATE = "NEGOTIATION.XML";
	
	/**
	 * 评审纪律(用于专家签到界面使用)
	 */
	public static final String EVAL_RECORD_FILE = "EVAL_RECORD.txt";
	
	/**
	 * 资源目录
	 **/
	public static String RESOURCE_FOLDER = "resource";
	
	public static File getResourceFile(String fileName){
		File resourceFile = new File(Platform.getInstallLocation().getURL().getPath()+File.separator + FileConstants.RESOURCE_FOLDER + File.separator + fileName);
		if(null != resourceFile && resourceFile.exists()){
			return resourceFile;
		}
		resourceFile = new File(FileConstants.PRODUCT_ROOT + File.separator + FileConstants.RESOURCE_FOLDER + File.separator + fileName);
		return resourceFile;
	}
	
	/**
	 * 系统安装目录
	 **/
	public static String getInstallPath(){
		File resourceFile = new File(Platform.getInstallLocation().getURL().getPath()+File.separator);
		if(null != resourceFile && resourceFile.exists()){
			return resourceFile.getAbsolutePath().concat(File.separator);
		}
		return FileConstants.PRODUCT_ROOT + File.separator;
	}
	
	/**
	 * 系统默认模板
	 **/
	public static String DEFAULT_TEMPLATE_FOLDER = "template";
	
	/**
	 * 系统评标步骤默认模板
	 **/
	public static String DEFAULT_EVALUATION_STEP_FOLDER = "evaluationStep";
	/**
	 * 系统开标一览表默认模板
	 **/
	public static String DEFAULT_BID_PRICE_TABLE_FOLDER = "bidPriceTable";
	
	/**
	 * 系统废标条款默认模板
	 **/
	public static String DEFAULT_WASTE_CLAUSE_FOLDER = "wasteClause";
	
	/**
	 * license公钥
	 **/
	public static final String LICENSE_PUBKEY_PATH = PRODUCT_ROOT + "pubkey.dat";
	/**
	 * 评标纪律目录
	 **/
	public static String EVALUATION_LAWS_FOLDER = "evaluationLaws";
	/**
	 * 评审纪律文件
	 **/
	public static final String REVIEW_DISCIPLINE_FILE = "REVIEW_DISCIPLINE";
	
	/**
	 * 评审手册
	 **/
	public static final String EVALUATION_HELP_FILE = "EVALUATION_HELP";

	/**
	 * 投标信封 signType
	 */
	public static String SIGNTYPE = "DianJu";
	public static String ETB = ".etb";// 电子投标文件
	public static String EZG = ".etb";//".ezg";// 电子资格投标文件
	public static String EZB = ".ezb";// 电子招标文件
	public static String ECN = ".enc";
	public static String XLS = ".xls";
	public static String PDF = ".pdf";
	public static String DOC = ".doc";
	public static String DOCX = ".docx";
	public static String XML = ".xml";
	public static String LOG = ".log";
	public static String XLSX = ".xlsx";
	
	public static String EVAL_DISCIPLINE = "评标纪律-承诺书";
	public static String KBYLB_DOC = "开标一览表";
	public static String TBWJ_DOC = "投标文件";
	public static String ZBWJ_DOC = Messages.BiddingFile;
	public static String PURCHASEFILE = "BiddingFile";
	public static String BID_PRICE_XML = "BidPrice.xml";
	public static String SUPPLIER_SIGN_PDF = "Sign.pdf";
	public static String PSZB_XML = "EvaluationFactors.xml";
	public static String PSD_XML = "RespItem.xml";
	public static String BID_FILE_INDEX_XML = "BidFileIndex.xml";
	public static String PROJECT_DATA_XML = "DataItems.xml";
	public static String TENDER_ENVELOP_XML = "BidEnvelop.xml";//投标信封
	public static String DECENVELOP_XML = "DecEnvelop.xml";//解密信封
	public static String TABLE_ANALYSIS_XML = "TableAnalysis.xml";//投标人比选文件
	public static String BUSINESS_TENDER_XML = "BusinessTender.xml";// 建筑工程清单
	public static String FEATURE_IDENTIFY = "Identification.xml";// 特征码检查文件
	public static String TENDER_APPENDIX = "TenderAppendix.xml";// 附件描述文件
	public static String BIDDING_APPENDIX_INDEX = "BiddingAppendixIndex";// 招标文件附件描述文件
	public static String BIDDING_APPENDIX_INDEX_XML = "BiddingAppendixIndex.xml";// 招标文件附件描述文件
	public static String BILL_XML = "bill.xml";// 清单文件
	public static String BILLZB_XML = "billzb.xml";// 清单文件
	
	public static String ANSWER_FILE = "询标函（澄清）.docx";
	public static String ANSWER_FILE_PDF = "询标函（澄清）.pdf";
	
	public static String[] SUPPLIER_BID_FILES = {BID_PRICE_XML,PSZB_XML,BID_FILE_INDEX_XML};// 评审点文件不是必须的
	public static String getBidFileName(String bidName){
		String bidFileName = "";
		if(null == bidName || "".equals(bidName)){
			bidFileName = "";
		}else if(bidName.equals(BID_PRICE_XML)){
			bidFileName = "报价文件";
		}else if(bidName.equals(PSZB_XML)){
			bidFileName = "评审指标文件";
		}else if(bidName.equals(PSD_XML)){
			bidFileName = "评审点文件";
		}else if(bidName.equals(BID_FILE_INDEX_XML)){
			bidFileName = "投标分册";
		}
		return bidFileName;
	}
	
	/**
	 * 评标须知
	 */
	public static final String EVAL_TO_NOTE = "expertToNote.pdf";	
	public static final String EXPERT_COMMITMENT = "EXPERT_COMMITMENT.docx";	
	
	
	
	/**
	 * @Description: 获得投标人的投标文件路径
	 * @author guoyr
	 * @param tenderId
	 * @param supplierId
	 * @return 
	 */
	public static String getSupplierSignFilePath(String tenderId, String supplierName){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(SIGN_FILE_FOLDER).concat(File.separator).concat(supplierName).concat(".pdf");
	}

	
	/**
	 * 获取utorrent程序路径
	 * @return
	 */
	public static String getUtorrentRoot(){
		String fileroot = FileConstants.PRODUCT_ROOT + "uTorrent/" + "uTorrent.exe";
		return fileroot;
	}
	
	/**
	 * 好视通视频文件夹
	 **/
	public static String DEFAULT_HST_FILE_FOLDER = "FMClient";
	
	/**
	 * @Description: 区块链附件文件路径
	 * @author wangxiang
	 * @param tenderId
	 * @return 
	 */
	public static String getQKLFilePath(String tenderId){
		return BID_FILE_PATH.concat(tenderId).concat(File.separator).concat(QKL_FOLDER).concat(File.separator);
	}
	
}

