package com.gx.obe.common.file.http;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;

import org.apache.log4j.Logger;
import org.eclipse.swt.core.CallBack;

import com.gx.obe.business.http.MyX509TrustManager;
import com.gx.obe.common.file.entity.DownBlock;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.util.utils.MD5Utils;
import com.gx.obe.util.utils.StringUtils;

public class HttpURLDown {

	private static final DecimalFormat fmt = new DecimalFormat("###,##0.00");  
	private DownBlock block;
	private CallBack callBack;
	private Logger LOG = Logger.getLogger(HttpURLDown.class);
	public HttpURLDown(String downURL, String savePath) {
		block = new DownBlock(downURL, savePath);
	}
	public HttpURLDown(String downURL, String savePath, CallBack callBack) {
		this.callBack = callBack;
		block = new DownBlock(downURL, savePath);
	}
	
	/** 
	 * @Description: 下载 
	 * @author guoyr
	 * @param   messageReceiver 消息机制  
	 * @return  
	 * @Exception   
	 */
	public boolean  downLoad() throws IOException {
		return downLoad(null);
	}
	public boolean downLoad(String auth) throws IOException {
		
		File file = new File(block.getSaveFile());
		// 如果文件已经存在
		if (file.isFile() && file.exists()) {
			file.delete();
//			return;
		}else if(!file.getParentFile().exists()){
			file.getParentFile().mkdirs();
		}

		// 任务描述文件
		File taskFile = new File(block.getSaveFile() + ".r_task");

		// 真正下载的文件
		File saveFile = new File(block.getSaveFile() + ".r_save");
		boolean taskFileExist = taskFile.exists();
		
		// 如果不存在断点续传的文件，则删除当前目录
//		if(!taskFileExist){
//			FileUtils.delAllFile(saveFile.getParent());
//		}
		boolean success = true;
		RandomAccessFile taskRandomFile = null;
		RandomAccessFile downRandomFile = null;
		try {
			if(null != auth){
				callBackMsg("正在获取"+auth+"下载资源...");
			}else {
				callBackMsg("正在获取下载资源...");
			}
			long rtnLen = getContentLength(block.getDownURL());
			if(rtnLen > 0){
				taskRandomFile = new RandomAccessFile(taskFile, "rw");
				downRandomFile = new RandomAccessFile(saveFile, "rw");
				if(null != auth){
					callBackMsg(Messages.Tips_downloading+auth+"资源（" + FileUtils.getFileSize(rtnLen)+"）");
				}else {
					callBackMsg("正在下载资源（" + FileUtils.getFileSize(rtnLen)+"）");
				}
				block.setContentLength(rtnLen);
				if (!taskFileExist) {
					// 如果文件不存在，就初始化任务文件和下载文件
					initTaskFile(taskRandomFile, block);
					downRandomFile.setLength(rtnLen);
				} else {
					// 任务描述文件存在就读取任务文件
					block.read(taskRandomFile);
					if (block.getContentLength() != rtnLen) {
						LOG.error("======下载出错=====");
					}else {
						taskFile.delete();
						initTaskFile(taskRandomFile, block);
						downRandomFile.setLength(rtnLen);
					}
				}
				block.setDownLoadTime(System.currentTimeMillis());
				List<Integer> downBlockList = new ArrayList<Integer>();
				for(int i = 0; i < block.getThreadCount(); i++){
					downBlockList.add(i);
				}
				
				List<Integer> errorDownBlockList = downFileByBlock(taskRandomFile, downRandomFile, block, downBlockList);
				// 如果有下载出错的分块，则重新下载一次
				if(null != errorDownBlockList && errorDownBlockList.size() > 0){
					errorDownBlockList = downFileByBlock(taskRandomFile, downRandomFile, block, errorDownBlockList);
					if(null != errorDownBlockList && errorDownBlockList.size() > 0){
						success = false;
					}
				}
			}else {
				success = false;
				if(null != auth){
					callBackMsg(auth+"下载资源获取失败...");
				}else {
					callBackMsg("下载资源获取失败...");
				}
				if(taskFile.exists()){
					taskFile.delete();
				}
			}
		
		} catch (Exception e) {
			success = false;
		} finally {
			if (null != taskRandomFile) {
				taskRandomFile.close();
				taskRandomFile = null;
			}
			if (null != downRandomFile) {
				downRandomFile.close();
				downRandomFile = null;
			}
			
			// 如果下载成功，去掉任务描述文件、帮下载文件改名
			if (success) {
				if(taskFile.exists()){
					
					taskFile.delete();
				}
				FileUtils.rename(saveFile.getAbsolutePath(), file.getAbsolutePath());
				if(!saveFile.delete()){
					saveFile.deleteOnExit();
				}
//				block.setDownLoadSpeed(downloadSpeed(block.getContentLength(),System.currentTimeMillis()-block.getDownLoadTime()));
//				message.setSize(block.getProgress());
//				messageReceiver.receive(message);
				// 由于当文件较大时获得MD5文件的大小较慢，所以不用MD5验证，改用总字节大小验证
				if(valDownloadFileSize(file)){
//				if(valDownloadFileMD5(file)){
					if(null != auth){
						callBackMsg(true, auth+"的资源下载成功，用时:"+DateUtils.timeSpan(System.currentTimeMillis()-block.getDownLoadTime()));
					}else {
						callBackMsg(true, "下载成功，"+Messages.Tips_timeUsed+":"+DateUtils.timeSpan(System.currentTimeMillis()-block.getDownLoadTime()));
					}
//					receiveMessage("平均下载速度"+block.getDownLoadSpeed());
				}else {
					success = false;
					callBackMsg(false, "下载文件验证失败！");
				}
			}
			if(saveFile.exists()){
				saveFile.delete();
			}
		}
		return success;
	}

	private boolean valDownloadFileSize(File file){
		long downloadFileSize = file.length();
		// "答疑文件"不进行MD5验证，只有招标文件验证。
		if(downloadFileSize > 0 && downloadFileSize == block.getContentLength()){
			LOG.debug("下载成功:要下载文件的大小为"+block.getContentLength()+"，下载后的文件大小为："+downloadFileSize);
			return true;
		}else {
			LOG.error("下载失败:要下载文件的大小为"+block.getContentLength()+"，下载后的文件大小为："+downloadFileSize);
			return false;
		}
	}
	private boolean valDownloadFileMD5(File file){
		String downMd5 = "";
		try {
			downMd5 = MD5Utils.getFileMD5String(file);
		} catch (IOException e) {
		}
		// "答疑文件"不进行MD5验证，只有招标文件验证。
		if(StringUtils.isEmpty(block.getMd5()) || (null != block.getMd5() && block.getMd5().equals(downMd5))){
			LOG.debug("下载成功:整个下载文件的MD5"+block.getMd5()+"接收后MD5"+downMd5);
//			receiveMessage("平均下载速度"+block.getDownLoadSpeed());
			return true;
		}else {
			LOG.error("下载文件验证失败:整个下载文件的MD5"+block.getMd5()+"接收后MD5"+downMd5);
			return false;
		}
	}
	// 往消息机制里面注册消息
	private void receiveMessage(String msgStr){
//		message.setMessage(msgStr);
//		message.setSize(0);
//		messageReceiver.receive(message);
	}
	
	/**
	 * @Description: 按分块下载文件
	 * @author guoyr
	 * @param taskRandomFile
	 * @param downRandomFile
	 * @param block
	 * @param blockNoList
	 * @return 
	 */
	private List<Integer> downFileByBlock(final RandomAccessFile taskRandomFile, final RandomAccessFile downRandomFile, final DownBlock block, List<Integer> blockNoList){
		List<Integer> errorDownBlockList = null;
		List<Future<Integer>> resultList = new ArrayList<Future<Integer>>();  
		// 分配线程去下载，这里用到线程池
		ExecutorService executorService = Executors.newFixedThreadPool(blockNoList.size());
		for(final int blockNo : blockNoList){
			LOG.debug("下载第"+blockNo+"块分块文件...");
			
			Future<Integer> future = executorService.submit(new Callable<Integer>() {
				public Integer call() {
					boolean downFlag = false;
					try {
						downFlag = down(taskRandomFile, downRandomFile, block, blockNo);
					} catch (IOException e1) {
						LOG.error(e1);
					}
					// 下载失败时返回下载失败的分块
					return downFlag ? -1 : blockNo;
				}
			});
			resultList.add(future);
		}
		executorService.shutdown();
		
		for (Future<Integer> fs : resultList) {  
			try {  
				int errorBlockNo = fs.get();
				if(errorBlockNo > -1){
					if(null == errorDownBlockList){
						errorDownBlockList = new ArrayList<Integer>();
					}
					errorDownBlockList.add(errorBlockNo);
				}
			} catch (InterruptedException e) {  
			} catch (ExecutionException e) {  
			}  
		}  
//		try {
//			es.awaitTermination(24 * 3600, TimeUnit.SECONDS);
//		} catch (InterruptedException e) {
//			LOG.error("==========downErrorBlock===:", e);
//			receiveMessage("下载超时....");
//			success.set(false);
//		}
		// 用了fs.get()返回执行结果，就不用判断是否完成线程池了
//		try {
//			while (!es.awaitTermination(1, TimeUnit.SECONDS)) {   
//			    System.out.println("线程池没有关闭");   
//			}
//		} catch (InterruptedException e) {
//		} 
		return errorDownBlockList;
	}

	/** 
	 * @Description: 调用HttpURLConnection下载 
	 * @author guoyr
	 * @param   
	 * @return  
	 * @Exception   
	 */
	private synchronized boolean down(RandomAccessFile taskRandomFile, RandomAccessFile downRandomFile, DownBlock task, int sectionNo) throws IOException {
		boolean downFlag = true;
		long start = task.getSectionsOffset()[sectionNo];
		long end = -1;

		// 这里是计算当前块的长度
		if (sectionNo < task.getBlockCount() - 1) {
			long per = task.getContentLength() / task.getBlockCount();
			end = per * (sectionNo + 1);
		} else {
			end = task.getContentLength();
		}

		if (start >= end) {
			LOG.debug("已经在第 " + sectionNo+"分块下载完...");
			return true;

		}
		HttpURLConnection httpURLConnection = null;
		try {
			SSLContext sslcontext= SSLContext.getInstance("SSL","SunJSSE");
			sslcontext.init(null ,  new TrustManager[]{new MyX509TrustManager()}, new java.security.SecureRandom());
//		URL url = new URL(urlPost);
			URL u = new URL(task.getDownURL());
			HostnameVerifier ignoreHostnameVerifier = new HostnameVerifier() {
				public boolean verify(String s, SSLSession sslsession) {
					System.out.println("WARNING: Hostname is not matched for cert.");
					return true;
				}
			};
			HttpsURLConnection.setDefaultSSLSocketFactory(sslcontext.getSocketFactory());
			HttpsURLConnection.setDefaultHostnameVerifier(ignoreHostnameVerifier);
			
			httpURLConnection = (HttpURLConnection) u.openConnection();
			httpURLConnection.setRequestMethod("POST");
			String range = "bytes=" + start + "-" + (end - 0);// 这里是end-1
			httpURLConnection.setRequestProperty("Range", range);
			httpURLConnection.setRequestProperty("Content-Type", "application/octet-stream");
			httpURLConnection.setRequestProperty("Signature", "UE");
//        httpURLConnection.setRequestProperty("FileSize", String.valueOf(task.getContentLength()-1));
			
			httpURLConnection.setConnectTimeout(30000); 
			httpURLConnection.setReadTimeout(30000); 
			long totalLength = 0;
			httpURLConnection.connect();
			if (!(httpURLConnection.getResponseCode() == 206 || httpURLConnection.getResponseCode() == 200)) {
				LOG.error("请求URL异常 httpURLConnection.getResponseCode()="+httpURLConnection.getResponseCode());
//				throw new RuntimeException();
			}

			long contentLength = httpURLConnection.getContentLength();
			if(contentLength <= 0){
				contentLength = Long.parseLong(httpURLConnection.getHeaderField("Content-Length1"));
			}
			if (contentLength != (end - start)) {
				LOG.error("======下载出错=====httpURLConnection.getContentLength()"+contentLength+"=="+(end +"-"+ start));
//				throw new RuntimeException();
			}
			InputStream is = httpURLConnection.getInputStream();
			byte[] temp = new byte[task.getBufferSize()];
			BufferedInputStream bis = new BufferedInputStream(is, temp.length);
			
			int readed = 0;
			int count = 0;
			while ((readed = bis.read(temp)) != -1) {
				synchronized (task) {
					count ++;
					totalLength += (readed);
					long offset = task.getSectionsOffset()[sectionNo];
					
					// 下载之后顺便更新描述文件，这里效率比较低，在一个线程下载里进行两次文件操作，可以实现一个缓冲写。
					downRandomFile.seek(offset);
					downRandomFile.write(temp, 0, readed);
					offset += readed;
					task.getSectionsOffset()[sectionNo] = offset;
					task.writeOffset(taskRandomFile);
					task.setDownLoadLength(task.getDownLoadLength()+readed);
//					if(count%10 == 0){
//						task.setDownLoadSpeed(downloadSpeed(task.getBlockCount()*totalLength,System.currentTimeMillis()-task.getDownLoadTime()));
//						message.setSize(task.getProgress());
//						message.setMessage(Messages.Tips_downloaded+task.getProgress()+"%下载速度"+task.getDownLoadSpeed());
//						messageReceiver.receive(message);
//						LOG.debug("已下载"+task.getProgress()+"%下载速度"+task.getDownLoadSpeed());
//					}
				}
			}
			
//			RandomAccessFile readRaf = new RandomAccessFile(fileAbsolutePath, "r");
	    	
	    	byte[] readByte = new byte[Integer.valueOf((end - start)+"")];
	    	synchronized (downRandomFile){
	    		downRandomFile.seek(start);
	    		downRandomFile.read(readByte);
	    	}
	    	
	    	/**这里的验证有问题,暂时先不验证**/
//			MappedByteBuffer byteBuffer = downRandomFile.getChannel().map(FileChannel.MapMode.READ_ONLY, start, end - start);
//			String receiveMD5 = MD5Utils.getFileMD5String(byteBuffer);
//	    	String receiveMD5 = MD5Util.getMD5String(readByte);
	    	
//			if(null != httpURLConnection.getHeaderField("MD5") && httpURLConnection.getHeaderField("MD5").equals(receiveMD5)){
//				receiveMessage("接收到的分块文件验证正确！");
//			}else {
//				receiveMessage("第" + sectionNo + "块文件分块下载异常!");
//				LOG.debug("第" + sectionNo + "块文件分块下载异常!");
//				downFlag = false;
//			}
//			LOG.debug("接收的MD5"+httpURLConnection.getHeaderField("MD5")+"接收后MD5"+receiveMD5);
			
		} catch (Exception e) {
			LOG.error("", e);
			downFlag = false;
		}finally {
			if(null != httpURLConnection){
				httpURLConnection.disconnect();
			}
		}
//		LOG.debug("第" + sectionNo + "块文件分块下载完成"+downFlag);
		return downFlag;

	}

	/**
	 * @author guoyr
	 * @param
	 * @return
	 * @Exception
	 */
	public void initTaskFile(RandomAccessFile taskRandomFile, DownBlock task)
			throws IOException {
		int blockCount = task.getBlockCount();
		long per = task.getContentLength() / blockCount;
		long[] sectionsOffset = new long[blockCount];
		for (int i = 0; i < blockCount; i++) {
			sectionsOffset[i] = per * i;
		}
		task.setSectionsOffset(sectionsOffset);
		task.create(taskRandomFile);
	}

	/**
	 * @author guoyr
	 * @param
	 * @return
	 * @Exception
	 */
	public long getContentLength(String url) {
		URL u = null;
		HttpURLConnection httpURLConnection = null;
		try {
			u = new URL(url);
			httpURLConnection = (HttpURLConnection) u.openConnection();
			httpURLConnection.setRequestMethod("POST");
			httpURLConnection.setRequestProperty("Content-Type", "application/octet-stream");
			httpURLConnection.setRequestProperty("Signature", "UE");
			// httpURLConnection.setRequestProperty("Range", "bytes=0-0");//
			httpURLConnection.setRequestProperty("FileSize", "0");

			if (!(httpURLConnection.getResponseCode() == 206 || httpURLConnection.getResponseCode() == 200)) {
				LOG.error("请求URL异常==========httpURLConnection.getResponseCode()==========="+httpURLConnection.getResponseCode());
//			throw new RuntimeException();
			}
			//{X-Frame-Options=[SAMEORIGIN], Transfer-Encoding=[chunked], null=[HTTP/1.1 200 OK], Server=[nginx], Connection=[keep-alive], Set-Cookie=[], Date=[Sat, 08 Feb 2020 15:08:13 GMT], Content-Type=[text/html;charset=UTF-8]}
			long contentLength = httpURLConnection.getContentLength();
			if(contentLength <= 0){
				try {
					contentLength = Long.parseLong(httpURLConnection.getHeaderField("Content-Length1"));
				} catch (NumberFormatException e) {
				}
			}
			LOG.debug("要下载的文件长度为"+contentLength);
			try {
				httpURLConnection.setConnectTimeout(100000); 
				httpURLConnection.setReadTimeout(100000);
			} catch (Exception e) {
				LOG.error("", e);
				receiveMessage("连接服务器超时....");
				return -1;
			} 
			block.setMd5(httpURLConnection.getHeaderField("MD5"));
			return contentLength;
		} catch (MalformedURLException e) {
			LOG.error("", e);
			return -1;
		} catch (IOException e) {
			LOG.error("", e);
			return -1;
		}finally{
			if(null != httpURLConnection)
				httpURLConnection.disconnect();
		}

	}
	
	/** 
	 * @Description: 计算下载速度 
	 * @author guoyr
	 * @param  length 下载文件的长度 
	 * @param  time 下载用时（毫秒）
	 * @return  
	 * @Exception   
	 */
	public static String downloadSpeed(double length, long time){
		double speed = length*1024*1024/(double)(time==0?1:time*1000);
		String unit = "KB";
		if(speed < 0)speed = 0;
		if(speed > (1024*1024*1024)){
			unit = "GB";
			speed = (double)speed/(1024*1024*1024);
		}else if(speed > (1024*1024)){
			unit = "MB";
			speed = (double)speed/(1024*1024);
		}else if(speed > (1024)){
			unit = "KB";
			speed = (double)speed/(1024);
		}else {
			unit = "字节";
		}
    	return fmt.format(speed)+unit+"/s";
	}
	
	public DownBlock getBlock() {
		return block;
	}

	public void setBlock(DownBlock block) {
		this.block = block;
	}

	private void callBackMsg(String msg){
		if(null != callBack){
			callBack.callBackString("  "+msg);
		}
	}
	private void callBackMsg(boolean bool, String msg){
		if(null != callBack){
			callBack.callBackString((bool?"√":"×")+ msg);
		}
	}
}
