package com.gx.obe.common.file.http;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;

import org.apache.log4j.Logger;

import com.gx.obe.business.http.MyX509TrustManager;
import com.gx.obe.common.file.download.infer.FileDownloadListener;
import com.gx.obe.common.file.download.infer.FileDownloadListener.DOWN_STATUS;
import com.gx.obe.common.file.entity.DownBlock;
import com.gx.obe.util.utils.FileUtils;
import com.gx.utils.ActionLogUtils;

public abstract class HttpURLDownload {

	private DownBlock block;
	private Logger LOG = Logger.getLogger(HttpURLDownload.class);
	private FileDownloadListener listener;
	private ExecutorService executorService;
	private boolean isDeleteSourceFile = false;
	private List<File> deleteSourceFileList = new ArrayList<File>();
	private String messageStartName;
	private long tempFileSize;
	private String downUrl;

	public HttpURLDownload(String downURL, long fileSize, String savePath, FileDownloadListener listener) {
		this.listener = listener;
		this.tempFileSize = fileSize;
		block = new DownBlock(downURL, savePath);
		this.downUrl = downURL;
	}

	/**
	 * @Description: 下载
	 * @author guoyr
	 * @return
	 * @Exception
	 */
	public boolean downLoad() {
		File file = new File(block.getSaveFile());
		// 如果文件已经存在
		if (file.isFile() && file.exists()) {
			file.delete();
		} else if (!file.getParentFile().exists()) {
			file.getParentFile().mkdirs();
		}
		// 任务描述文件
		File taskFile = new File(block.getSaveFile() + ".r_task");
		// 真正下载的文件
		File saveFile = new File(block.getSaveFile() + ".r_save");
		deleteSourceFileList.add(taskFile);
		deleteSourceFileList.add(saveFile);
		boolean taskFileExist = taskFile.exists();
		boolean success = true;
		RandomAccessFile taskRandomFile = null;
		RandomAccessFile downRandomFile = null;
		long totalFileSize = 0;
		try {
			totalFileSize = getContentLength(null != downUrl ? downUrl : block.getDownURL());
			if (null != listener) {
				listener.setTotalSize(totalFileSize);
			}
			if (totalFileSize > 0) {
				taskRandomFile = new RandomAccessFile(taskFile, "rw");
				downRandomFile = new RandomAccessFile(saveFile, "rw");
				block.setContentLength(totalFileSize);
				if (!taskFileExist) {
					// 如果文件不存在，就初始化任务文件和下载文件
					initTaskFile(taskRandomFile, block);
					downRandomFile.setLength(totalFileSize);
				} else {
					// 任务描述文件存在就读取任务文件
					block.read(taskRandomFile);
					long[] sourceBlockOffset = getBlockOffset(block);
					long[] downloadedOffset = block.getSectionsOffset();
					if (block.getContentLength() != totalFileSize || sourceBlockOffset.length != downloadedOffset.length) {
						LOG.error("======下载出错====block.getContentLength()=" + block.getContentLength() + "   totalFileSize = "+totalFileSize + "  sourceBlockOffset.length = " +sourceBlockOffset.length +" downloadedOffset.length="+ downloadedOffset.length);
						taskFile.delete();
						saveFile.delete();
						initTaskFile(taskRandomFile, block);
						downRandomFile.setLength(totalFileSize);
					} else {
						long downloadedSize = 0;
						for (int i = 0; i < downloadedOffset.length; i++) {
							downloadedSize += downloadedOffset[i] - sourceBlockOffset[i];
						}
						if (null != listener) {
							listener.hisDownloadSize(downloadedSize);
						}
						block.setDownLoadLength(downloadedSize);
					}
				}
				block.setDownLoadTime(System.currentTimeMillis());
				List<Integer> downBlockList = new ArrayList<Integer>();
				for (int i = 0; i < block.getThreadCount(); i++) {
					downBlockList.add(i);
				}
				List<Integer> errorDownBlockList = downFileByBlock(taskRandomFile, downRandomFile, block, downBlockList);
				// 如果有下载出错的分块，则重新下载一次
				if(null != errorDownBlockList && errorDownBlockList.size() > 0){
					int errorRepartCount = 1;
					// 暂停下载
					DOWN_STATUS downloadStatus = downloadStatus();
					while (errorRepartCount < 10 && downloadStatus != DOWN_STATUS.CANCEL) {
						errorDownBlockList = downFileByBlock(taskRandomFile, downRandomFile, block, downBlockList);
						if (null == errorDownBlockList || errorDownBlockList.size() == 0) {
							LOG.error(messageStartName + " - 第" + errorRepartCount + "次重复下载, 下载全部完成");
							break;
						}
						LOG.error(messageStartName + " - 第" + errorRepartCount + "次重复下载，下载失败的片段数量：" + errorDownBlockList.size());
						errorRepartCount++;
						downloadStatus = downloadStatus();
					}
				}else {
					LOG.debug(messageStartName + " - 下载成功！");
				}
				if (null != errorDownBlockList && errorDownBlockList.size() > 0) {
					success = false;
				}
//				if (null != errorDownBlockList && errorDownBlockList.size() > 0) {
//					errorDownBlockList = downFileByBlock(taskRandomFile, downRandomFile, block, errorDownBlockList);
//					if (null != errorDownBlockList && errorDownBlockList.size() > 0) {
//						success = false;
//					}
//				}
			} else {
				success = false;
				if (taskFile.exists()) {
					taskFile.delete();
				}
			}
		} catch (Exception e) {
			LOG.error("下载失败", e);
			success = false;
		} finally {
			try {
				if (null != taskRandomFile) {
					taskRandomFile.close();
					taskRandomFile = null;
				}
				if (null != downRandomFile) {
					downRandomFile.close();
					downRandomFile = null;
				}
			} catch (IOException e) {
				LOG.error("", e);
			}
			// 如果下载成功，去掉任务描述文件、帮下载文件改名
			String errors = null;
			if (success) {
				if (block.getDownLoadLength() > 0 && block.getDownLoadLength() == totalFileSize) {
					FileUtils.rename(saveFile.getAbsolutePath(), file.getAbsolutePath());
					if (valDownloadFileSize(file)) {
						if (taskFile.exists()) {
							taskFile.delete();
						}
					} else {
						errors = messageStartName + "下载失败:要下载文件的大小为" + block.getContentLength() + "，下载后的文件大小为：" + file.length();
						LOG.error(errors);
						success = false;
						if (file.isFile() && file.exists()) {
							file.delete();
						}
					}
				} else {
					errors = messageStartName + "下载失败:block.getDownLoadLength() = " + block.getDownLoadLength() + "，totalFileSize = " + totalFileSize;
					LOG.error(errors);
					success = false;
				}
			}
			if (!success && isDeleteSourceFile) {
				deleteDownloadFile();
			}
			if(!success) {
				ActionLogUtils.addServerLog(null, (null != messageStartName ? messageStartName : "") +"下载文件失败", downUrl, errors);
			}
		}
		return success;
	}

	private boolean valDownloadFileSize(File file) {
		long downloadFileSize = file.length();
		// "答疑文件"不进行MD5验证，只有Messages.BiddingFile验证。
		if (downloadFileSize > 0 && downloadFileSize == block.getContentLength()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * @Description: 按分块下载文件
	 * @author guoyr
	 * @param taskRandomFile
	 * @param downRandomFile
	 * @param block
	 * @param blockNoList
	 * @return
	 */
	private List<Integer> downFileByBlock(final RandomAccessFile taskRandomFile, final RandomAccessFile downRandomFile, final DownBlock block, List<Integer> blockNoList) {
		List<Integer> errorDownBlockList = null;
		List<Future<Integer>> resultList = new ArrayList<Future<Integer>>();
		// 分配线程去下载，这里用到线程池
		executorService = Executors.newFixedThreadPool(blockNoList.size());
		for (final int blockNo : blockNoList) {
			LOG.debug("下载第" + blockNo + "块分块文件...");
			Future<Integer> future = executorService.submit(new Callable<Integer>() {
				public Integer call() {
					boolean downFlag = false;
					try {
						downFlag = down(taskRandomFile, downRandomFile, block, blockNo);
					} catch (IOException e) {
						LOG.error("", e);
					}
					// 下载失败时返回下载失败的分块
					return downFlag ? -1 : blockNo;
				}
			});
			resultList.add(future);
		}
		executorService.shutdown();
		for (Future<Integer> fs : resultList) {
			int errorBlockNo = 0;
			try {
				errorBlockNo = fs.get();
			} catch (InterruptedException e) {
				LOG.error("", e);
			} catch (ExecutionException e) {
				LOG.error("", e);
			}
			if (errorBlockNo > -1) {
				if (null == errorDownBlockList) {
					errorDownBlockList = new ArrayList<Integer>();
				}
				errorDownBlockList.add(errorBlockNo);
			}
		}
		return errorDownBlockList;
	}

	/**
	 * @Description: 调用HttpURLConnection下载
	 * @author guoyr
	 * @param
	 * @return
	 * @Exception
	 */
	private boolean down(RandomAccessFile taskRandomFile, RandomAccessFile downRandomFile, DownBlock task, int sectionNo) throws IOException {
		boolean downFlag = false;
		long start = task.getSectionsOffset()[sectionNo];

		long end = -1;
		// 这里是计算当前块的长度
		if (sectionNo < task.getBlockCount() - 1) {
			long per = task.getContentLength() / task.getBlockCount();
			end = per * (sectionNo + 1);
		} else {
			end = task.getContentLength();
		}
		if (start >= end) {
			LOG.debug("已经在第 " + sectionNo + "分块下载完...");
			return true;
		}
		HttpURLConnection httpURLConnection = null;
		try {
			SSLContext sslcontext=SSLContext.getInstance("SSL","SunJSSE");
			sslcontext.init(null ,  new TrustManager[]{new MyX509TrustManager()}, new java.security.SecureRandom());
			URL u = new URL(null != downUrl ? downUrl : task.getDownURL());
			HostnameVerifier ignoreHostnameVerifier = new HostnameVerifier() {
				public boolean verify(String s, SSLSession sslsession) {
					System.out.println("WARNING: Hostname is not matched for cert.");
					return true;
				}
			};
			HttpsURLConnection.setDefaultSSLSocketFactory(sslcontext.getSocketFactory());
			HttpsURLConnection.setDefaultHostnameVerifier(ignoreHostnameVerifier);
			httpURLConnection = (HttpURLConnection) u.openConnection();
			
			httpURLConnection.setDoOutput(true);
			httpURLConnection.setDoInput(true);
			httpURLConnection.setRequestMethod("POST");
			httpURLConnection.setUseCaches(false);
			
			httpURLConnection.setRequestMethod("POST");
			String range = "bytes=" + start + "-" + (end - 0);// 这里是end-1
			httpURLConnection.setRequestProperty("Range", range);
			httpURLConnection.setRequestProperty("Content-Type", "application/octet-stream");
			httpURLConnection.setRequestProperty("Signature", "UE");
			httpURLConnection.setConnectTimeout(100000);
			httpURLConnection.setReadTimeout(100000);
			httpURLConnection.connect();
			if (!(httpURLConnection.getResponseCode() == 206 || httpURLConnection.getResponseCode() == 200)) {
				LOG.error("下载失败=========第"+sectionNo+"个片段"+range+"请求URL异常 httpURLConnection.getResponseCode()=" + httpURLConnection.getResponseCode());
			}
			long contentLength = httpURLConnection.getContentLength();
			if(contentLength <= 0){
				contentLength = Long.parseLong(httpURLConnection.getHeaderField("Content-Length1"));
			}
			if (contentLength != (end - start)) {
				LOG.error("下载失败=========第"+sectionNo+"个片段请求下载范围为"+range+" 共"+(end - start)+"个字节，服务器返回长度为" + contentLength + "与请求的下载字节数不相同");
			}
			InputStream is = httpURLConnection.getInputStream();
			task.setBufferSize(8 * 1024 * 1024);
			byte[] temp = new byte[task.getBufferSize()];
			BufferedInputStream bis = new BufferedInputStream(is);
			// BufferedInputStream bis = new BufferedInputStream(is, temp.length);
			int readed = 0;
			while ((readed = bis.read(temp)) != -1) {
				if (null != listener) {
					listener.writeSize(readed);
				}
				synchronized (task) {
					long offset = task.getSectionsOffset()[sectionNo];
					// 下载之后顺便更新描述文件，这里效率比较低，在一个线程下载里进行两次文件操作，可以实现一个缓冲写。
					downRandomFile.seek(offset);
					downRandomFile.write(temp, 0, readed);
					offset += readed;
					task.getSectionsOffset()[sectionNo] = offset;
					task.writeOffset(taskRandomFile);
					task.setDownLoadLength(task.getDownLoadLength() + readed);
				}
				DOWN_STATUS downloadStatus = downloadStatus();
				// 暂停下载
				while (downloadStatus == DOWN_STATUS.PAUSE) {
					Thread.sleep(1000);
					downloadStatus = downloadStatus();
				}
				// 停止下载
				if (downloadStatus == DOWN_STATUS.CANCEL || downloadStatus == DOWN_STATUS.FAIL) {
					break;
				}
			}
			if (task.getSectionsOffset()[sectionNo] == end) {
				downFlag = true;
			}
		} catch (Exception e) {
			LOG.error("", e);
		} finally {
			if(!downFlag){
				LOG.error(messageStartName + " - 第"+sectionNo+"个片段下载失败");
			}
			if(null != httpURLConnection){
				httpURLConnection.disconnect();
			}
		}
		return downFlag;
	}

	/**
	 * @author guoyr
	 * @param
	 * @return
	 * @Exception
	 */
	public void initTaskFile(RandomAccessFile taskRandomFile, DownBlock task) throws IOException {
		task.setSectionsOffset(getBlockOffset(task));
		task.create(taskRandomFile);
	}

	public long[] getBlockOffset(DownBlock task) {
		int blockCount = task.getBlockCount();
		long per = task.getContentLength() / blockCount;
		long[] blockOffset = new long[blockCount];
		for (int i = 0; i < blockCount; i++) {
			blockOffset[i] = per * i;
		}
		return blockOffset;
	}

	/**
	 * @author guoyr
	 * @param
	 * @return
	 * @Exception
	 */
	public long getContentLength(String url) {
		URL u = null;
		HttpURLConnection httpURLConnection = null;
		long contentLength = -1;
		try {
			SSLContext sslcontext=SSLContext.getInstance("SSL","SunJSSE");
			sslcontext.init(null ,  new TrustManager[]{new MyX509TrustManager()}, new java.security.SecureRandom());
			u = new URL(url);
			HostnameVerifier ignoreHostnameVerifier = new HostnameVerifier() {
				public boolean verify(String s, SSLSession sslsession) {
					System.out.println("WARNING: Hostname is not matched for cert.");
					return true;
				}
			};
			HttpsURLConnection.setDefaultSSLSocketFactory(sslcontext.getSocketFactory());
			HttpsURLConnection.setDefaultHostnameVerifier(ignoreHostnameVerifier);
			
			httpURLConnection = (HttpURLConnection) u.openConnection();
			httpURLConnection.setRequestMethod("POST");
			httpURLConnection.setRequestProperty("Content-Type", "application/octet-stream");
			httpURLConnection.setRequestProperty("Signature", "UE");
			httpURLConnection.setRequestProperty("FileSize", "0");
			if (!(httpURLConnection.getResponseCode() == 206 || httpURLConnection.getResponseCode() == 200)) {
				LOG.error("请求URL异常==========httpURLConnection.getResponseCode()===========" + httpURLConnection.getResponseCode());
			}
			contentLength = httpURLConnection.getContentLength();
			if(contentLength <= 0){
				String contentLengthStr = httpURLConnection.getHeaderField("Content-Length1");
				if(null != contentLengthStr && contentLengthStr.length() > 0){
					try {
						contentLength = Long.parseLong(httpURLConnection.getHeaderField("Content-Length1"));
					} catch (Exception e) {
					}
				}
			}
			if(contentLength <= 0){
				contentLength = tempFileSize;
				LOG.error("使用申请文件尺寸===========" + tempFileSize);
			}
			if(contentLength <= 0){
				LOG.error("下载地址：" + url +"下载文件大小：" +contentLength);
				return -1;
			}else {
				LOG.debug(url + "要下载的文件长度为" + contentLength);
			}
			try {
				httpURLConnection.setConnectTimeout(100000);
				httpURLConnection.setReadTimeout(100000);
			} catch (Exception e) {
				LOG.error("下载地址：" + url +"下载文件大小：" +contentLength , e);
				return -1;
			}
			block.setMd5(httpURLConnection.getHeaderField("MD5"));
			return contentLength;
		} catch (Exception e) {
			LOG.error("下载地址：" + url +"下载文件大小：" +contentLength , e);
			return -1;
		}finally {
			if (null != httpURLConnection) httpURLConnection.disconnect();
		}
	}

	public DownBlock getBlock() {
		return block;
	}

	public void setBlock(DownBlock block) {
		this.block = block;
	}

	public void setDeleteSourceFile(boolean isDeleteSourceFile) {
		this.isDeleteSourceFile = isDeleteSourceFile;
	}

	public void deleteDownloadFile() {
		for (File deleteSourceFile : deleteSourceFileList) {
			if (null != deleteSourceFile && deleteSourceFile.exists()) {
				deleteSourceFile.delete();
			}
		}
	}

	public abstract DOWN_STATUS downloadStatus();

	public void setMessageStartName(String messageStartName) {
		this.messageStartName = messageStartName;
	}

}
