package com.gx.obe.common.file.service;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.LockSupport;

import org.apache.log4j.Logger;

import com.gx.obe.business.entity.SignatureFile;
import com.gx.obe.business.entity.SignatureUser;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.upload.UploadFileUtil;
import com.gx.obe.common.file.upload.infer.UploadFileListener;
import com.gx.obe.components.core.Constants;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.web.entity.auth.AuthUser;
import  com.gx.obe.web.utils.NetworkRequest;
import com.gx.obe.web.utils.ResultStatus;
import com.gx.obe.web.vo.SaveBatchSignatureFileVo;
import com.gx.obe.web.vo.SaveSignatureFileVo;
import com.gx.obe.web.vo.UpdateAssignPropertyVo;
import com.gx.utils.NowTimeUtils;

/** 
 * @Description: 
 * @author mazc 
 */
public class SignatureFileService {
static final String URL = "signatureFile";
	
	private Logger logger = Logger.getLogger(SignatureFileService.class);

	/**
	 * @Description: 获得当前项目的流转签章文件
	 * @author mazc
	 * @param tenderId
	 * @return 
	 */
	public List<SignatureFile> getSignatureFileList(String tenderId,String reportType){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/getSignatureFileList");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("tenderId", tenderId );
		param.put("reportType", reportType);
  		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			if(ResultStatus.getReultStatusIsNotEmpty(result)){
 				return JsonUtil.strToList(result, SignatureFile.class);	 
 			}
 			return null;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return null;
		}
	}
	
	
	/**
	 * @Description: 添加流转签章报表
	 * @author mazc
	 * @param tenderId
	 * @param fileList
	 * @param userList 
	 */
	public SignatureFile saveSignatureFile(String tenderId, File file,String reportType, List<AuthUser> userList){
		List<File> fileList = new ArrayList<File>();
		fileList.add(file);
		List<SignatureFile> signatureFileList = saveBatchSignatureFile(tenderId,reportType, fileList, userList);
		if(null != signatureFileList && signatureFileList.size() > 0) {
			return signatureFileList.get(0);
		}
		return null;
	}
	/**
	 * @Description: 添加流转签章报表
	 * @author mazc
	 * @param tenderId
	 * @param fileList
	 * @param userList 
	 */
	public List<SignatureFile> saveBatchSignatureFile(String tenderId,String reportType, List<File> fileList, List<AuthUser> userList){
		List<SignatureFile> signatureFileList = new ArrayList<SignatureFile>();
		if(null != fileList && null != userList){
			List<SignatureUser> signatureUserList = new ArrayList<SignatureUser>();
			for(File file : fileList){
				if(uploadReportFile(file,reportType, tenderId)){
					SignatureFile signatureFile = new SignatureFile();
					signatureFile.setId(UuidUtils.getUUID());
					signatureFile.setCreateTime(NowTimeUtils.getNowDate());
					signatureFile.setTenderId(tenderId);
					signatureFile.setFileName(file.getName());
					signatureFile.setReportType(reportType);
					signatureFileList.add(signatureFile);
					for(AuthUser user : userList){
						SignatureUser signatureUser = new SignatureUser();
						signatureUser.setId(UuidUtils.getUUID());
						signatureUser.setSignatureId(signatureFile.getId());
						signatureUser.setUserName(user.getUserName());
						signatureUser.setUserId(user.getId());
						signatureUserList.add(signatureUser);
					}
				}
			}
			SaveBatchSignatureFileVo  saveBatchSignatureFileVo = new SaveBatchSignatureFileVo();
			saveBatchSignatureFileVo.setSignatureFileList(signatureFileList);
			saveBatchSignatureFileVo.setSignatureUserList(signatureUserList);
			//请求地址
			String strWebServerUrl = Constants.getServiceUrl(URL + "/saveBatchSignatureFile");
			try{
				String result = NetworkRequest.post(strWebServerUrl, JsonUtil.jsonObjectToStr(saveBatchSignatureFileVo));
				if(ResultStatus.getReultStatusBoolean(result)){	
					return signatureFileList;
				}
				return signatureFileList;
			} catch (Exception e) {
				logger.error(e.getMessage(), e);
				return null;
			}
		}
		return signatureFileList;	
	}
	
    /**
     * @Description: 上传报表文件
     * @author chenxw
     * @param file
     * @param tenderId
     * @return 
     */
	public boolean uploadReportFile(File file,String reportType, String tenderId) {
        final Thread thread = Thread.currentThread();
        UploadReportFile uploadReportFile = new UploadReportFile() {
            private boolean isCompleted;
            
            public void Completed(boolean isCompleted) {
                this.isCompleted = isCompleted;
                LockSupport.unpark(thread);
            }
            
            public boolean isCompleted() {
                return isCompleted;
            }
        };
        UploadFileUtil uploadFileUtil = new UploadFileUtil(file.getAbsolutePath(), tenderId, null, reportType, uploadReportFile);
        uploadFileUtil.startUploadFileFile();
        LockSupport.park();
        return uploadReportFile.isCompleted();
    }
	
    /**
     * @Description: 检查报表PDF文件是否为流转文件
     * @author chenxw
     * @param tenderId
     * @param pdfFileName
     * @return 
     */
    public boolean checkIsReportFile(String tenderId, String pdfFileName) {
        String strWebServerUrl = Constants.getServiceUrl(URL + "/checkIsReportFile");
        Map<String, Object> param = new HashMap<String, Object>();
        param.put("tenderId", tenderId);
        param.put("pdfFileName", pdfFileName);
        try {
            String result = NetworkRequest.get(strWebServerUrl, param);
            return ResultStatus.getReultStatusBoolean(result);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return false;
        }
    }

    
    class UploadReportFile implements UploadFileListener {
        
        public boolean isCompleted() {
            return false;
        }
        
        public void Completed(boolean loginFlag) {}
        
        public void computeherLoadProgress(int percent) {}
        
        public void computeDownloadSpeed(String Speed) {}
        
        public void computeRemainingTimeCurrent(String time) {}
        
        public void MD5check(boolean isCheck) {}
        
    }
    
    /**
	 * @Description: 获得签名文件
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	public SignatureFile getSignatureFileById(String id) {
   		String strWebServerUrl = Constants.getServiceUrl( URL + "/getSignatureFileById");
   		Map<String, Object> param = new HashMap<String, Object>();
   		param.put("id", id );
   		try {
   			String result = NetworkRequest.get(strWebServerUrl, param);
   			if(ResultStatus.getReultStatusIsNotEmpty(result)){
   				return JsonUtil.jsonToBean(result, SignatureFile.class);
   			}
   			return null;
   		} catch (Exception e) {
   			logger.error(e.getMessage(), e);
   			return null;
   		}
   	}
	/**
	 * @Description: 根据文件名称获取待签名的文件
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	public SignatureFile getSignatureFileByFileName(String tenderId, String fileName){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/getSignatureFileByFileName");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("tenderId", tenderId );
		param.put("fileName", fileName );
		try {
			String result = NetworkRequest.get(strWebServerUrl, param);
			if(ResultStatus.getReultStatusIsNotEmpty(result)){
				return JsonUtil.jsonToBean(result, SignatureFile.class);
			}
			return null;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return null;
		}
		
	}

	 /**
	 * @Description: 批量申请流转签名
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @param userName
	 * @return 
	 */
	public int batchApplySignatureFile(String tenderId, String userId, String userName) {
		String strWebServerUrl = Constants.getServiceUrl( URL + "/batchApplySignatureFile");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("tenderId", tenderId );
		param.put("userId", userId );
		param.put("userName", userName );
		int applyCount = 0;
  		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			if(ResultStatus.getReultStatusIsNotEmpty(result)){
 				if (ResultStatus.isReultStatusInteger(result)) {
 					applyCount = Integer.valueOf(result);
 				}
 			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return applyCount;
	}
	
	/**
	 * @Description: 批量释放已申请签章
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	public void batchReleaseApplySignatureFile(String tenderId, String userId) {
		String strWebServerUrl = Constants.getServiceUrl( URL + "/batchReleaseApplySignatureFile");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("tenderId", tenderId );
		param.put("userId", userId );
		try {
			NetworkRequest.get(strWebServerUrl, param);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
	}
	
	/**
	 * @Description: 释放已申请签章
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	public void releaseApplySignatureFile(String signatureId, String userId) {
		String strWebServerUrl = Constants.getServiceUrl( URL + "/releaseApplySignatureFile");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("signatureId", signatureId );
		param.put("userId", userId );
		try {
			NetworkRequest.get(strWebServerUrl, param);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
	}
    
    /**
	 * @Description: 申请签章
	 * @author mazc
	 * @param signatureId
	 * @param userId
	 * @param userName
	 * @return 
	 */
	public SignatureFile applySignatureFile(String signatureId, String userId, String userName){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/applySignatureFile");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("signatureId", signatureId );
		param.put("userId", userId );
		param.put("userName", userName );
  		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			if(ResultStatus.getReultStatusIsNotEmpty(result)){
 				return JsonUtil.jsonToBean(result, SignatureFile.class);
 			}
 			return null;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return null;
		}
		
	}
	
	/**
	 * @Description: 保存签章文件
	 * @author mazc
	 * @param signatureId
	 * @param signatureFileByte
	 * @param userId
	 * @param signatureTime
	 * @return 
	 */
	public boolean saveSignatureFile(String signatureId, SignatureUser signatureUser, Date signatureTime){
		SaveSignatureFileVo saveSignatureFileVo = new SaveSignatureFileVo();
		saveSignatureFileVo.setSignatureId(signatureId);
		saveSignatureFileVo.setSignatureTime(signatureTime);
		saveSignatureFileVo.setSignatureUser(signatureUser);
		//请求地址
		String strWebServerUrl = Constants.getServiceUrl(URL + "/saveSignatureFile");
		try{
			String result = NetworkRequest.post(strWebServerUrl, JsonUtil.jsonObjectToStr(saveSignatureFileVo));
			return ResultStatus.getReultStatusBoolean(result);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return false;
		}
	}
	
	/**
	 * @Description: 删除流转签章文件
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	public boolean deleteSignatureFile(String signatureId){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/deleteSignatureFile");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("signatureId", signatureId );
  		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			return ResultStatus.getReultStatusBoolean(result);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return false;
		}
	}
	
	public SignatureFile getById(String id){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/getById");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("id", id );
  		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			if(ResultStatus.getReultStatusIsNotEmpty(result)){
 				return JsonUtil.jsonToBean(result, SignatureFile.class);
 			}
 			return null;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return null;
		}
	}
	
	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param signatureFile
	 * @param attributes
	 * @return 
	 */
	public boolean updateAssignProperty(SignatureFile signatureFile ,String [] attributes){
	  //登陆请求地址
        String strWebServerUrl = Constants.getServiceUrl( URL + "/updateAssignProperty");
        UpdateAssignPropertyVo<SignatureFile>updateAssignPropertyVo = new UpdateAssignPropertyVo<SignatureFile>();
        updateAssignPropertyVo.setEntity(signatureFile);
        updateAssignPropertyVo.setAttributes(attributes);
        try{
            String result = NetworkRequest.post(strWebServerUrl, JsonUtil.jsonObjectToStr(updateAssignPropertyVo));
            return ResultStatus.getReultStatusBoolean(result);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return false;
        }
	}
	
	public List<SignatureFile> getSignatureFileByTenderIdAndOrderByCreateTime(String tenderId){
		String strWebServerUrl = Constants.getServiceUrl(URL + "/getSignatureFileByTenderIdAndOrderByCreateTime");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("tenderId", tenderId );
		try{
			String result = NetworkRequest.get(strWebServerUrl, param);
			if(ResultStatus.getReultStatusIsNotEmpty(result)){	
				return JsonUtil.jsonToList(result,SignatureFile.class);
			}
			return null;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return null;
		}
	}
	
	/**
	 * @Description: 获取当前签名文件的签名人ID
	 * @author mazc
	 * @param signatureId
	 * @return 
	 */
	public String getSignatureUserId(String signatureId){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/getSignatureUserId");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("signatureId", signatureId );
  		try {
 			String userId = NetworkRequest.get(strWebServerUrl, param);
 			return userId;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
  		return null;
	}
}
