package com.gx.obe.common.file.service;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.gx.obe.business.entity.SignatureFile;
import com.gx.obe.business.entity.SignatureUser;
import com.gx.obe.components.core.Constants;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.http.util.DateUtil;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.web.entity.auth.AuthUser;
import  com.gx.obe.web.utils.NetworkRequest;
import com.gx.obe.web.utils.ResultStatus;
import com.gx.obe.web.vo.UpdateAssignPropertyVo;

/** 
 * @Description: 
 * @author mazc 
 */
public class SignatureUserService {
	
	static final String URL = "signatureUser";
	
	private Logger logger = Logger.getLogger(SignatureUserService.class);

	
	/**
	 * @Description: 获得当前用户需要签章的报表
	 * @author mazc
	 * @param tenderId
	 * @param userId
	 * @return 
	 */
	public List<SignatureUser> getUserSignatureFileList(String tenderId, String userId){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/getUserSignatureFileList");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("tenderId", tenderId );
		param.put("userId", userId );
 		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			if(ResultStatus.getReultStatusIsNotEmpty(result)){
 				return JsonUtil.strToList(result, SignatureUser.class);	 
 			}
 			return null;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return null;
		}
	}
	
	/**
	 * @Description: 获得当前用户需要签章的报表
	 * @author mazc
	 * @param signatureId
	 * @param userId
	 * @return 
	 */
	public SignatureUser getSignatureUserById(String signatureUserId){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/getSignatureUserById");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("id", signatureUserId );
 		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			if(ResultStatus.getReultStatusIsNotEmpty(result)){
 				return JsonUtil.jsonToBean(result, SignatureUser.class);	 
 			}
 			return null;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return null;
		}
	}

	public boolean updateSignatureUser(String id, String signatureType, Date signatureTime){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/updateSignatureUser");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("id", id );
		param.put("signatureType", signatureType );
		param.put("signatureTime", DateUtil.formatDateTime(signatureTime));
 		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			return ResultStatus.getReultStatusBoolean(result);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return false;
		}
	}
	
	public List<SignatureUser> getSignatureUserBySignatureId(String signatureId){
		String strWebServerUrl = Constants.getServiceUrl( URL + "/getSignatureUserBySignatureId");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("signatureId", signatureId );
		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			if(ResultStatus.getReultStatusIsNotEmpty(result)){
 				return JsonUtil.jsonToList(result, SignatureUser.class);	 
 			}
 			return null;
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return null;
		}
	}
	
	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param signatureFile
	 * @param attributes
	 * @return 
	 */
	public boolean updateAssignProperty(SignatureUser signatureUser ,String [] attributes){
	  //登陆请求地址
        String strWebServerUrl = Constants.getServiceUrl( URL + "/updateAssignProperty");
        UpdateAssignPropertyVo<SignatureUser>updateAssignPropertyVo = new UpdateAssignPropertyVo<SignatureUser>();
        updateAssignPropertyVo.setEntity(signatureUser);
        updateAssignPropertyVo.setAttributes(attributes);
        try{
            String result = NetworkRequest.post(strWebServerUrl, JsonUtil.jsonObjectToStr(updateAssignPropertyVo));
            return ResultStatus.getReultStatusBoolean(result);
        } catch (Exception e) {
            logger.error(e.getMessage(), e);
            return false;
        }
	}
	
	/**
	 * @Description: 添加流转签章用户
	 * @author mazc
	 * @param signatureFile
	 * @param userList
	 */
	public int addSignatureUser(SignatureFile signatureFile, List<AuthUser> userList) {
		List<SignatureUser> signatureUserList = new ArrayList<SignatureUser>();
		for(AuthUser user : userList){
			SignatureUser signatureUser = new SignatureUser();
			signatureUser.setId(UuidUtils.getUUID());
			signatureUser.setSignatureId(signatureFile.getId());
			signatureUser.setUserName(user.getUserName());
			signatureUser.setUserId(user.getId());
			signatureUserList.add(signatureUser);
		}
		
		//登陆请求地址
		String strWebServerUrl = Constants.getServiceUrl( URL + "/insertByBatch");
		String result = NetworkRequest.post(strWebServerUrl, JsonUtil.jsonObjectToStr(signatureUserList));
		if(ResultStatus.isReultStatusInteger(result)){
			return Integer.valueOf(result);
		}
		return 0;
	}
	
	/**
	 * @Description: 通过用户名修改用户签章状态
	 * @author guoyr
	 * @param signatureId
	 * @param userName
	 * @param signatureType
	 * @param signatureTime
	 * @return 
	 */
	public boolean updateSignatureTypeByUserName(String signatureId, String userName, String signatureType){
		//登陆请求地址
		String strWebServerUrl = Constants.getServiceUrl( URL + "/updateSignatureTypeByUserName");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("signatureId", signatureId );
		param.put("userName", userName );
		param.put("signatureType", signatureType );
 		try {
 			String result = NetworkRequest.get(strWebServerUrl, param);
 			return ResultStatus.getReultStatusBoolean(result);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			return false;
		}
	}
}
