package com.gx.obe.common.file.service;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Logger;

import com.gx.obe.components.core.Constants;
import com.gx.obe.http.HttpResponse;
import com.gx.obe.http.Status;
import com.gx.obe.util.utils.MD5Utils;
import com.gx.obe.web.utils.NetworkRequest;
import com.gx.obe.web.utils.ResultStatus;

/**
 * @Description: 软件文件接口
 * @author chenxw
 */
public class SoftwareUploadService {
	
	private static final String URL = "softwareUpload";
	
	private static final Logger LOG = Logger.getLogger(SoftwareUploadService.class);
	
	/**
	 * @Description: 检查文件是否存在
	 * @author chenxw
	 * @param fileName
	 * @return
	 */
	public boolean checkFileisExists(String fileName) {
		String strWebServerUrl = Constants.getServiceUrl(URL + "/checkFile");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("filename", fileName);
		try {
			HttpResponse strResult = NetworkRequest.postHttpResponse(strWebServerUrl, param);
			if (strResult.getStatus() == Status.HTTP_OK) {
				return ResultStatus.getReultStatusBoolean(strResult.body());
			}
		} catch (Exception e) {
			LOG.error(e.getMessage(), e);
		}
		return false;
	}
	
	/**
	 * @Description: 检查文件md5值是否相同
	 * @author chenxw
	 * @param fileName
	 * @param fileMD5
	 * @return
	 */
	public boolean checkFileChunkMD5(String fileName, File file) {
		String strWebServerUrl = Constants.getServiceUrl(URL + "/checkFileMD5");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("fileName", fileName);
		try {
			param.put("fileMD5", MD5Utils.getFileMD5String(file));
			HttpResponse strResult = NetworkRequest.postHttpResponse(strWebServerUrl, param);
			if (strResult.getStatus() == Status.HTTP_OK) {
				return ResultStatus.getReultStatusBoolean(strResult.body());
			}
		} catch (Exception e) {
			LOG.error(e.getMessage(), e);
		}
		return false;
	}
	
}
