package com.gx.obe.common.file.shell;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.SwtUtils;
import org.eclipse.swt.core.widgets.ELabel;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.core.widgets.ImageButton;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.file.composite.FileDownloadProjectComposite;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.inter.IFileDownloadProject;
import com.gx.obe.common.file.listener.DownloadListener;
import com.gx.obe.common.file.listener.IFileDownload;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.util.utils.XmlUtils.UpdateXml;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 下载
 * @author chenxw
 */
public class FileDownloadShell extends Shell {
	private static FileDownloadShell fileDownloadShell;
	private static ExecutorService executorService;
	private CLabel label;
	private Label messagelabel;
	private Composite downloadComposite;
	private Composite buttonComposite;
	private Composite downloadingFrame;
	private Composite downloadedFrame;
	private Composite downloading;
	private Composite downloaded;
	private ScrolledComposite downloadingScrolledComposite;
	private ScrolledComposite downloadedScrolledComposite;
	private StackLayout stackLayout = new StackLayout();
	private ImageButton downloadingButton;
	private ImageButton downloadButton;
	private DownloadListener downloadListener;
	private Map<String, IFileDownloadProject> mapDownloading;
	private Map<String, IFileDownloadProject> mapDownloaded;
	private File xmlFile = new File(FileConstants.SYS_USER_ROOT.concat("downloaded").concat(File.separator).concat("downloaded.xml"));

	/**
	 * @Description: 获得单例
	 * @author chenxw
	 * @return
	 */
	public static FileDownloadShell getInstance() {
		if (null == fileDownloadShell || fileDownloadShell.isDisposed()) {
			fileDownloadShell = new FileDownloadShell(Display.getDefault());
//			fileDownloadShell.initDownloaded();
			executorService = Executors.newFixedThreadPool(5);
			fileDownloadShell.open();
		}
		return fileDownloadShell;
	}

	/**
	 * @Description: Http招标文件下载
	 * @author chenxw
	 * @param tenderProjectEntity
	 */
	public void httpBiddingDownload(TenderProjectEntity tenderProject) {
		download(tenderProject, null, null, DownloadListener.HTTP_BIDDING);
	}

	/**
	 * @Description: Http投标文件下载
	 * @author chenxw
	 * @param supplier
	 */
	public void httpBidDownload(TenderProjectEntity tenderProject, Supplier supplier, String BIDOPENING_STAGE) {
		download(tenderProject, supplier, BIDOPENING_STAGE, DownloadListener.HTTP_BID);
	}
	/**
	 * @Description: Http投标文件下载
 	 * @param supplier
	 */
	public void serviceHttpBidDownload(TenderProjectEntity tenderProject, Supplier supplier, String BIDOPENING_STAGE) {
		download(tenderProject, supplier, BIDOPENING_STAGE, DownloadListener.SERVICE_HTTP_BID);
	}
	
	public void serviceHttpBiddingDownload(TenderProjectEntity tenderProject, String BIDOPENING_STAGE) {
		download(tenderProject,null, BIDOPENING_STAGE, DownloadListener.SERVICE_BIDDING);
	}
	
	/**
	 * @Description: Ftp招标文件下载
	 * @author chenxw
	 * @param tenderProjectEntity
	 */
	public void ftpBiddingDownload(TenderProjectEntity tenderProject) {
		download(tenderProject, null, null, DownloadListener.FTP_BIDDING);
	}

	/**
	 * @Description: Ftp投标文件下载
	 * @author chenxw
	 * @param supplier
	 */
	public void ftpBidDownload(TenderProjectEntity tenderProject, Supplier supplier, String BIDOPENING_STAGE) {
		download(tenderProject, supplier, BIDOPENING_STAGE, DownloadListener.FTP_BID);
	}

	/**
	 * @Description: 下载
	 * @author chenxw
	 * @param selectSupplierList
	 * @param tenderProjectEntity
	 * @param bIDOPENING_STAGE
	 * @param downloadListener
	 * @param isFtp
	 */
	private void download(TenderProjectEntity tenderProject, Supplier supplier, String BIDOPENING_STAGE, String downloadType) {
		if (null == fileDownloadShell || this.isDisposed()) {
			getInstance().open();
			SwtUtils.setCenter(fileDownloadShell);
		}
		fileDownloadShell.addFileDownloadingProject(tenderProject, supplier, BIDOPENING_STAGE, downloadType);
		this.setVisible(true);
		this.setMinimized(false);
		this.forceActive();
	}

	/**
	 * @author chenxw
	 * @param display
	 */
	public FileDownloadShell(Display display) {
		super(display, SWT.NO_TRIM);
		GridLayout gl_shell = new GridLayout(1, false);
		gl_shell.marginHeight = 1;
		gl_shell.marginWidth = 1;
		gl_shell.horizontalSpacing = 0;
		gl_shell.verticalSpacing = 0;
		setLayout(gl_shell);
		setBackgroundMode(SWT.INHERIT_FORCE);
		setBackground(ColorConstants.SKIN_BG);
		
		Composite composite = new Composite(this, SWT.NONE);
		GridLayout gl_composite = new GridLayout(3, false);
		gl_composite.marginHeight = 0;
		gl_composite.verticalSpacing = 0;
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false, 1, 1));

		label = new CLabel(composite, SWT.NONE);
		label.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		label.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/white/download.png"));
		label.setForeground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		label.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		label.setText(Messages.Button_download);

		ELabel min_size_btn = new ELabel(composite, SWT.NONE | SWT.SELECTED);
		min_size_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/min_size.png"));
		min_size_btn.setMargins(5, 5, 5, 5);
		min_size_btn.setActiveColor(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		min_size_btn.setAlpha(10);
		min_size_btn.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				setMinimized(true);
			}
		});
		ELabel close_btn = new ELabel(composite, SWT.NONE | SWT.SELECTED);
		close_btn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/close.png"));
		close_btn.setMargins(5, 5, 5, 5);
		close_btn.setActiveColor(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		close_btn.setAlpha(10);
		close_btn.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				closeAction();
			}
		});

		Composite composite_3 = new Composite(this, SWT.NONE);
		composite_3.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_composite_3 = new GridLayout(1, true);
		gl_composite_3.horizontalSpacing = 0;
		gl_composite_3.verticalSpacing = 0;
		composite_3.setLayout(gl_composite_3);
		composite_3.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		buttonComposite = new Composite(composite_3, SWT.NONE);
		buttonComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridLayout gl_buttonComposite = new GridLayout(2, false);
		gl_buttonComposite.verticalSpacing = 0;
		gl_buttonComposite.marginWidth = 0;
		gl_buttonComposite.marginHeight = 0;
		buttonComposite.setLayout(gl_buttonComposite);
		buttonComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		ESepator topSep = new ESepator(composite_3, SWT.NONE);
		topSep.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		topSep.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));

		Composite composite_4 = new Composite(composite_3, SWT.NONE);
		composite_4.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		composite_4.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		GridLayout gl_composite_4 = new GridLayout(1, false);
		gl_composite_4.marginHeight = 0;
		gl_composite_4.marginWidth = 0;
		gl_composite_4.verticalSpacing = 0;
		gl_composite_4.horizontalSpacing = 0;
		composite_4.setLayout(gl_composite_4);

		downloadComposite = new Composite(composite_4, SWT.NONE);
		downloadComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		downloadComposite.setLayout(stackLayout);
		downloadComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));

		downloadingFrame = new Composite(downloadComposite, SWT.NONE);
		GridLayout gl_downloadingFrame = new GridLayout(1, false);
		gl_downloadingFrame.verticalSpacing = 0;
		gl_downloadingFrame.marginWidth = 0;
		gl_downloadingFrame.marginHeight = 0;
		downloadingFrame.setLayout(gl_downloadingFrame);

		downloadedFrame = new Composite(downloadComposite, SWT.NONE);
		GridLayout gl_downloadedFrame = new GridLayout(1, false);
		gl_downloadedFrame.verticalSpacing = 0;
		gl_downloadedFrame.marginWidth = 0;
		gl_downloadedFrame.marginHeight = 0;
		downloadedFrame.setLayout(gl_downloadedFrame);

		downloadingScrolledComposite = new ScrolledComposite(downloadingFrame, SWT.H_SCROLL | SWT.V_SCROLL);
		downloadingScrolledComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		downloadingScrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		downloadingScrolledComposite.setExpandHorizontal(true);
		downloadingScrolledComposite.setExpandVertical(true);

		downloading = new Composite(downloadingScrolledComposite, SWT.NONE);
		GridLayout gl_downloading = new GridLayout(1, false);
		downloading.setLayout(gl_downloading);

		messagelabel = new Label(downloading, SWT.NONE);
		messagelabel.setForeground(SWTResourceManager.getColor(SWT.COLOR_GRAY));
		messagelabel.setFont(SWTResourceManager.getFont("楷体", 14, SWT.BOLD));
		messagelabel.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, true, 1, 1));
		messagelabel.setText(Messages.Tips_noDownloadTask);
		downloadingScrolledComposite.setContent(downloading);
		downloadingScrolledComposite.setMinSize(downloading.computeSize(SWT.DEFAULT, SWT.DEFAULT));

		downloadedScrolledComposite = new ScrolledComposite(downloadedFrame, SWT.H_SCROLL | SWT.V_SCROLL);
		downloadedScrolledComposite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		downloadedScrolledComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		downloadedScrolledComposite.setExpandHorizontal(true);
		downloadedScrolledComposite.setExpandVertical(true);

		downloaded = new Composite(downloadedScrolledComposite, SWT.NONE);
		GridLayout gl_downloaded = new GridLayout(1, false);
		downloaded.setLayout(gl_downloaded);
		downloadedScrolledComposite.setContent(downloaded);
		downloadedScrolledComposite.setMinSize(downloaded.computeSize(SWT.DEFAULT, SWT.DEFAULT));

		downloadingButton = createImageButton(Messages.Tips_downloading, downloadingFrame);
		createImageButton(Messages.Tips_downloaded, downloadedFrame);

		init();

		createContents();
		checkSubclass();
	}

	/**
	 * @Description: 初始化
	 * @author chenxw
	 */
	private void init() {
		SwtUtils.setMouseDragListener(label);
		SwtUtils.setCenter(this);
		setMinimized(true);
		setVisible(false);
		mapDownloading = new HashMap<String, IFileDownloadProject>();
		mapDownloaded = new HashMap<String, IFileDownloadProject>();
	}

	/**
	 * @Description: 关闭
	 * @author chenxw
	 */
	private void closeAction() {
		if (fileDownloadShell.isContainDownloadingTask()) {
			if (MessageDialog.openConfirm(new Shell(), Messages.Quit, Messages.Tips_confirmExit, Messages.Tips_stopAndExit, Messages.Tips_minimizeDownload)) {
				fileDownloadShell.closeDownloadTask();
			} else {
				setMinimized(true);
			}
		} else {
			fileDownloadShell.closeDownloadTask();
		}
	}

	/**
	 * @Description: 添加正在下载界面列表数据
	 * @author chenxw
	 * @param selectSupplierList
	 * @param tenderProjectEntity
	 * @param bIDOPENING_STAGE
	 * @param isFtp
	 */
	public void addFileDownloadingProject(TenderProjectEntity tenderProject, Supplier supplier, String BIDOPENING_STAGE, String downloadType) {
		if (null != downloadButton && !downloadButton.isDisposed()) {
			downloadButton.setSelection(false);
		}
		downloadButton = downloadingButton;
		downloadButton.setSelection(true);
		stackLayout.topControl = downloadingFrame;
		downloadComposite.layout();
		((GridData) messagelabel.getLayoutData()).exclude = true;
		messagelabel.setVisible(false);
		String key = tenderProject.getId();
		FileDownloadProjectComposite composite;
		if (mapDownloading.containsKey(key)) {
			composite = (FileDownloadProjectComposite) mapDownloading.get(key);
		} else {
			composite = new FileDownloadProjectComposite(downloading, SWT.NONE);
			composite.addMainDownloadListener(new DownListener() {
				/**
				 * @Description: 下载成功
				 * @author chenxw
				 * @param tenderProjectEntity
				 * @param fileDownload
				 */
				public void downloadSuccess(final TenderProjectEntity tenderProject, final IFileDownload fileDownload) {
					if(!DownloadListener.SERVICE_HTTP_BID.equals(downloadType)) {
						synchronized (xmlFile) {
							if (!xmlFile.exists()) {
								Document document = DocumentHelper.createDocument();
								document.addElement("downloadeds");
								XmlUtils.createXmlFile(document, xmlFile.getAbsolutePath(), "UTF-8");
							}
							XmlUtils.updateXmlFile(xmlFile, new UpdateXml() {
								public void update(Element rootElement) {
									addDownloadedXml(rootElement, tenderProject, fileDownload);
								}
							});
						}
					}
					Display.getDefault().syncExec(new Runnable() {
						public void run() {
							removeIFileDownloadingProject(tenderProject, fileDownload);
							addFileDownloadedProject(tenderProject, fileDownload);
						}
					});
				}

				/**
				 * @Description: 取消下载
				 * @author chenxw
				 * @param tenderProjectEntity
				 * @param fileDownload
				 */
				public void cancelDownload(final TenderProjectEntity tenderProject, final IFileDownload fileDownload) {
					Display.getDefault().syncExec(new Runnable() {
						public void run() {
							removeIFileDownloadingProject(tenderProject, fileDownload);
						}
					});
				}
			});
			composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
			mapDownloading.put(key, composite);
		}
		composite.addIFileDownloading(tenderProject, supplier, BIDOPENING_STAGE, downloadType, executorService, downloadListener);
		downloading.layout();
		downloadingScrolledComposite.setMinSize(downloading.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	/**
	 * @Description: 初始化已下载
	 * @author chenxw 
	 */
//	private void initDownloaded() {
//		if (xmlFile.exists()) {
//			new Thread(new Runnable() {
//				public void run() {
//					Document document = XmlUtils.getXmlDocument(xmlFile.getAbsolutePath());
//					Element downloadedsElement = document.getRootElement();
//					final List<?> downloadedElementList = downloadedsElement.elements("downloaded");
//					Display.getDefault().syncExec(new Runnable() {
//						public void run() {
//							for (Object object : downloadedElementList) {
//								Element downloadedElement = (Element) object;
//								initFileDownloadedProject(downloadedElement);
//							}
//							downloaded.layout();
//							downloadedScrolledComposite.setMinSize(downloaded.computeSize(SWT.DEFAULT, SWT.DEFAULT));
//						}
//					});
//				}
//			}).start();
//		}
//	}
	
	/**
	 * @Description: 初始化已下载列表
	 * @author chenxw
	 * @param downloadedElement 
	 */
//	private void initFileDownloadedProject(Element downloadedElement) {
//		if (downloaded.isDisposed()) return;
//		String tenderId = downloadedElement.elementTextTrim("tenderId");
//		FileDownloadProjectComposite composite;
//		if (mapDownloaded.containsKey(tenderId)) {
//			composite = (FileDownloadProjectComposite) mapDownloaded.get(tenderId);
//		} else {
//			composite = new FileDownloadProjectComposite(downloaded, SWT.NONE);
//			composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
//			composite.addMainDownloadListener(mainDownListener);
//			mapDownloaded.put(tenderId, composite);
//		}
//		composite.initFileDownloaded(downloadedElement);
//	}

	/**
	 * @Description: 添加XML文件已下载信息
	 * @author chenxw
	 * @param rootElement
	 * @param tenderProjectEntity
	 * @param fileDownload 
	 */
	private void addDownloadedXml(Element rootElement, TenderProjectEntity tenderProject, IFileDownload fileDownload) {
		String supplierId = null != fileDownload.getSupplier() ? fileDownload.getSupplier().getId() : "";
		String bidopeningStage = null != fileDownload.getBidopeningStage() ? fileDownload.getBidopeningStage() : "";
		String fileName = null != fileDownload.getFileName() ? fileDownload.getFileName() : "";
		String fileSize = null != fileDownload.getFileSize() ? fileDownload.getFileSize() : "";
		String downloadTime = null != fileDownload.getDownloadTime() ? fileDownload.getDownloadTime() : "";
		String filePath = null != fileDownload.getBidFilePath() ? fileDownload.getBidFilePath() : "";
		List<?> downloadeds = rootElement.elements("downloaded");
		Element downloaded = null;
		for (Object object : downloadeds) {
			Element downloadedElement = (Element) object;
			if (tenderProject.getId().equals(downloadedElement.elementTextTrim("tenderId")) && fileName.equals(downloadedElement.elementTextTrim("fileName"))) {
				downloaded = downloadedElement;
			}
		}
		if (null != downloaded) {
			downloaded.element("fileSize").setText(fileSize);
			downloaded.element("downloadTime").setText(downloadTime);
		} else {
			downloaded = rootElement.addElement("downloaded");
			downloaded.addElement("tenderId").setText(tenderProject.getId());
			downloaded.addElement("tenderNo").setText(tenderProject.getTenderNo());
			downloaded.addElement("tenderName").setText(tenderProject.getTenderName());
			downloaded.addElement("supplierId").setText(supplierId);
			downloaded.addElement("bidopeningStage").setText(bidopeningStage);
			downloaded.addElement("fileName").setText(fileName);
			downloaded.addElement("fileSize").setText(fileSize);
			downloaded.addElement("downloadTime").setText(downloadTime);
			downloaded.addElement("filePath").setText(filePath);
		}
	}
	
	/**
	 * @Description: 去除正在下载界面列表数据
	 * @author chenxw
	 * @param tenderProjectEntity
	 * @param iFileDownload
	 */
	private void removeIFileDownloadingProject(TenderProjectEntity tenderProject, IFileDownload fileDownload) {
		if (downloading.isDisposed()) return;
		String key = tenderProject.getId();
		if (mapDownloading.containsKey(key)) {
			IFileDownloadProject composite = mapDownloading.get(key);
			composite.removeIFileDownload(fileDownload);
			boolean isStop = composite.isStop();
			if (isStop) {
				mapDownloading.remove(key);
			}
			if (0 == mapDownloading.size()) {
				((GridData) messagelabel.getLayoutData()).exclude = false;
				messagelabel.setVisible(true);
			}
			downloading.layout();
			downloadingScrolledComposite.setMinSize(downloading.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		}
	}

	/**
	 * @Description: 添加已下载界面列表数据
	 * @author chenxw
	 * @param tenderProjectEntity
	 * @param downloadComposites
	 */
	private void addFileDownloadedProject(TenderProjectEntity tenderProject, IFileDownload iFileDownload) {
		if (downloaded.isDisposed()) return;
		String key = tenderProject.getId();
		FileDownloadProjectComposite composite;
		if (mapDownloaded.containsKey(key)) {
			composite = (FileDownloadProjectComposite) mapDownloaded.get(key);
		} else {
			composite = new FileDownloadProjectComposite(downloaded, SWT.NONE);
			composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
			composite.addMainDownloadListener(mainDownListener);
			mapDownloaded.put(key, composite);
		}
		composite.addIFileDownloaded(tenderProject, iFileDownload);
		downloaded.layout();
		downloadedScrolledComposite.setMinSize(downloaded.computeSize(SWT.DEFAULT, SWT.DEFAULT));
	}

	/**
	 * @Description: 获得按钮
	 * @author chenxw
	 * @param string
	 * @param composite
	 * @return
	 */
	private ImageButton createImageButton(String string, final Composite downloadFrame) {
		final ImageButton button = new ImageButton(buttonComposite, SWT.NONE);
		button.setRightMargin(10);
		button.setLeftMargin(10);
		button.setTopMargin(5);
		button.setBottomMargin(5);
		button.setRightMargin(10);
		button.setFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.NORMAL));
		button.setForeground(ColorConstants.FONT_BLACK_COLOR);
		button.setActiveForeground(SWTResourceManager.getColor(255, 153, 0));
		button.setSelectionForeground(ColorConstants.FONT_BLUE_COLOR);
		// button.setForeground(ColorConstants.NOTE_TIPS_COLOR);
		// button.setBackgroundColor(SWTResourceManager.getColor(245, 245, 245));
		// button.setActiveForeground(ColorConstants.SKIN_ACTIVE_FOREGROUND);
		// button.setActiveBackgroundColor(ColorConstants.SKIN_ACTIVE_BG);
		// button.setSelectionForeground(ColorConstants.SKIN_SELECTOIN_FOREGROUND);
		// button.setSelectionBgColor(ColorConstants.SKIN_BG);
		button.setText(string);
		button.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				downloadButton.setSelection(false);
				downloadButton = button;
				downloadButton.setSelection(true);
				stackLayout.topControl = downloadFrame;
				downloadComposite.layout();
			}
		});
		return button;
	}

	/**
	 * @Description: 是否含有正在下载任务
	 * @author chenxw
	 * @return 
	 */
	public boolean isContainDownloadingTask() {
		return mapDownloading.size() > 0;
	}
	
	/**
	 * @Description: 关闭下载
	 * @author chenxw 
	 */
	public void closeDownloadTask() {
		for (String key : mapDownloading.keySet()) {
			IFileDownloadProject downloadProject = mapDownloading.get(key);
			downloadProject.closeDownloadTask();
		}
		close();
	}
	
	/**
	 * @Description: 添加下载完成监听
	 * @author chenxw
	 * @param downloadListener
	 */
	public void addDownloadListener(DownloadListener downloadListener) {
		this.downloadListener = downloadListener;
	}

	DownListener mainDownListener = new DownListener() {
		public void closeDownloaded(String tenderId, IFileDownload fileDownload) {
			if (mapDownloaded.containsKey(tenderId)) {
				IFileDownloadProject composite = mapDownloaded.get(tenderId);
				composite.removeIFileDownload(fileDownload);
				boolean isStop = composite.isStop();
				if (isStop) {
					mapDownloaded.remove(tenderId);
				}
			}
			downloaded.layout();
			downloadedScrolledComposite.setMinSize(downloaded.computeSize(SWT.DEFAULT, SWT.DEFAULT));
		};
	};
	
	public void close() {
		if (null != executorService) {
			executorService.shutdownNow();
		}
		super.close();
	}

	/**
	 * Create contents of the shell.
	 */
	protected void createContents() {
		setText(Messages.Button_download);
		setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/download.png"));
		setSize(800, 600);
	}

	protected void checkSubclass() {

	}

	public static class DownListener {

		/**
		 * @Description: 下载成功
		 * @author chenxw
		 * @param tenderProjectEntity
		 * @param iFileDownload
		 */
		public void downloadSuccess(TenderProjectEntity tenderProject, IFileDownload iFileDownload) {}

		/**
		 * @Description: 取消下载
		 * @author chenxw
		 * @param tenderProjectEntity
		 * @param fileDownload
		 */
		public void cancelDownload(TenderProjectEntity tenderProject, IFileDownload fileDownload) {}
		
		/**
		 * @Description: 关闭已下载记录
		 * @author chenxw
		 * @param tenderProjectEntity
		 * @param fileDownload
		 */
		public void closeDownloaded(String tenderId, IFileDownload fileDownload) {}
	}

}
