package com.gx.obe.common.file.thread;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.ConnectException;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.log4j.Logger;

import com.gx.obe.common.file.entity.BidFile;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.web.entity.RemoteFile;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.auth.AuthLoginRecordEntity;
import com.gx.obe.web.service.UserLoginRecordService;

/** 
 * @Description: 获取FTP服务器文件信息
 * @author chenxw 
 */
public class CheckFtpBidFileThread implements Runnable {

	private CheckCallBack checkCallBack;
	private Logger LOG = Logger.getLogger(CheckFtpBidFileThread.class);
	private Thread thread = null;
	private Supplier supplier;
	private FTPClient ftpClient = new FTPClient();
	private String hostname;
	private String username;
	private String password;
	private String remotedir;
	private String localDir;
	private long totalSize = 0;
	private String BIDOPENING_STAGE;

	public CheckFtpBidFileThread(Supplier supplier, String hostname, String username, String password, String remotedir, String localDir, String BIDOPENING_STAGE, CheckFtpBidFileThread.CheckCallBack checkCallBack) {
		this.supplier = supplier;
		this.hostname = hostname;
		this.username = username;
		this.password = password;
		this.remotedir = remotedir;
		this.localDir = localDir;
		this.BIDOPENING_STAGE = BIDOPENING_STAGE;
		this.checkCallBack = checkCallBack;
	}

	public void start() {
		thread = new Thread(this);
		thread.setName(supplier.getSupplierName());
		thread.start();
	}

	public void join() {
		if (null != thread) {
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}

	public void stop() {
		if (null != thread) {
			thread = null;
		}
	}

	public void run() {
		try {
			if (!connection()) {
				return;
			}
			List<RemoteFile> remoteFileList = getDownLoadFTPFiles(ftpClient, remotedir);
			checkCallBack.bidFile(totalSize, remoteFileList);
		} catch (Exception e) {
			LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
		} finally {
			if (ftpClient.isConnected()) {
				try {
					ftpClient.disconnect();
				} catch (IOException e) {
				}
			}
		}
	}

	/**
	 * @Description: 连接FTTP服务器
	 * @author chenxw
	 * @return 
	 */
	private boolean connection() {
		hostname = PropertiesUtils.getProperty(ConfigConstants.FTP_IP, "");
		int port = 21;
		try {
			port = Integer.parseInt(PropertiesUtils.getProperty(ConfigConstants.FTP_PORT, "21"));
		} catch (NumberFormatException e) {
		}

		if (null == hostname || hostname.length() == 0) {
			UserLoginRecordService service = new UserLoginRecordService();
			AuthLoginRecordEntity authLoginRecordEntity = service.getLoginRecordByTenderId(supplier.getTenderId());
			if (null != authLoginRecordEntity && null != authLoginRecordEntity.getIp() && authLoginRecordEntity.getIp().trim().length() > 0) {
				hostname = authLoginRecordEntity.getIp();
			}
		}

		if (null == hostname || hostname.length() == 0) {
			LOG.error("未设置FTP服务器IP");
		}
		boolean connection = false;
		try {
			//登录FTP服务器
			try {
				if (port > 0) {
					ftpClient.connect(hostname, port);
				} else {
					ftpClient.connect(hostname);
				}
				connection = true;
			} catch (ConnectException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			} catch (SocketException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			} catch (IOException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			}

			// 连接成功后登录FTP服务器
			if (connection) {
				//得到连接的返回编码
				if (!FTPReply.isPositiveCompletion(ftpClient.getReplyCode())) {
					ftpClient.disconnect();
				}
				//登录失败、权限验证失败
				if (ftpClient.login(username, password)) {
					//设置超时时间  
					ftpClient.setDataTimeout(7200);
					//设置默认编码
					ftpClient.setControlEncoding("GBK");
					//设置默认端口
					ftpClient.setDefaultPort(21);
					//设置是否显示隐藏文件
					ftpClient.setListHiddenFiles(false);
					// 设置被动模式
					ftpClient.enterLocalPassiveMode();
					// 设置以二进制方式传输
					ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
					//设置ftp工作目录
					ftpClient.changeWorkingDirectory(remotedir);

					LOG.info("成功登录FTP服务器：" + hostname + " 端口：" + port);
				} else {
					connection = false;
					//退出
					ftpClient.quit();
					//断开
					ftpClient.disconnect();

					checkCallBack.callBackString("连接FTP服务器用户名或密码错误！");

					LOG.error("连接FTP服务器用户名或密码错误：" + ftpClient.getReplyString());
				}
			} else {
				checkCallBack.callBackString("FTP服务器连接失败！");
			}
		} catch (IOException e) {
			checkCallBack.callBackString("FTP服务器连接失败！");
			LOG.error("登录FTP服务器失败：" + hostname + " 端口：" + port);
		}
		return connection;
	}

	/**
	 * @Description: 获取FTP服务器上要下载的文件
	 * @author chenxw
	 * @param ftp
	 * @param remoteDir
	 * @return 
	 */
	private List<RemoteFile> getDownLoadFTPFiles(FTPClient ftp, String remoteDir) {
		List<String> fileNames = getBidFileList();
		if (null == fileNames || fileNames.size() == 0) {
			return null;
		}
		List<RemoteFile> downLoadFTPFiles = new ArrayList<RemoteFile>();
		try {
			for (FTPFile ftpfile : ftp.listFiles()) {
				if (!fileNames.contains(ftpfile.getName())) {
					continue;
				}
				String remotePath = remoteDir.concat(File.separator).concat(ftpfile.getName());
				RemoteFile file = new RemoteFile();
				file.setPath(remotePath);
				file.setName(ftpfile.getName());
				if (ftpfile.isFile() && ftpfile.getSize() > 0) {
					totalSize += ftpfile.getSize();
					file.setSize(ftpfile.getSize());
					file.setIsDirectory(false);
					downLoadFTPFiles.add(file);
				} else if (ftpfile.isDirectory()) {
					file.setIsDirectory(true);
					downLoadFTPFiles.add(file);
					FTPFile[] listFile = ftp.listFiles(ftpfile.getName());
					if (null != listFile && listFile.length > 0) {
						for (FTPFile ftpFile : listFile) {
							RemoteFile subFile = new RemoteFile();
							subFile.setPath(file.getPath().concat(File.separator).concat(ftpFile.getName()));
							subFile.setName(ftpFile.getName());
							if (ftpFile.isFile() && ftpFile.getSize() > 0) {
								totalSize += ftpFile.getSize();
								subFile.setSize(ftpFile.getSize());
								subFile.setIsDirectory(false);
								downLoadFTPFiles.add(subFile);
							}
						}
					}
				}
			}
		} catch (IOException e) {
			LOG.error("获取ftp服务器文件失败！", e);
		}
		return downLoadFTPFiles;
	}

	/**
	 * @Description: FTP单文件下载
	 * @author chenxw
	 * @param ftpClient
	 * @param fileName
	 * @return 
	 */
	private boolean ftpDownload(FTPClient ftpClient, String fileName) {
		File file = new File(localDir.concat(File.separator).concat(remotedir).concat(File.separator).concat(fileName));
		if (file.exists()) {
			return true;
		}
		FileOutputStream out = null;
		try {
			if (!file.getParentFile().exists()) {
				file.getParentFile().mkdirs();
			}
			FTPFile[] ftpFiles = ftpClient.listFiles();
			for (FTPFile ftpFile : ftpFiles) {
				if (!fileName.equals(ftpFile.getName())) {
					continue;
				}
				if (!ftpFile.isFile() || ftpFile.getSize() == 0) {
					continue;
				}
				out = new FileOutputStream(file);
				ftpClient.retrieveFile(fileName, out);
				return true;
			}
		} catch (IOException e) {
			LOG.error("下载" + fileName + "失败", e);
		} finally {
			if (null != out) {
				try {
					out.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
		}
		return false;
	}

	/**
	 * @Description: 获取需下载文件文件名列表
	 * @author chenxw
	 * @return 
	 */
	private List<String> getBidFileList() {
		if (!ftpDownload(ftpClient, FileConstants.BID_FILE_INDEX_XML)) {
			return null;
		}
		return BidFile.getBidFileListByBidopeningStage(localDir.concat(File.separator).concat(remotedir).concat(File.separator), BIDOPENING_STAGE);
	}

	public static interface CheckCallBack {

		public void bidFile(long totalSize, List<RemoteFile> downLoadFTPFiles);

		public void callBackString(String returnMessage);

	}
}
