package com.gx.obe.common.file.thread;

import java.util.Date;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.TransactionServer;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierService;
import com.gx.obe.web.service.TenderProjectService;

import net.sf.json.JSONObject;

/** 
 * @Description: 检测投标文件是否最新线程
 * @author guoyr 
 */
public class CheckHttpBidFileThread implements Runnable{
	
	private TransactionServer transactionServer;
	private TenderProjectEntity parentProject;
	private TenderProjectEntity tenderProject;
	private Supplier supplier;
	private CheckCallBack checkCallBack;
	private TenderProjectService tenderProjectService ;
	private SupplierService supplierService = new SupplierService();
	private Thread thread = null;
	private Logger LOG = Logger.getLogger(CheckHttpBidFileThread.class);
	private boolean downloadDecryptFile = false;// 下载明文标书
	private String openStageType;
	
	public CheckHttpBidFileThread(TransactionServer transactionServer, TenderProjectEntity parentProject, TenderProjectEntity tenderProject, Supplier suppier, boolean downloadDecryptFile, String openStageType, CheckCallBack checkCallBack){
		this.transactionServer = transactionServer;
		this.parentProject = parentProject;
		this.tenderProject = tenderProject;
		this.supplier = suppier;
		this.checkCallBack = checkCallBack;
		this.downloadDecryptFile = downloadDecryptFile;
		this.openStageType = openStageType;
	}
	public CheckHttpBidFileThread(TenderProjectEntity tenderProject, Supplier suppier, CheckCallBack checkCallBack){
		this.tenderProject = tenderProject;
		this.supplier = suppier;
		this.checkCallBack = checkCallBack;
		if(null != tenderProject.getTransactionServerId()){
			transactionServer = HttpUtils.getTransactionServer(tenderProject.getTransactionServerId());
		}
		if(null != transactionServer && null != tenderProject.getParentId()){
			if(null == tenderProjectService){
				tenderProjectService = new TenderProjectService();
			}
			parentProject = tenderProjectService.getById(tenderProject.getParentId());
		}
	}
	public void start(){
		thread = new Thread(this);
		thread.setName(supplier.getSupplierName());
		thread.start();
	}
	public void join(){
		if(null != thread){
			try {
				thread.join();
			} catch (InterruptedException e) {
			}
		}
	}
	public void stop(){
		if(null != thread){
			thread = null;
		}
	}
	public void run(){
		if(null == transactionServer){
			checkCallBack.callBackString(Messages.Tips_FailedToConnectServer);
			return ;
		}
		HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProject, parentProject, transactionServer, PropertiesUtils.getWebserviceProperty(WebserviceConstants.DOWN_LOAD_BID_FILE_METHOD, ""));
		if(null != httpUrl){
			httpUrl.setAttribute("orgCode", supplier.getOrgCode());
		}
		httpUrl.setAttribute("mainId", null != supplier.getOrgId() ? supplier.getOrgId() : "");
//		是否下载解密文件[00否01是 默认00]
		httpUrl.setAttribute("isDownDecrypt", downloadDecryptFile ? "01" : "00");
		httpUrl.setAttribute("openStageType", openStageType);
		httpUrl.setAttribute("mainId", null != supplier.getOrgId() ? supplier.getOrgId() : "");
		HttpResponse httpResponse = HttpUtils.doPost(httpUrl);
		if (httpResponse.getStatus()) {
			Date tenderTime = null;
			long bidFileSize = 0;
			String downUrl = null;
			boolean hasbidFile = false;
			tenderTime = DateUtils.toDate(httpResponse.getDataParam("tenderTime"));
			if (null != httpResponse.getDataParam("fileSize") && !"".equals(httpResponse.getDataParam("fileSize"))) {
				try {
					bidFileSize = (Long) Long.parseLong((String)httpResponse.getDataParam("fileSize"));
				} catch (Exception e) {
				}
			}
			downUrl = httpResponse.getDataParam("downUrl");
			if(null != tenderTime && bidFileSize > 0 && null != downUrl && downUrl.length() > 0){
				hasbidFile = true;
			}
			supplier.setTenderIP(ObjectUtils.getObjString(httpResponse.getDataParam("tenderIp")));
			supplierService.updateAssignProperty(supplier, new String[]{"tenderIP"});
			
			if(!hasbidFile){
				LOG.error(supplier.getSupplierName() + "未获取到标书信息："+httpUrl.getUrl() + httpUrl.getAttributs() + "	服务器内容："+httpResponse.getContent());
			}
			checkCallBack.bidFile(hasbidFile, tenderTime, bidFileSize, downUrl);
		}else{
			checkCallBack.callBackString(httpResponse.getMsg());
			LOG.error(supplier.getSupplierName() + "未获取到标书信息："+httpUrl.getUrl() + httpUrl.getAttributs() + "	服务器内容："+httpResponse.getContent());
			return ;
		}
	}
	public static interface CheckCallBack{
		public void bidFile(boolean hasbidFile, Date tenderTime, long bidFileSize, String downUrl);
		public void callBackString(String returnMessage);
	}
}
