package com.gx.obe.common.file.thread;

import java.io.File;
import java.util.Objects;
import java.util.Optional;
import java.util.function.Function;

import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.enumeration.FileTypeEnum;
import com.gx.obe.common.file.listener.ClearProjectFileListener;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.web.entity.tender.TenderProjectEntity;

public class ClearProjectFileThread implements Runnable {
	
	private TenderProjectEntity tenderProject;
	private int fileType;
	
	private ClearProjectFileListener listener;
	
	public ClearProjectFileThread(TenderProjectEntity tenderProject, int fileType) {
		this.tenderProject = tenderProject;
		this.fileType = fileType;
	}
	
	@Override
	public void run() {
		if ((FileTypeEnum.BIDDING.getKey() & fileType) != 0) {
			deleteProjectFile(FileConstants::getBiddingFilePath);
		}
		if ((FileTypeEnum.BID.getKey() & fileType) != 0) {
			deleteProjectFile(FileConstants::getBidFilePath);
		}
		if ((FileTypeEnum.ANSWER.getKey() & fileType) != 0) {
			deleteProjectFile(FileConstants::getAnswerQuestionFilePath);
		}
		if ((FileTypeEnum.REPORT.getKey() & fileType) != 0) {
			deleteProjectFile(FileConstants::getReportFilePath);
		}
		Optional.ofNullable(listener).ifPresent(ClearProjectFileListener::finish);
	}
	
	private void deleteProjectFile(Function<String, String> getFilePathFun) {
		boolean success = false;
		try {
			if (Objects.isNull(getFilePathFun)) {
				return;
			}
			File file = Optional.ofNullable(tenderProject).map(TenderProjectEntity::getId).map(getFilePathFun::apply).map(File::new).orElse(null);
			if (Objects.isNull(file)) {
				return;
			}
			if (!file.exists()) {
				return;
			}
			FileUtils.delFolder(file.getAbsolutePath());
			success = true;
		} finally {
			if (success) {}
		}
	}
	
	public void addClearProjectFileListener(ClearProjectFileListener listener) {
		this.listener = listener;
	}
	
}
