package com.gx.obe.common.file.thread;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import org.apache.log4j.Logger;
import org.eclipse.swt.core.CallBack;

import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.http.HttpURLDown;
import com.gx.obe.common.file.utils.ZipUtils;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierService;
import com.gx.utils.NowTimeUtils;

/**
 * @Description: 下载投标文件
 * @author guoyr
 */
public class DownloadBidFileThread implements Runnable{
	
	private CallBack callBack;
	private Logger LOG = Logger.getLogger(DownloadBidFileThread.class);	
	private SupplierService supplierService = new SupplierService();
	private TenderProjectEntity tenderProjectEntity;
	private List<Supplier> supplierList;
	private Thread thread;
	private ExecutorService executorService;
	private boolean downloadDecryptFile = false;// 下载明文标书
	public DownloadBidFileThread(TenderProjectEntity tenderProjectEntity, List<Supplier> supplierList, boolean downloadDecryptFile, CallBack callBack){
		this.callBack = callBack;
		this.tenderProjectEntity = tenderProjectEntity;
		this.supplierList = supplierList;
		this.downloadDecryptFile = downloadDecryptFile;
	}
	public Thread getThread(){
		return thread;
	}
	public void start(){
		thread = new Thread(this);
		thread.setName("下载投标文件");
		thread.start();
	}
	public void stop(){
		thread = null;
		if(null != executorService){
			executorService.shutdownNow();
		}
		executorService = null;
	}
	public void join(){
		try {
			thread.join();
		} catch (InterruptedException e) {
		}
	}
	public void run() {
		
		if(null != supplierList && supplierList.size() > 0){
			
			long start = Calendar.getInstance().getTimeInMillis();
			final String downLoadPath = FileConstants.getBidFilePath(tenderProjectEntity.getId());
			
			List<Future<String>> resultList = new ArrayList<Future<String>>();  
			// 分配线程去下载，这里用到线程池
			executorService = Executors.newFixedThreadPool(supplierList.size() < 5 ? supplierList.size() : 5);
			final List<String> downFaileList = new ArrayList<String>();
			final List<String> downSuccessList = new ArrayList<String>();
			int supplierIndex = 0;
			for(final Supplier supplier : supplierList){
				Future<String> future = executorService.submit(new Callable<String>(){
					public String call(){
						boolean downloadFlag = false;
						String donwUrl = supplier.getApplyUrl();
						supplier.setBidFileType(SupplierEnum.ONLINE_ELECTRONIC);
						String saveDir = downLoadPath.concat(supplier.getId()).concat(File.separator);
						// 加密标书下载前先将目录清空，重新下载
						if(!downloadDecryptFile){
							FileUtils.delAllFile(saveDir);
						}
						downloadFlag = downLoadFile(supplier, donwUrl, saveDir.concat(FileConstants.BID_FILE).concat(".zip"));
						if(downloadFlag){
							// 下载成功后重新记录当前下载标书的投递时间，并记录下载时间
							supplier.setBidFileDownloadTime(NowTimeUtils.getNowDate());
							supplierService.updateAssignProperty(supplier, new String[]{"tenderTime", "bidFileDownloadTime", "bidFileType"});
							downSuccessList.add(supplier.getSupplierName());
							return "√"+supplier.getSupplierName() + "的标书下载成功！";
						}else {
							supplier.setBidFileDownloadTime(null);
							supplierService.updateAssignProperty(supplier, new String[]{"bidFileDownloadTime", "bidFileType"});
							downFaileList.add(supplier.getSupplierName());
							return "×"+supplier.getSupplierName() + "的标书下载失败！";
						}
					}
				});
				resultList.add(future);
				callBack.callBackInt(supplierIndex ++);
			}
			
			executorService.shutdown();
			int downCount = 0;
			int supplierCount = supplierList.size();
			for (Future<String> fs : resultList) {  
				try {  
					fs.get();
				} catch (InterruptedException e) {  
				} catch (ExecutionException e) {  
				} finally{
					downCount ++;
					callBack.callBackInt((int)((downCount/(double)supplierCount) *100));
				}
			}  
			callBackMsg(" \r\n\r\n下载结果如下：");
			int i = 1;
			for(String supplierName : downSuccessList){
				callBackMsg(true, (i++)+"、"+supplierName + "的标书下载成功！");
			}
			i = 1;
			for(String supplierName : downFaileList){
				callBackMsg(false, (i++)+"、"+supplierName + "的标书下载失败！");
			}
			callBackMsg("\r\n\r\n此次下载结束,共用时" +DateUtils.timeSpan(Calendar.getInstance().getTimeInMillis()-start));
		}else{
			callBackMsg(false, tenderProjectEntity.getTenderName()+"下没有可下载的"+FileConstants.TBWJ_DOC+"！");
		}
		if(null != callBack){
			callBack.after();
		}
	}

	/**
	 * @Description: 下载投标文件
	 * @author guoyr
	 * @param downLoadUrl
	 * @param savePath
	 * @return
	 */
	private boolean downLoadFile(Supplier supplier, String downLoadUrl, String savePath){
		boolean downFlag = false;
		if(null != downLoadUrl && downLoadUrl.length() > 0){
			HttpURLDown httpURLDown = new HttpURLDown(downLoadUrl, savePath, callBack);
			File downloadZipFile = new File(savePath);
			
			try {
				downFlag = httpURLDown.downLoad(supplier.getSupplierName());
				if(downFlag){
					downFlag = unZipFile(downloadZipFile.getAbsolutePath(), downloadZipFile.getParent());
					if(!downFlag){
						callBackMsg(false, supplier.getSupplierName() + "的标书解压失败，请手动解压！");
					}else {
						downloadZipFile.delete();
					}
				}
			} catch (IOException e) {
			}finally{
			}
		}else {
			callBackMsg(false, supplier.getSupplierName()+"的下载地址无效...");
		}
		return downFlag;
	}
	
	/**
	 * @Description: 解压投标文件
	 * @author guoyr
	 * @param archive
	 * @param decompressDir
	 * @return
	 */
	private boolean unZipFile(String archive, String decompressDir){
		boolean unZipFlag = false;
		unZipFlag = ZipUtils.unZip(archive, decompressDir, true, true, true,callBack);
		return unZipFlag;
	}
	
	private void callBackMsg(String msg){
		if(null != callBack){
			callBack.callBackString("  "+msg);
		}
	}
	private void callBackMsg(boolean bool, String msg){
		if(null != callBack){
			callBack.callBackString((bool?"√":"×")+ msg);
		}
	}

}
