package com.gx.obe.common.file.thread;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.ConnectException;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.log4j.Logger;

import com.gx.obe.common.file.download.infer.FileDownloadListener;
import com.gx.obe.common.file.download.infer.FileDownloadListener.DOWN_STATUS;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.inter.FileDownloadThreadInterface;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.web.entity.RemoteFile;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.auth.AuthLoginRecordEntity;
import com.gx.obe.web.service.UserLoginRecordService;
import com.gx.security.SystemTool;

/**
 * @Description: Ftp下载投标文件
 * @author chenxw
 */
public class FtpBidFileDownloadThread implements Runnable, FileDownloadThreadInterface {

	private Supplier supplier;
	private FileDownloadListener listener;
	private DOWN_STATUS downloadStatus = DOWN_STATUS.WAITE;
	private boolean isDeleteSourceFile = false;
	private FTPClient ftpClient = new FTPClient();
	private static String DEAFULT_REMOTE_CHARSET = "GBK";
	private static int DEAFULT_REMOTE_PORT = 21;// 默认端口
	private Logger LOG = Logger.getLogger(FtpBidFileDownloadThread.class);
	private List<File> deleteSourceFileList = new ArrayList<File>();

	public FtpBidFileDownloadThread(Supplier supplier, FileDownloadListener listener) {
		this.supplier = supplier;
		this.listener = listener;
	}

	public void run() {
		try {
			startDownload();
			if (connection()) {
				String localDir = FileConstants.BID_FILE_PATH;
				List<RemoteFile> remoteFileList = supplier.getDownLoadFTPFiles();
				listener.setTotalSize(supplier.getTotalSize());
				for (final RemoteFile remoteFile : remoteFileList) {
					if (downloadStatus == DOWN_STATUS.CANCEL) {
						break;
					}
					if (downloadStatus == DOWN_STATUS.FAIL) {
						break;
					}
					if (remoteFile.isDirectory()) {
						File dir = new File(localDir.concat(File.separator).concat(remoteFile.getPath()));
						if (!dir.exists()) {
							dir.mkdirs();
						}
						continue;
					}
					if (!download(ftpClient, remoteFile, remoteFile.getPath(), localDir + File.separator + remoteFile.getPath())) {
						downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
					}
				}
				if (downloadStatus == DOWN_STATUS.START) {
					downloadStatus = DOWN_STATUS.SUCCESS;
				}
			} else {
				downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			}
		} catch (Exception e) {
			downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			LOG.error("FTP下载失败！", e);
		} finally {
			try {
				logOut();
			} catch (IOException e) {
				LOG.error("FTP注销失败！", e);
			}
			listener.setDownloadStatus(downloadStatus);
			if (isDeleteSourceFile) {
				deleteDownloadFile();
			}
		}
	}

	/**
	 * @Description: 注销Ftp
	 * @author guoyr
	 * @throws IOException
	 */
	private void logOut() throws IOException {
		if (null != ftpClient) {
			// FTP退出
			ftpClient.logout();
			// 断开FTP连接
			if (ftpClient.isConnected()) {
				ftpClient.disconnect();
			}
		}
	}

	/**
	 * @Description: 从FTP服务器上下载文件,支持断点续传，上传百分比汇报
	 * @author guoyr
	 * @param ftpClient
	 * @param remoteFile远程文件
	 * @param remoteFilePath 远程文件路径
	 * @param localFilePath 远程文件路径
	 * @return
	 */
	private boolean download(FTPClient ftpClient, RemoteFile remoteFile, String remoteFilePath, String localFilePath) {
		boolean download = false;
		FileOutputStream out = null;
		InputStream in = null;
		// 本地存在文件，进行断点下载
		boolean isDdxc = false;
		try {
			long lRemoteSize = remoteFile.getSize();
			File localFile = new File(localFilePath);
			if (localFile.exists()) {
				long localSize = localFile.length();
				// 判断本地文件大小是否大于远程文件大小
				if (localSize == lRemoteSize) {
					listener.hisDownloadSize(localSize);
					LOG.debug(localFile.getName() + "已存在！");
					download = true;
					return download;
				} else {
					listener.hisDownloadSize(localSize);
					// 进行断点续传，并记录状态
					isDdxc = true;
					ftpClient.setRestartOffset(localSize);
				}
			} else {
				if (!localFile.getParentFile().exists()) {
					localFile.getParentFile().mkdirs();
				}
			}
			deleteSourceFileList.add(localFile);
			byte[] bytes = new byte[1024];
			in = ftpClient.retrieveFileStream(remoteFilePath);
			if (null != in) {
				out = new FileOutputStream(localFile, isDdxc);
				int c;
				while (downloadStatus == DOWN_STATUS.START && (c = in.read(bytes)) != -1) {
					out.write(bytes, 0, c);
					listener.writeSize(c);
					// 暂停下载
					while (downloadStatus == DOWN_STATUS.PAUSE) {
						Thread.sleep(1000);
					}
					// 停止下载
					if (downloadStatus == DOWN_STATUS.CANCEL || downloadStatus == DOWN_STATUS.FAIL) {
						break;
					}
				}
				if (downloadStatus == DOWN_STATUS.START) {
					download = ftpClient.completePendingCommand();
				}
			}
		} catch (IOException e) {
			download = false;
			LOG.error(remoteFile.getPath() + "下载失败！", e);
		} catch (InterruptedException e) {
			download = false;
			LOG.error(remoteFile.getPath() + "下载失败！", e);
		} finally {
			if (null != in) {
				try {
					in.close();
				} catch (IOException e) {
					LOG.error("", e);
				}
			}
			if (null != out) {
				try {
					out.close();
				} catch (IOException e) {
					LOG.error("", e);
				}
			}
		}
		return download;
	}

	/**
	 * @Description: 连接FTP服务器
	 * @author chenxw
	 * @return
	 */
	private boolean connection() {
		boolean connection = false;
		String username = "anonymous";
		String password = null;
		int port = 21;
		String hostname = PropertiesUtils.getProperty(ConfigConstants.FTP_IP, "");
		try {
			port = Integer.parseInt(PropertiesUtils.getProperty(ConfigConstants.FTP_PORT, "21"));
		} catch (NumberFormatException e) {
		}
		
		if(null == hostname || hostname.length() == 0){
			UserLoginRecordService service = new UserLoginRecordService();
			AuthLoginRecordEntity authLoginRecordEntity = service.getLoginRecordByTenderId(supplier.getTenderId());
			if(null != authLoginRecordEntity && null != authLoginRecordEntity.getIp() && authLoginRecordEntity.getIp().trim().length() > 0){
				hostname = authLoginRecordEntity.getIp();
			}
		}
		username = PropertiesUtils.getProperty(ConfigConstants.FTP_USERNAME, username);
		password = PropertiesUtils.getProperty(ConfigConstants.FTP_PASSWORD, password);
		if (hostname.equals(SystemTool.getIp())) {
			return false;
		}
		try {
			// 登录FTP服务器
			try {
				// 设置超时时间
				ftpClient.setDataTimeout(7200);
				// 设置默认编码
				ftpClient.setControlEncoding(DEAFULT_REMOTE_CHARSET);
				// 设置默认端口
				ftpClient.setDefaultPort(DEAFULT_REMOTE_PORT);
				// 设置是否显示隐藏文件
				ftpClient.setListHiddenFiles(false);
				// 连接ftp服务器
				if (port > 0) {
					ftpClient.connect(hostname, port);
				} else {
					ftpClient.connect(hostname);
				}
				connection = true;
			} catch (ConnectException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			} catch (SocketException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			} catch (IOException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			}
			// 连接成功后登录FTP服务器
			if (connection) {
				// 得到连接的返回编码
				int reply = ftpClient.getReplyCode();
				if (!FTPReply.isPositiveCompletion(reply)) {
					ftpClient.disconnect();
				}
				// 登录失败权限验证失败
				if (!ftpClient.login(username, password)) {
					ftpClient.quit();
					ftpClient.disconnect();
					LOG.error("连接FTP服务器用户或者密码失败：" + ftpClient.getReplyString());
					connection = false;
				}
				if (LOG.isDebugEnabled()) {
					LOG.info("成功登录FTP服务器：" + hostname + " 端口：" + port);
				}
				ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
				// 设置被动模式
				ftpClient.enterLocalPassiveMode();
				// 设置以二进制方式传输
				ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
			} else {
				connection = false;
			}
		} catch (IOException e) {
			connection = false;
			LOG.error("登录FTP服务器失败：" + hostname + " 端口：" + port);
		}
		return connection;
	}

	private void startDownload() {
		downloadStatus = DOWN_STATUS.START;
		if (null != listener) {
			listener.setDownloadStatus(DOWN_STATUS.START);
		}
	}

	public void start() {
		new Thread(this).start();
	}

	public void setDownloadStatus(DOWN_STATUS downloadStatus) {
		this.downloadStatus = downloadStatus;
		if (null != listener) {
			listener.setDownloadStatus(downloadStatus);
		}
	}

	public void initDownload() {
		ftpClient = new FTPClient();
		downloadStatus = DOWN_STATUS.WAITE;
		isDeleteSourceFile = false;
		deleteSourceFileList.clear();
		if (null != listener) {
			listener.reset();
		}
	}

	public void setDeleteDownloadFile(boolean isDelete) {
		this.isDeleteSourceFile = isDelete;
	}

	public void deleteDownloadFile() {
		for (File deleteSourceFile : deleteSourceFileList) {
			if (null != deleteSourceFile && deleteSourceFile.exists()) {
				deleteSourceFile.delete();
			}
		}
	}

	public Runnable getRunnable() {
		return this;
	}
	
}
