package com.gx.obe.common.file.thread;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.ConnectException;
import java.net.SocketException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.log4j.Logger;

import com.gx.obe.common.file.download.infer.FileDownloadListener;
import com.gx.obe.common.file.download.infer.FileDownloadListener.DOWN_STATUS;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.inter.FileDownloadThreadInterface;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.web.entity.auth.AuthLoginRecordEntity;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.UserLoginRecordService;
import com.gx.security.SystemTool;

public class FtpBiddingFileDownloadThread implements Runnable, FileDownloadThreadInterface {

	private Logger LOG = Logger.getLogger(FtpBidFileDownloadThread.class);
	private TenderProjectEntity tenderProject;
	private FileDownloadListener listener;
	private DOWN_STATUS downloadStatus = DOWN_STATUS.WAITE;
	private FTPClient ftpClient = new FTPClient();
	private boolean isDeleteSourceFile = false;
	private List<File> deleteSourceFileList = new ArrayList<File>();

	public FtpBiddingFileDownloadThread(TenderProjectEntity tenderProject, FileDownloadListener listener) {
		this.tenderProject = tenderProject;
		this.listener = listener;
	}

	public void run() {
		try {
			startDownload();
			if (connection()) {
				String remoteDir = tenderProject.getId().concat(File.separator).concat(FileConstants.BIDDING_FILE_FOLDER).concat(File.separator);
				String localDir = FileConstants.getBiddingFilePath(tenderProject.getId());
//				String localDir = "C:/Users/xwchen/Desktop/";
				FTPFile[] ftpFiles = ftpClient.listFiles(remoteDir);
				if (null != ftpFiles && ftpFiles.length > 0) {
					long totalSize = 0;
					for (FTPFile ftpFile : ftpFiles) {
						totalSize += ftpFile.getSize();
					}
					listener.setTotalSize(totalSize);
					for (FTPFile ftpFile : ftpFiles) {
						if (downloadStatus == DOWN_STATUS.CANCEL) {
							break;
						}
						if (downloadStatus == DOWN_STATUS.FAIL) {
							break;
						}
						if (!download(ftpClient, ftpFile, remoteDir, localDir.concat(ftpFile.getName()))) {
							downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
						}
					}
				}
				if (downloadStatus == DOWN_STATUS.START) {
					downloadStatus = DOWN_STATUS.SUCCESS;
				}
			} else {
				downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			}
		} catch (Exception e) {
			downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			LOG.error("FTP下载失败！", e);
		} finally {
			try {
				logOut();
			} catch (IOException e) {
				LOG.error("FTP注销失败！", e);
			}
			listener.setDownloadStatus(downloadStatus);
			if (isDeleteSourceFile) {
				deleteDownloadFile();
			}
		}
	}

	/**
	 * @Description: 从FTP服务器上下载文件,支持断点续传，上传百分比汇报
	 * @author guoyr
	 * @param ftpClient
	 * @param remoteFile远程文件
	 * @param remoteFilePath
	 *            远程文件路径
	 * @param localFilePath
	 *            远程文件路径
	 * @return
	 */
	private boolean download(FTPClient ftpClient, FTPFile ftpFile, String remoteDir, String localFilePath) {
		boolean download = false;
		FileOutputStream out = null;
		InputStream in = null;
		// 本地存在文件，进行断点下载
		boolean isDdxc = false;
		try {
			File localFile = new File(localFilePath);
			if (localFile.exists()) {
				long ftpFileSize = ftpFile.getSize();
				long localSize = localFile.length();
				// 判断本地文件大小是否大于远程文件大小
				if (localSize == ftpFileSize) {
					listener.hisDownloadSize(localSize);
					return true;
				} else {
					listener.hisDownloadSize(localSize);
					// 进行断点续传，并记录状态
					isDdxc = true;
					ftpClient.setRestartOffset(localSize);
				}
			} else {
				if (!localFile.getParentFile().exists()) {
					localFile.getParentFile().mkdirs();
				}
			}
			deleteSourceFileList.add(localFile);
			byte[] bytes = new byte[1024];
			in = ftpClient.retrieveFileStream(remoteDir.concat(ftpFile.getName()));
			if (null != in) {
				out = new FileOutputStream(localFile, isDdxc);
				int c;
				while (downloadStatus == DOWN_STATUS.START && (c = in.read(bytes)) != -1) {
					out.write(bytes, 0, c);
					listener.writeSize(c);
					// 暂停下载
					while (downloadStatus == DOWN_STATUS.PAUSE) {
						Thread.sleep(1000);
					}
					// 停止下载
					if (downloadStatus == DOWN_STATUS.CANCEL || downloadStatus == DOWN_STATUS.FAIL) {
						break;
					}
				}
				if (downloadStatus == DOWN_STATUS.START) {
					download = ftpClient.completePendingCommand();
				}
			}
		} catch (IOException e) {
		} catch (InterruptedException e) {
		} finally {
			if (null != in) {
				try {
					in.close();
				} catch (IOException e) {
					LOG.error("", e);
				}
			}
			if (null != out) {
				try {
					out.close();
				} catch (IOException e) {
					LOG.error("", e);
				}
			}
		}
		return download;
	}

	/**
	 * @Description: 连接FTP服务器
	 * @author chenxw
	 * @return
	 */
	private boolean connection() {
		boolean connection = false;
		String username = "anonymous";
		String password = null;
		int port = 21;
		String hostname = PropertiesUtils.getProperty(ConfigConstants.FTP_IP, "");
		try {
			port = Integer.parseInt(PropertiesUtils.getProperty(ConfigConstants.FTP_PORT, "21"));
		} catch (NumberFormatException e) {
		}

		if (null == hostname || hostname.length() == 0) {
			UserLoginRecordService service = new UserLoginRecordService();
			AuthLoginRecordEntity authLoginRecordEntity = service.getLoginRecordByTenderId(tenderProject.getId());
			if (null != authLoginRecordEntity && null != authLoginRecordEntity.getIp() && authLoginRecordEntity.getIp().trim().length() > 0) {
				hostname = authLoginRecordEntity.getIp();
			}
		}
		username = PropertiesUtils.getProperty(ConfigConstants.FTP_USERNAME, username);
		password = PropertiesUtils.getProperty(ConfigConstants.FTP_PASSWORD, password);
		if (hostname.equals(SystemTool.getIp())) {
			//			return false;
		}
		try {
			// 登录FTP服务器
			try {
				// 设置超时时间
				ftpClient.setDataTimeout(7200);
				// 设置默认编码
				ftpClient.setControlEncoding("GBK");
				// 设置默认端口
				ftpClient.setDefaultPort(21);
				// 设置是否显示隐藏文件
				ftpClient.setListHiddenFiles(false);
				// 连接ftp服务器
				if (port > 0) {
					ftpClient.connect(hostname, port);
				} else {
					ftpClient.connect(hostname);
				}
				connection = true;
			} catch (ConnectException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			} catch (SocketException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			} catch (IOException e) {
				LOG.error("FTP服务器连接失败：" + ftpClient.getReplyString() + ftpClient.getReplyCode(), e);
			}
			// 连接成功后登录FTP服务器
			if (connection) {
				// 得到连接的返回编码
				int reply = ftpClient.getReplyCode();
				if (!FTPReply.isPositiveCompletion(reply)) {
					ftpClient.disconnect();
				}
				// 登录失败权限验证失败
				if (!ftpClient.login(username, password)) {
					ftpClient.quit();
					ftpClient.disconnect();
					LOG.error("连接FTP服务器用户或者密码失败：" + ftpClient.getReplyString());
					connection = false;
				}
				if (LOG.isDebugEnabled()) {
					LOG.info("成功登录FTP服务器：" + hostname + " 端口：" + port);
				}
				ftpClient.setFileType(FTPClient.BINARY_FILE_TYPE);
				// 设置被动模式
				ftpClient.enterLocalPassiveMode();
				// 设置以二进制方式传输
				ftpClient.setFileType(FTP.BINARY_FILE_TYPE);
			} else {
				connection = false;
			}
		} catch (IOException e) {
			connection = false;
			LOG.error("登录FTP服务器失败：" + hostname + " 端口：" + port);
		}
		return connection;
	}

	/**
	 * @Description: 注销Ftp
	 * @author guoyr
	 * @throws IOException
	 */
	private void logOut() throws IOException {
		if (null != ftpClient) {
			// FTP退出
			ftpClient.logout();
			// 断开FTP连接
			if (ftpClient.isConnected()) {
				ftpClient.disconnect();
			}
		}
	}

	/** 
	 * @Description: 启动
	 * @author chenxw 
	 */
	public void start() {
		new Thread(this).start();
	}

	/**
	 * @Description: 开始下载
	 * @author chenxw 
	 */
	private void startDownload() {
		downloadStatus = DOWN_STATUS.START;
		if (null != listener) {
			listener.setDownloadStatus(DOWN_STATUS.START);
		}
	}
	
	/** 
	 * @Description: 设置下载状态
	 * @author chenxw
	 * @param downloadStatus 
	 */
	public void setDownloadStatus(DOWN_STATUS downloadStatus) {
		this.downloadStatus = downloadStatus;
		if (null != listener) {
			listener.setDownloadStatus(downloadStatus);
		}
	}

	/** 
	 * @Description: 初始化下载
	 * @author chenxw 
	 */
	public void initDownload() {
		ftpClient = new FTPClient();
		downloadStatus = DOWN_STATUS.WAITE;
		isDeleteSourceFile = false;
		deleteSourceFileList.clear();
		if (null != listener) {
			listener.reset();
		}
	}

	/** 
	 * @Description: 设置是否删除已下载文件
	 * @author chenxw
	 * @param isDelete 
	 */
	public void setDeleteDownloadFile(boolean isDelete) {
		this.isDeleteSourceFile = isDelete;
	}

	public void deleteDownloadFile() {
		for (File deleteSourceFile : deleteSourceFileList) {
			if (null != deleteSourceFile && deleteSourceFile.exists()) {
				deleteSourceFile.delete();
			}
		}
	}

	public Runnable getRunnable() {
		return this;
	}

}
