package com.gx.obe.common.file.thread;

import java.io.File;
import java.io.FilenameFilter;

import org.apache.log4j.Logger;

import com.gx.obe.business.enumeration.SupplierEnum;
import com.gx.obe.common.file.download.infer.FileDownloadListener;
import com.gx.obe.common.file.download.infer.FileDownloadListener.DOWN_STATUS;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.http.HttpURLDownload;
import com.gx.obe.common.file.inter.FileDownloadThreadInterface;
import com.gx.obe.common.file.utils.ZipUtils;
import com.gx.obe.components.core.enumeration.TenderProjectEnum;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.Supplier;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.SupplierService;
import com.gx.utils.NowTimeUtils;

/**
 * @Description: Http下载投标文件
 * @author chenxw
 */
public class HttpBidFileDownloadThread implements Runnable, FileDownloadThreadInterface {

	private DOWN_STATUS downloadStatus = DOWN_STATUS.WAITE;
	private FileDownloadListener listener;
	private boolean isDeleteSourceFile = false;
	private Supplier supplier;
	private SupplierService supplierService = new SupplierService();
	private Logger LOG = Logger.getLogger(HttpBidFileDownloadThread.class);
	private HttpURLDownload download;
	private String BIDOPENING_STAGE;
	private TenderProjectEntity tenderProject;

	public HttpBidFileDownloadThread(TenderProjectEntity tenderProject, Supplier supplier, String BIDOPENING_STAGE, FileDownloadListener listener) {
		this.tenderProject = tenderProject;
		this.supplier = supplier;
		this.listener = listener;
		this.BIDOPENING_STAGE = BIDOPENING_STAGE;
	}

	public void run() {
		try {
			startDownload();
			boolean downloadFlag = false;
			String donwUrl = supplier.getApplyUrl();
			supplier.setBidFileType(SupplierEnum.ONLINE_ELECTRONIC);
			BIDOPENING_STAGE = null != BIDOPENING_STAGE ? BIDOPENING_STAGE : TenderProjectEnum.STAGE_ALL;
			String tempdSavePath = FileConstants.TEMP_FOLDER.concat("bidFile").concat(File.separator).concat(supplier.getTenderId()).concat(File.separator).concat(supplier.getId()).concat(File.separator).concat(BIDOPENING_STAGE).concat(File.separator).concat(FileConstants.BID_FILE).concat(".zip");
			String downLoadPath = FileConstants.getBidFilePath(supplier.getTenderId(), supplier.getId());
			downloadFlag = downLoadFile(donwUrl, supplier.getFileSize(), tempdSavePath, downLoadPath);
			if (downloadFlag) {
				supplier.setBidFileDownloadTime(NowTimeUtils.getNowDate());
				supplierService.updateAssignProperty(supplier, new String[] { "tenderTime", "bidFileDownloadTime", "bidFileType" });
				downloadStatus = DOWN_STATUS.SUCCESS;
			} else {
				supplier.setBidFileDownloadTime(null);
				supplierService.updateAssignProperty(supplier, new String[] { "bidFileDownloadTime", "bidFileType" });
				downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			}
		} catch (Exception e) {
			downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			LOG.error("FTP下载失败！", e);
		} finally {
			listener.setDownloadStatus(downloadStatus);
		}
	}

	/**
	 * @Description: 下载文件
	 * @author chenxw
	 * @param downLoadUrl
	 * @param savePath
	 * @return
	 */
	private boolean downLoadFile(String downLoadUrl, long fileSize, String tempdSavePath, String savePath) {
		if (StringUtils.isEmpty(downLoadUrl)) {
			return false;
		}
		download = new HttpURLDownload(downLoadUrl,fileSize,  tempdSavePath, listener) {
			public DOWN_STATUS downloadStatus() {
				setDeleteSourceFile(isDeleteSourceFile);
				return downloadStatus;
			}
		};
		download.setMessageStartName(tenderProject.getTenderName() + " - " + supplier.getSupplierName());
		File downloadZipFile = new File(tempdSavePath);
		boolean downFlag = download.downLoad();
		if (downFlag) {
			downFlag = unZipFile(downloadZipFile.getAbsolutePath(), savePath);
			if (downFlag) {
				downloadZipFile.delete();

				File[] files = new File(savePath).listFiles(new FilenameFilter() {
					public boolean accept(File dir, String name) {
						return name.equals("files.zip");
					}
				});
				if (null != files && files.length > 0) {
					for (File file : files) {
						unZipFile(file.getAbsolutePath(), savePath);
					}
				}
			
			} else {
				LOG.error("解压失败");
			}
		} else {
			LOG.error("下载失败");
		}
		return downFlag;
	}

	/**
	 * @Description: 减压文件
	 * @author chenxw
	 * @param archive
	 * @param decompressDir
	 * @return
	 */
	private boolean unZipFile(String archive, String decompressDir) {
		boolean unZipFlag = false;
		unZipFlag = ZipUtils.unZip(archive, decompressDir, false, true, true, null);
		return unZipFlag;
	}

	private void startDownload() {
		downloadStatus = DOWN_STATUS.START;
		if (null != listener) {
			listener.setDownloadStatus(DOWN_STATUS.START);
		}
	}

	public void start() {
		new Thread(this).start();
	}

	public void setDownloadStatus(DOWN_STATUS downloadStatus) {
		this.downloadStatus = downloadStatus;
		if (null != listener) {
			listener.setDownloadStatus(downloadStatus);
		}
	}

	public void initDownload() {
		downloadStatus = DOWN_STATUS.WAITE;
		isDeleteSourceFile = false;
		if (null != listener) {
			listener.reset();
		}
	}

	public void setDeleteDownloadFile(boolean isDelete) {
		this.isDeleteSourceFile = isDelete;
	}

	public void deleteDownloadFile() {
		download.deleteDownloadFile();
	}

	public Runnable getRunnable() {
		return this;
	}
	
}
