package com.gx.obe.common.file.thread;

import java.io.File;

import org.apache.log4j.Logger;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.file.download.infer.FileDownloadListener;
import com.gx.obe.common.file.download.infer.FileDownloadListener.DOWN_STATUS;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.http.HttpURLDownload;
import com.gx.obe.common.file.inter.FileDownloadThreadInterface;
import com.gx.obe.common.file.utils.ZipUtils;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.tender.TenderProjectEntity;

/**
 * @Description: Http下载招标文件
 * @author chenxw
 */
public class HttpBiddingFileDownloadThread implements Runnable, FileDownloadThreadInterface {

	private DOWN_STATUS downloadStatus = DOWN_STATUS.WAITE;
	private FileDownloadListener listener;
	private boolean isDeleteSourceFile = false;
	private TenderProjectEntity tenderProject;
	private Logger LOG = Logger.getLogger(HttpBiddingFileDownloadThread.class);
	private HttpURLDownload download;

	public HttpBiddingFileDownloadThread(TenderProjectEntity tenderProject, FileDownloadListener listener) {
		this.tenderProject = tenderProject;
		this.listener = listener;
	}

	public void run() {
		try {
			startDownload();
			if (StringUtils.isNotEmpty(tenderProject.getTransactionServerId())) {
				HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProject, PropertiesUtils.getWebserviceProperty(WebserviceConstants.DOWN_LOAD_BIDDING_FILE_METHOD, "biddingFileHS.do?getBiddingFileApply"));
				try {
					downloadBiddingFile(HttpUtils.doPost(httpUrl));
				} catch (Exception e) {
					downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
					return;
				}
			} else {
				LOG.error("无下载请求地址！");
				downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			}
		} catch (Exception e) {
			LOG.error("下载失败！");
			downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
		} finally {
			if (null != listener) {
				listener.setDownloadStatus(downloadStatus);
			}
		}
	}

	/**
	 * @Description: 解析获取招标文件的下载地址
	 * @author guoyr
	 * @param
	 * @return
	 * @Exception
	 */
	private void downloadBiddingFile(HttpResponse httpResponse) {
		if (null == httpResponse.getContent() || "".equals(httpResponse.getContent())) {
			LOG.debug("服务器返回数据为空！");
			downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			return;
		}
		String downUrl = httpResponse.getDataParam("downUrl");
		long bidFileSize = 0;
		if (null != httpResponse.getDataParam("fileSize") && !"".equals(httpResponse.getDataParam("fileSize"))) {
			try {
				bidFileSize = (Long) Long.parseLong(httpResponse.getDataParam("fileSize"));
			} catch (Exception e) {
			}
		}
		String tempdSavePath = FileConstants.TEMP_FOLDER.concat("biddingFile").concat(File.separator).concat(tenderProject.getId()).concat(File.separator).concat(FileConstants.BIDDING_FILE).concat(".zip");
		String downLoadPath = new File(FileConstants.getBiddingFilePath(tenderProject.getId())).getAbsolutePath();
		if (downLoadFile(downUrl, bidFileSize, tempdSavePath, downLoadPath)) {
			downloadStatus = DOWN_STATUS.SUCCESS;
		} else {
			downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
		}
	}

	/**
	 * @Description: 下载文件
	 * @author chenxw
	 * @param downLoadUrl
	 * @param savePath
	 * @return
	 */
	private boolean downLoadFile(String downLoadUrl, long fileSize, String tempdSavePath, String savePath) {
		if (StringUtils.isEmpty(downLoadUrl)) {
			return false;
		}
		download = new HttpURLDownload(downLoadUrl, fileSize, tempdSavePath, listener) {
			public DOWN_STATUS downloadStatus() {
				setDeleteSourceFile(isDeleteSourceFile);
				return downloadStatus;
			}
		};
		download.setMessageStartName(tenderProject.getTenderName() + " - 招标文件");
		File downloadZipFile = new File(tempdSavePath);
		boolean downFlag = download.downLoad();
		if (downFlag) {
			downFlag = unZipFile(downloadZipFile.getAbsolutePath(), savePath);
			if (downFlag) {
				downloadZipFile.delete();
			} else {
				LOG.error("解压失败");
			}
		} else {
			LOG.error("下载失败");
		}
		return downFlag;
	}

	/**
	 * @Description: 减压文件
	 * @author chenxw
	 * @param archive
	 * @param decompressDir
	 * @return
	 */
	private boolean unZipFile(String archive, String decompressDir) {
		boolean unZipFlag = false;
		unZipFlag = ZipUtils.unZip(archive, decompressDir, false, true, true, null);
		return unZipFlag;
	}

	private void startDownload() {
		downloadStatus = DOWN_STATUS.START;
		if (null != listener) {
			listener.setDownloadStatus(DOWN_STATUS.START);
		}
	}

	public DOWN_STATUS getDownloadStatus() {
		return downloadStatus;
	}
	
	public void start() {
		new Thread(this).start();
	}

	public void setDownloadStatus(DOWN_STATUS downloadStatus) {
		this.downloadStatus = downloadStatus;
		if (null != listener) {
			listener.setDownloadStatus(downloadStatus);
		}
	}

	public void initDownload() {
		downloadStatus = DOWN_STATUS.WAITE;
		isDeleteSourceFile = false;
		if (null != listener) {
			listener.reset();
		}
	}

	public void setDeleteDownloadFile(boolean isDelete) {
		this.isDeleteSourceFile = isDelete;
	}

	public void deleteDownloadFile() {
		if(null != download){
			download.deleteDownloadFile();
		}
	}

	public Runnable getRunnable() {
		return this;
	}

}
