package com.gx.obe.common.file.thread;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.zip.ZipException;

import org.apache.log4j.Logger;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.eclipse.swt.core.CallBack;

import com.gx.obe.WebserviceConstants;
import com.gx.obe.business.http.HttpResponse;
import com.gx.obe.business.http.HttpUrl;
import com.gx.obe.business.http.HttpUtils;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.http.HttpURLDown;
import com.gx.obe.common.file.utils.ZipUtils;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.util.entity.BaseEntity;
import com.gx.obe.util.utils.LogUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.web.entity.tender.TenderProjectEntity;

/**
 * @Description: 下载招标文件
 * @author guoyr
 */
public class NewDownloadBiddingFileThread implements Runnable{
	
	private CallBack callBack;
	private Logger LOG = Logger.getLogger(NewDownloadBiddingFileThread.class);	
	private TenderProjectEntity tenderProjectEntity;
	private Thread thread;
	
	public NewDownloadBiddingFileThread(TenderProjectEntity tenderProjectEntity, CallBack callBack){
		this.callBack = callBack;
		this.tenderProjectEntity = tenderProjectEntity;
	}
	public void start(){
		thread = new Thread(this);
		thread.start();
	}
	
	public void stop(){
		thread = null;
	}
	public void join(){
		try {
			thread.join();
		} catch (InterruptedException e) {
		}
	}
	public void run() {
		callBackMsg("准备下载招标文件...");
		if(StringUtils.isNotEmpty(tenderProjectEntity.getTransactionServerId())){
			callBackMsg("正在获取下载请求地址...");
			HttpUrl httpUrl = HttpUtils.getProjectHttpUrl(tenderProjectEntity, PropertiesUtils.getWebserviceProperty(WebserviceConstants.DOWN_LOAD_BIDDING_FILE_METHOD, "biddingFileHS.do?getBiddingFileApply"));
			HttpResponse httpResponse = HttpUtils.doPost(httpUrl);
			downloadBiddingFile(httpResponse);
		}else {
			callBackMsg(false, "无下载请求地址！");
		}
		
		if(null != callBack){
			callBack.after();
		}
	}
	
	
	
	/** 
	 * @Description: 解析获取招标文件的下载地址
	 * @author guoyr
	 * @param   
	 * @return  
	 * @Exception   
	 */
	private void downloadBiddingFile(HttpResponse httpResponse){
	
		String  downUrl = ""; 
		if(null == httpResponse.getContent() || "".equals(httpResponse.getContent())){
			LOG.debug("服务器返回数据为空！");
			callBackMsg(false, "服务器返回数据为空！");
			callBackMsg(false, "未获取到下载地址！");
			return ;
		}
		
		downUrl = httpResponse.getDataParam("downUrl");
		if(StringUtils.isNotEmpty(downUrl)){
			LOG.debug("下载地址获取成功"+downUrl);
			callBackMsg(true, "下载地址获取成功，开始解析下载地址...");
			String downLoadPath = new File(FileConstants.getBiddingFilePath(tenderProjectEntity.getId()) + FileConstants.BIDDING_FILE+".zip").getAbsolutePath();
			downLoadFile(downUrl, downLoadPath);
		}else {
			callBackMsg(false, "获取下载地址失败！");
		}
	}

	
	
	private void downLoadFile(String downLoadUrl, String savePath){
		callBackMsg("开始下载招标文件...");
		HttpURLDown httpURLDown = new HttpURLDown(downLoadUrl, savePath, callBack);
		try {
			boolean downFlag = httpURLDown.downLoad(Messages.BiddingFile);
			callBackMsg("招标文件下载结束！");
			// 下载成功并且是.zip文件才进行解压操作
			if(downFlag && savePath.contains(".zip")){
				unZipFile(savePath, FileConstants.getBiddingFilePath(tenderProjectEntity.getId()));
			}
		} catch (IOException e) {
			LOG.error("招标文件下载失败", e);
			callBackMsg("招标文件下载文件出现错误");
			callBackMsg("错误信息为："+e.getMessage());
			return;
			
		}
	}
	private void unZipFile(String archive, String decompressDir){
		if(!ZipUtils.unZip(archive, decompressDir, true, true, true,callBack)) {
			callBackMsg(false, "解压招标文件失败！");
		}
	}
	
	/**
	 * @Description: 解析返回Xml文件中的答疑文件部分
	 * @author guoyr
	 * @param xmlStr
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public List<AnswerQuestion> resolveAnswerQuestionFile(String xmlStr){
		List<AnswerQuestion> answerQuestionList = null;
		if(StringUtils.isEmpty(xmlStr)){
			return answerQuestionList;
		}
		try {
			Document document = DocumentHelper.parseText(xmlStr);
			Element root = document.getRootElement();
			Element header = root.element("header");
			String operTag = header.elementTextTrim("operTag");
			// 验证服务器返回结果是否正确
			if(null == operTag || !operTag.equals("Y")){
				LOG.error("服务器返回消息："+header.elementTextTrim("operDesc"));
				LOG.error("服务器返回异常："+header.elementTextTrim("operException"));
				LOG.error("服务器返回内容："+xmlStr);
				return answerQuestionList;
			}
			Element project = root.element("body").element("project");
			Element answerQuestFilesElement = project.element("answerQuestionFiles");
			if(null != answerQuestFilesElement){
				// 获得answerQuestionFile标签集合
				List<Element> answerQuestionFileList = answerQuestFilesElement.elements("answerQuestionFile");
				if(null != answerQuestionFileList && answerQuestionFileList.size() > 0){
					answerQuestionList = new ArrayList<AnswerQuestion>();
					// 遍历answerQuestionFile标签
					for(Element answerQuestionFileE : answerQuestionFileList){
						AnswerQuestion answerQuestion = new AnswerQuestion();
						// 答疑文件名称
						answerQuestion.setFileName(answerQuestionFileE.elementTextTrim("fileName"));
						// 下载地址
						answerQuestion.setDownUrl(answerQuestionFileE.elementTextTrim("downUrl"));
						answerQuestionList.add(answerQuestion);
					}
				}
			}
		} catch (DocumentException e) {
			callBackMsg(true, "解析xml失败，返回的xml结果为："+xmlStr);
			LOG.error("返回的xml结果为："+xmlStr, e);
		}
		return answerQuestionList;
	}
	
	/**
	 * @Description: 解析返回的xml
	 * @author guoyr
	 * @param xmlStr
	 * @return
	 */
	@SuppressWarnings({"unused"})
	private String resolveXml(TenderProjectEntity tenderProjectEntity, String xmlStr){
		if(StringUtils.isNotEmpty(xmlStr)){
			LOG.debug("服务器信息为空！");
			callBackMsg(true, "服务器信息为空！");
			return "";
		}
		Document document;
		String downUrl = null;
		try {
			document = DocumentHelper.parseText(xmlStr);
			Element root = document.getRootElement();
			
			Element header = root.element("header");
			String operTag = header.elementTextTrim("operTag");
			if(null == operTag || !operTag.equals("Y")){
				if(null != callBack){
					callBack.callBackError("服务器返回消息："+header.elementTextTrim("operDesc"));
				}
				LOG.error("服务器返回消息："+header.elementTextTrim("operDesc"));
				LOG.error("服务器返回异常："+header.elementTextTrim("operException"));
				LOG.error("服务器返回结果："+xmlStr);
				return null;
			}
			Element body = root.element("body");
			
			if(null != body){
				Element projectElement = body.element("project");
				if(null != projectElement){
					Element purchaseFileElement = projectElement.element("purchaseFile");
					downUrl = purchaseFileElement.elementTextTrim("downUrl");
				}
			}
			
		} catch (DocumentException e) {
			if(null != callBack){
				callBack.callBackError(e.getMessage());
			}
			LogUtils.logError(e.getMessage(), e);
			return null;
		}
		return downUrl;
	}
	private void callBackMsg(String msg){
		if(null != callBack){
			callBack.callBackString("  "+msg);
		}
	}
	private void callBackMsg(boolean bool, String msg){
		if(null != callBack){
			callBack.callBackString((bool?"√":"×")+ msg);
		}
	}

	/** 
	 * @Description: 答疑文件实体类
	 * @author guoyr 
	 */
	class AnswerQuestion implements BaseEntity {
		
		private static final long serialVersionUID = 1L;
		private String id;
		private String fileName;
		private String downUrl;
		
		public String getId() {
			return id;
		}
		
		public void setId(String id) {
			this.id = id;
		}
		
		public String getFileName() {
			return fileName;
		}
		
		public void setFileName(String fileName) {
			this.fileName = fileName;
		}
		
		public String getDownUrl() {
			return downUrl;
		}
		
		public void setDownUrl(String downUrl) {
			this.downUrl = downUrl;
		}
	}
}
