package com.gx.obe.common.file.thread;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

import com.gx.obe.common.file.download.DownloadFileUtill;
import com.gx.obe.common.file.download.infer.DownloadFileListener;
import com.gx.obe.common.file.download.infer.FileDownloadListener;
import com.gx.obe.common.file.download.infer.FileDownloadListener.DOWN_STATUS;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.inter.FileDownloadThreadInterface;
import com.gx.obe.common.file.service.BidFileUploadService;
import com.gx.obe.common.file.utils.ZipUtils;
import com.gx.obe.web.entity.FileInfoEntity;
import com.gx.obe.web.entity.tender.TenderProjectEntity;

/**
 * @Description: 服务端下载招标文件
  */
public class ServiceBiddingFileDownloadThread implements Runnable, FileDownloadThreadInterface {

	private DOWN_STATUS downloadStatus = DOWN_STATUS.WAITE;
	private FileDownloadListener listener;
	private boolean isDeleteSourceFile = false;
	private Logger LOG = Logger.getLogger(HttpBidFileDownloadThread.class);
	private TenderProjectEntity tenderProject;
	private BidFileUploadService bidFileUploadService = new BidFileUploadService();

	public ServiceBiddingFileDownloadThread(TenderProjectEntity tenderProject, FileDownloadListener listener) {
		this.tenderProject = tenderProject;
 		this.listener = listener;
 	}

	public void run() {
		try {
			startDownload();
			boolean downloadFlag = false;
			String downLoadPath = new File(FileConstants.getBiddingFilePath(tenderProject.getId())).getAbsolutePath();

			downloadFlag = downLoadFile(downLoadPath);
			if (downloadFlag) {
				downloadStatus = DOWN_STATUS.SUCCESS;
			} else {
 				downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			}
		} catch (Exception e) {
			downloadStatus = downloadStatus == DOWN_STATUS.CANCEL ? downloadStatus : DOWN_STATUS.FAIL;
			LOG.error("下载失败！", e);
		} finally {
			listener.setDownloadStatus(downloadStatus);
		}
	}

	boolean downFlag = false;
	DownloadFileUtill downloadFileUtill;
	private boolean downLoadFile(String savePath) {
		List<FileInfoEntity> fileInfoEntityList = bidFileUploadService.getBiddingFile(tenderProject.getId());
		long bidFileSize = 0;
		List<File> fileList = new ArrayList<File>();
		for(FileInfoEntity fileInfoEntity : fileInfoEntityList) {
			bidFileSize = bidFileSize + fileInfoEntity.getFileSize();
			File file = new File(savePath + File.separator + fileInfoEntity.getFileName());
			if(file.getName().endsWith(".zip")) {
				fileList.add(file);
			}
		}
		listener.setTotalSize(bidFileSize);
		for(FileInfoEntity fileInfoEntity : fileInfoEntityList) {
			if(DOWN_STATUS.CANCEL.equals(downloadStatus)) {
				break;
			}
			downloadFileUtill = new DownloadFileUtill(fileInfoEntity.getLocation(),fileInfoEntity.getFileName(),null,savePath ,new DownloadFileListener() {
				@Override
				public void speed(String speed) {
				}
				@Override
				public void remainingTime(String remainingTime) {}
				@Override
				public void progress(int percent) {
				}
				@Override
				public void complate(boolean flag) {
					downFlag = flag;
				}
				@Override
				public void checkMD5(boolean flag) {}
				@Override
				public void writeSize(long size) {
					listener.writeSize(size);
				}
			});
			 downloadFileUtill.startDownloadFile();
			 try {
				downloadFileUtill.getSiteFileFetch().join();
			} catch (InterruptedException e) {
 				e.printStackTrace();
			}
			
		}
		if (downFlag) {
			for(File file :fileList) {
				downFlag = unZipFile(file.getAbsolutePath(), savePath);
			}
 			if (!downFlag) {
 				LOG.error("解压失败");
 			}
		} else {
			LOG.error("下载失败");
		}
		return downFlag;
	}

	/**
	 * @Description: 减压文件
	 * @param archive
	 * @param decompressDir
	 * @return
	 */
	private boolean unZipFile(String archive, String decompressDir) {
		boolean unZipFlag = false;
		unZipFlag = ZipUtils.unZip(archive, decompressDir, false, true, true, null);
		return unZipFlag;
	}

	private void startDownload() {
		downloadStatus = DOWN_STATUS.START;
		if (null != listener) {
			listener.setDownloadStatus(DOWN_STATUS.START);
		}
	}

	public void start() {
		new Thread(this).start();
	}

	public void setDownloadStatus(DOWN_STATUS downloadStatus) {
		this.downloadStatus = downloadStatus;
		if (null != listener) {
			listener.setDownloadStatus(downloadStatus);
		}
		if(null != downloadFileUtill && DOWN_STATUS.CANCEL.equals(downloadStatus)) {
			downloadFileUtill.stop();
		}else if(null != downloadFileUtill && DOWN_STATUS.PAUSE.equals(downloadStatus)) {
			downloadFileUtill.setPause(true);
		}else if(null != downloadFileUtill && DOWN_STATUS.START.equals(downloadStatus)) {
			downloadFileUtill.setPause(false);
		}
	}

	public void initDownload() {
		downloadStatus = DOWN_STATUS.WAITE;
		isDeleteSourceFile = false;
		if (null != listener) {
			listener.reset();
		}
	}

	public void setDeleteDownloadFile(boolean isDelete) {
		this.isDeleteSourceFile = isDelete;
	}

	public void deleteDownloadFile() {
		if(this.isDeleteSourceFile) {
			String downLoadPath = new File(FileConstants.getBiddingFilePath(tenderProject.getId())).getAbsolutePath();
			deleteDir(downLoadPath); 
		}
	}
	
	public Runnable getRunnable() {
		return this;
	}
	
	private void deleteDir(String dirPath){
		File file = new File(dirPath);
		if(file.isFile()){
			file.delete();
		}else{
			File[] files = file.listFiles();
			if(files == null){
				file.delete();
			}else{
				for (int i = 0; i < files.length; i++) {
					deleteDir(files[i].getAbsolutePath());
				}
				file.delete();
			}
		}
	}

}
