package com.gx.obe.common.file.upload;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import com.gx.obe.business.entity.Result;
import com.gx.obe.business.entity.VideoFile;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.upload.infer.UploadFileListener;
import com.gx.obe.common.file.utils.ComputeUtill;
import com.gx.obe.common.file.utils.FileMd5Util;
import com.gx.obe.http.HttpResponse;
import com.gx.obe.http.Status;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.web.service.VideoReportFileService;
import com.gx.obe.web.utils.NetworkRequest;
 
/** 
 * @Description: 客户端发送
 * @author mazc 
 */
public class FileUpload extends Thread{
	
	private long fileTotalSzie = 0;

	//当前下载大小
	private long uploadTotalSize = 0;
	
	//上一次下载大小
	private long oldUploadTotalSize = 0;
	
	//当前下载速度
	private double speed = 0; 
	
	//当前下载进度
	private int percent = 0; 
	
	// 剩余时间
	private long remainderTime = 0;
	
	private InfoBean bean ;
	
	private File tmpFile;
	
	private FileInputStream fis;
	
	private UploadFileListener listener;
	
	private CountDownLatch countDownLatch;
	
	private ExecutorService service;
	
	private boolean isUplaod = false;

	private Result<String> result;
	
	private int uploadSuccessCount,uploadFailedCount = 0;
	
	//计算频率
	long sepTime = 1000;
	
	
	/**
	 * @Description: 上传大文件
	 * @author mazc
	 * @param bean 
	 */
	public FileUpload(InfoBean bean ,UploadFileListener listener) {
		this.bean = bean;
		this.listener = listener;
		this.bean.setsFileName(new File(bean.getsFilePath()).getName());
	}
	
	public void run(){
		File file = new File(bean.getsFilePath());
		fileTotalSzie = file.length();
		String fileMD5 = FileMd5Util.getFileMD5(file);
		this.bean.setFileMD5(fileMD5);
		// 记录上传记录
		this.tmpFile = new File(file.getParentFile().getAbsoluteFile() +  File.separator + bean.getsFileName() + ".info");	
		isUplaod = false;
		largeFileUpload();
	}
	

	private synchronized void uploadCount(boolean flag) {
		if(flag) {
			 uploadSuccessCount ++;
		}else {
			uploadFailedCount ++;
		}
	}
     /**
     * @Description: 
     * @author mazc
     * @param dirPath 本地文件路径
     * @param url 上传文件路径
     * @param fileName 文件名称
     */
    public void largeFileUpload() {
    	try {
    		File file = new File(bean.getsFilePath());
    		fis = new FileInputStream(file);
    	
    		//分段大小  2M为一段
    		long partSize = 1024*1024*2;
    		//获取总文件大小
    		long fileSize = fis.getChannel().size();
    		fis.close();
    		//分多少段
            int  partCount = (int) (fileSize/partSize);
            if(partSize*partCount<fileSize|| partCount == 0){
            	partCount++;
            }
            //线程池数量
            int iThreadNuber = 0;
            //获取当前系统cup核数
			int iCpuNumber = Runtime.getRuntime().availableProcessors();
			//当前供应商数量大于iCpuNumber*2+1扩大线程池数量比便于调高效率 (2n + 1)
			if(partCount >= (iCpuNumber * 2 + 1)){
				iThreadNuber = iCpuNumber * 2 + 1;
			}else{
				iThreadNuber = iCpuNumber - 1;
			}	
			//根据系统当前cup数量计算线程数量
			iThreadNuber = partCount < iThreadNuber ? partCount : Math.max(2, iThreadNuber);
            //线程大小为分段数
			service = Executors.newFixedThreadPool(iThreadNuber);
 
            //partCount个线程全部处理完后  countDownLatch.await()阻塞通过 
            //哨兵
            countDownLatch = new CountDownLatch(partCount);
            //获取文件ID
            String fileId =UUID.randomUUID().toString().replace("-","");   
            new Thread(new Runnable(){
				public void run(){
					while(!isUplaod){
						compute(sepTime);
						try{
							Thread.sleep(sepTime);
						}catch (Exception e){
							e.printStackTrace();
						}
					}
				}
			}).start();
            uploadSuccessCount = uploadFailedCount = 0;
            for (int i = 0; i < partCount ; i++) {
            	int num = i+1;
            	//当前分段起始位置
            	long partStart=i*partSize;
            	//当前分段大小  如果为最后一个大小为fileSize-partStart  其他为partSize
            	long curPartSize=(i+1==partCount)?(fileSize-partStart):partSize;
            	FileChunk fileChunk = new FileChunk();
            	fileChunk.setId(1112313L + i);
            	//当前块
            	fileChunk.setChunkNumber(num);
            	//起始位置
            	fileChunk.setPartStart(partStart);
            	//分块大小
            	fileChunk.setChunkSize(1024*1024L);
            	//当前分块大小
            	fileChunk.setCurrentChunkSize(curPartSize);
            	//总大小
            	fileChunk.setTotalSize(fileSize);
            	//文件标识
            	fileChunk.setIdentifier(fileId);
            	//文件名
            	fileChunk.setFilename(bean.getsFileName());
         
            	//总块数
            	fileChunk.setTotalChunks(partCount);
              
            	fileChunk.setTenderId(bean.getTenderId());
              
            	fileChunk.setSupplierId(bean.getSupplierId());
              
            	fileChunk.setFileType(bean.getFileType());
              
            	FileUploadRunnable fileUploadRunnable = new FileUploadRunnable(bean.getURL(),countDownLatch,fileChunk,file,new UploadFileSizeListener() {
            		@Override
            		public void setFileSize(long size) {
            			setUploadTotalSize(size);
            		}

					@Override
					public void setUploadstatus(boolean uploadStatus) {
						uploadCount(uploadStatus);
					}
            	});
//            	fileUploadRunnable.run();
            	service.submit(fileUploadRunnable);
            }
            service.shutdown();
          //阻塞直至countDownLatch.countDown()被调用partCount次  即所有线程执行任务完毕
            countDownLatch.await();
          
          //如果块数大于1则进行文件拼装
            if(partCount > 1){
            	Map<String, Object> param = new HashMap<String, Object>();       
            	param.put("tenderId",bean.getTenderId());
            	param.put("supplierId",bean.getSupplierId());
            	param.put("fileType",bean.getFileType());
            	param.put("filename", bean.getsFileName());
            	param.put("fileMD5", bean.getFileMD5());
            	HttpResponse strResult = NetworkRequest.postHttpResponse(bean.getURL()+"/mergeFile", param);
            	if(strResult.getStatus()== Status.HTTP_OK){
            		result = JsonUtil.jsonToBean(strResult.body(), Result.class);
            	}
            }
            if(FileConstants.VIDEO_FILE_FOLDER.equals(bean.getFileType())) {
            	//插入数据
            	VideoReportFileService  VideoReportFileService = new VideoReportFileService();
            	VideoFile videoFile = new VideoFile();
            	videoFile.setId(UuidUtils.getUUID());
            	videoFile.setUserAccount(bean.getSupplierId());
            	videoFile.setTenderId(bean.getTenderId());
            	String filePath = "/" + bean.getTenderId() + "/" +bean.getSupplierId() + "/" + bean.getFileType() + "/" + bean.getsFileName();
            	videoFile.setFilePath(filePath);
            	videoFile.setFileName(bean.getsFileName());
            	videoFile.setFileSize(new File(bean.getsFilePath()).length());
            	videoFile.setCreateTime(new Date());
            	VideoReportFileService.saveOrUpadte(videoFile);
            }
//            URL url = new URL(bean.getURL()+"/mergeFile");
//            HttpURLConnection connection = (HttpURLConnection) url.openConnection();
//            connection.setRequestMethod("POST");
//            connection.setDoOutput(true);
//            connection.setDoInput(true);
//            connection.setUseCaches(false);
//
//            //设置请求头
//            connection.setRequestProperty("Connection", "Keep-Alive");
//            connection.setRequestProperty("Charset", "UTF-8");
//            connection.setRequestProperty("Content-Type","multipart/form-data; BOUNDARY=" + java.util.UUID.randomUUID().toString());
//            connection.setRequestProperty("Authorization","Bearer " + "xiaomapro");
//            connection.connect();
//
//            DataOutputStream outputStream = new DataOutputStream(connection.getOutputStream());
//            //写入请求参数
//            outputStream.write(getParamsString(param).getBytes());
//            //写入标记结束位
//            byte[] endData = (LINE_END + TWO_HYPHENS + BOUNDARY + TWO_HYPHENS + LINE_END).getBytes();
//            outputStream.write(endData);
//            outputStream.flush();
//            //得到响应
//            int responseCode = connection.getResponseCode();
//            Result<String> result = null;
//            if(responseCode == HttpURLConnection.HTTP_OK){
//            	//得到响应流
//            	InputStream inputStream = connection.getInputStream();
//            	//获取响应
//            	BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
//            	String line ;
//            	while ((line = reader.readLine()) != null){
//            		line += line;
//            	}
//            	reader.close();
//             	result = JsonUtil.jsonToBean(line, Result.class);
//            } 
            
            isUplaod = true;
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (InterruptedException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }finally{
        	try {
        		if(null != fis){
        			fis.close();        			
        		}
			} catch (IOException e) {
			}
        	if(listener != null){
             	//发送回调通知已经传输完毕
             	listener.Completed(uploadSuccessCount > 0 && uploadFailedCount == 0);            	
             }
        }
    }
    
    
    
//    private String getParamsString(Map<String, Object> paramsMap) {
//        StringBuffer strBuf = new StringBuffer();
//        for (String key : paramsMap.keySet()){
//        	strBuf.append(TWO_HYPHENS);
//        	strBuf.append(BOUNDARY);
//        	strBuf.append(LINE_END);
//        	strBuf.append("Content-Disposition: form-data; name=\"" + key + "\"");
//        	strBuf.append(LINE_END);
//        	strBuf.append("Content-Type: " + "text/plain" );
////            strBuf.append(LINE_END);
////            strBuf.append("Content-Lenght: "+paramsMap.get(key).toString().length());
//        	strBuf.append(LINE_END);
//        	strBuf.append(LINE_END);
//        	strBuf.append(paramsMap.get(key));
//        	strBuf.append(LINE_END);
//        }
//        return strBuf.toString();
//    }
    
   
    /**
     * @Description: 进行相应的计算
     * @author mazc
     * @param sepTime 
     */
    private void compute(long sepTime){
    	computeherLoadProgress();
    	computeDownloadSpeed(sepTime);
    	computeRemainingTimeCurrent();
    } 
   
    
	public void computeherLoadProgress(){
    	// 剩余的字节数
    	long laveLength = fileTotalSzie - getUploadTotalSize();
    	if(fileTotalSzie == 0) {
    		percent = 100;
    	}else {
    		percent = (int) ((fileTotalSzie - laveLength) * 100 / fileTotalSzie);
    	}
    	if(null != listener){
    		listener.computeherLoadProgress(percent);    		
    	}
    }
    
    public void computeRemainingTimeCurrent(){
    	if( oldUploadTotalSize == getUploadTotalSize()){
    		return;
    	}
    	if(speed == 0){
    		if(null != listener){
        		listener.computeRemainingTimeCurrent(ComputeUtill.getTime(86500 * 1000)); 		
        	}
    		return ;
    	}
    	long laveLength = fileTotalSzie - getUploadTotalSize();
    	remainderTime = laveLength / (long)speed;
    	if(null != listener){
    		listener.computeRemainingTimeCurrent(ComputeUtill.getTime(remainderTime * 1000)); 		
    	}
    }
    
    private void computeDownloadSpeed(long minsecond){
    	speed = (getUploadTotalSize() - oldUploadTotalSize) / minsecond * 1000;
        oldUploadTotalSize = getUploadTotalSize();
  		if(null != listener){
    		listener.computeDownloadSpeed(ComputeUtill.computeSize(speed).concat("/S")); 		
    	}
    }
    
    /**
     * @Description: 获取剩余时间
     * @author mazc
     * @return 
     */
    public String getRemainingTimeCurrent(){
    	return ComputeUtill.getTime(remainderTime * 1000);
    }

    /**
     * @Description: 获取下载速度
     * @author mazc
     * @return 
     */
    public String getDownloadSpeed(){
     	String strDownloadSpeed = ComputeUtill.computeSize(speed).concat("/S");
		return strDownloadSpeed;
    }
    
    /**
     * @Description: 获取当前进度
     * @author mazc
     * @return 
     */
    public int getherLoadProgress(){
    	return percent;
    }
    
    public long getUploadTotalSize() {
		return uploadTotalSize;
	}

	public synchronized void setUploadTotalSize(long uploadTotalSize) {
		this.uploadTotalSize += uploadTotalSize;
	}
	
	public Result<String> getResult(){
		return result;
	}
	
    /**
     * @Description: 停止文件上传
     * @author mazc 
     */
    public void siteStop(){
    	service.shutdownNow();
    	if(tmpFile.exists()){
    		tmpFile.delete();
    	}
    }
	
    public interface UploadFileSizeListener {
    	void setUploadstatus(boolean status);
    	void setFileSize(long size);
    } 
}
