package com.gx.obe.common.file.upload;

import java.io.Closeable;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.CountDownLatch;

import com.gx.obe.common.file.service.UploadFileService;
import com.gx.obe.components.core.Constants;
import com.gx.obe.util.utils.StringUtil;
 
/** 
 * @Description: 
 * @author mazc 
 */
public class FileUploadRunnable  implements Runnable{
	
    private final String BOUNDARY = java.util.UUID.randomUUID().toString();
    private final String TWO_HYPHENS = "--";
    private final String LINE_END = "\r\n";
    private final String FILE_TYPE_FILE = "file/*";
    private String url ;
    private CountDownLatch countDownLatch;
    private FileChunk fileChunk;
    private File file;
    private FileUpload.UploadFileSizeListener listener;
    private HttpURLConnection connection;
    private UploadFileService uploadFileService;
    public FileUploadRunnable(String url, CountDownLatch countDownLatch,FileChunk fileChunk ,File file, FileUpload.UploadFileSizeListener listener){
        this.url =url;
        this.countDownLatch = countDownLatch;
        this.fileChunk = fileChunk;
        this.file = file;
        this.listener = listener;
    }
 
    @Override
    public void run() {
    	//暂时去掉续传MD5校验
    	byte[] bytes = spiltDetailByte(fileChunk.getPartStart(),fileChunk.getCurrentChunkSize());
//    	String fileChunkMD5 = FileMd5Util.getByteMD5(bytes);
//    	fileChunk.setFileChunkMD5(fileChunkMD5);
//    	uploadFileService = new UploadFileService();
//    	if(uploadFileService.checkFileChunkMD5(fileChunk)){
//    		listener.setFileSize(fileChunk.getCurrentChunkSize());  
//    		return;
//    	}
    	try{
    		//请求接收分段上传的地址
            Map<String, Object> param = new HashMap<String, Object>();       
            param.put("id", fileChunk.getId());
            param.put("chunkNumber", fileChunk.getChunkNumber());
            param.put("chunkSize", fileChunk.getChunkSize());
            param.put("currentChunkSize", fileChunk.getCurrentChunkSize());
            param.put("totalSize", fileChunk.getTotalSize());
            param.put("identifier", fileChunk.getIdentifier());     
            param.put("filename", fileChunk.getFilename());
            param.put("totalChunks", fileChunk.getTotalChunks());
            param.put("type", fileChunk.getType());
            param.put("partStart", fileChunk.getPartStart());
            if(!StringUtil.empty(fileChunk.getTenderId())){
            	param.put("tenderId",fileChunk.getTenderId());            	
            }
            if(!StringUtil.empty(fileChunk.getSupplierId())){
            	param.put("supplierId",fileChunk.getSupplierId());
            }
            if(!StringUtil.empty(fileChunk.getFileType())){
            	param.put("fileType",fileChunk.getFileType());            	
            }
            
            try {
                URL url = new URL(this.url);
                connection = (HttpURLConnection) url.openConnection();
                connection.setRequestMethod("POST");
                connection.setDoOutput(true);
                connection.setDoInput(true);
                connection.setUseCaches(false);

                //设置请求头
                connection.setRequestProperty("Connection", "Keep-Alive");
                connection.setRequestProperty("Charset", "UTF-8");
                connection.setRequestProperty("Content-Type","multipart/form-data; BOUNDARY=" + BOUNDARY);
                connection.setRequestProperty("Authorization","Bearer " + "xiaomapro");
                connection.setRequestProperty("ATUOKTHEN", Constants.TOKEN);
                connection.connect();

                DataOutputStream outputStream = new DataOutputStream(connection.getOutputStream());
                //写入请求参数
                outputStream.write(getParamsString(param).getBytes("UTF-8"));
                //写入文件头信息
                outputStream.write(getFileParamsString(fileChunk.getCurrentChunkSize()).getBytes());
                //写入文件内容
                outputStream.write(bytes);
                outputStream.flush();        
                //写入标记结束位
                byte[] endData = (LINE_END + TWO_HYPHENS + BOUNDARY + TWO_HYPHENS + LINE_END).getBytes();//写结束标记位
                outputStream.write(endData);
                outputStream.flush();
                //得到响应
                int responseCode = connection.getResponseCode();
                if(responseCode == HttpURLConnection.HTTP_OK){
                	if(null != listener) {
                		listener.setFileSize(fileChunk.getCurrentChunkSize());  
                		listener.setUploadstatus(true);
                	}
//                	  //得到响应流
//                      InputStream inputStream = connection.getInputStream();
//                      //获取响应
//                      BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
//                      String line ;
//                      while ((line = reader.readLine()) != null){
//                          line += line;
//                      }
//                      reader.close();          
//                      @SuppressWarnings("unchecked")
//                      Result<String> re = JsonUtil.jsonToBean(line, Result.class);
                }else {
                	listener.setUploadstatus(false);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
            
            
        }catch (Exception e){
            e.printStackTrace();
        }finally{
        	if(null != connection){
        		//断开连接
        		connection.disconnect();	
        	}
        	if(null != countDownLatch){
        		countDownLatch.countDown();        		
        	}
        }
     }
    
    private String getParamsString(Map<String, Object> paramsMap) {
        StringBuffer strBuf = new StringBuffer();
        for (String key : paramsMap.keySet()){
        	strBuf.append(TWO_HYPHENS);
        	strBuf.append(BOUNDARY);
        	strBuf.append(LINE_END);
        	strBuf.append("Content-Disposition: form-data; name=\"" + key + "\"");
        	strBuf.append(LINE_END);
        	strBuf.append("Content-Type: " + "text/plain" );
//            strBuf.append(LINE_END);
//            strBuf.append("Content-Lenght: "+paramsMap.get(key).toString().length());
        	strBuf.append(LINE_END);
        	strBuf.append(LINE_END);
        	strBuf.append(paramsMap.get(key));
        	strBuf.append(LINE_END);
        }
        return strBuf.toString();
    }

    
    
    /**
     * 上传文件时得到一定格式的拼接字符串
     */
    private String getFileParamsString(long fileSize ) {
        StringBuffer strBuf = new StringBuffer();
        strBuf.append(LINE_END);
        strBuf.append(TWO_HYPHENS);
        strBuf.append(BOUNDARY);
        strBuf.append(LINE_END);
        strBuf.append("Content-Disposition: form-data; name=\"" + "file" + "\"; filename=\"" + this.file.getName() + "\"");
        strBuf.append(LINE_END);
        strBuf.append("Content-Type: " + FILE_TYPE_FILE );
        strBuf.append(LINE_END);
        strBuf.append("Content-Lenght: " + fileSize);
        strBuf.append(LINE_END);
        strBuf.append(LINE_END);
        return strBuf.toString();
    }
    
    private byte[] spiltDetailByte(long beginPos, long actualBlockSize) {
    	//创建源
    	byte[] flush = null;
        //选择流
        RandomAccessFile raf = null;  //输入流
         try {
        	raf = new RandomAccessFile(file, "r");
            //移动指针
            raf.seek(beginPos);
            //缓冲区
            flush = new byte[(int) actualBlockSize];
            raf.read(flush);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
        	close(raf);
        }
		return flush;
    }
    
    //关闭流
    public  void close(Closeable... io) {
        for (Closeable temp : io) {
            try {
                if (null != temp) {
                    temp.close();
                }
            } catch (Exception e) {
            }
        }
 
    }
}
