package com.gx.obe.common.file.upload;

import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.file.upload.infer.UploadFileAdapter;
import com.gx.obe.common.file.upload.infer.UploadFileListener;
import com.gx.obe.components.core.Constants;

/** 
 * @Description: 
 * @author mazc 
 */
public class UploadFileUtil {
	
	private FileUpload fileUpload;
 	
	/** 
	 * @author chenxw
	 * @param dirPath
	 * @param adapter 
	 * @param sFileName 
	 */
	public UploadFileUtil(String dirPath, String sFileName, UploadFileAdapter adapter) {
		String url = Constants.getServiceUrl("/templateUploader/chunk");
		InfoBean bean = new InfoBean(url, dirPath, null, null, null);
		fileUpload = new FileUpload(bean, adapter);
		bean.setsFileName(sFileName);
	}
	
	/**
	 * 	上传软件
	 * @param dirPath
	 * @param sFileName
	 * @param adapter
	 */
	public UploadFileUtil(String dirPath, String sFileName,UploadFileListener adapter) {
		String url = Constants.getServiceUrl("/softwareUpload/chunk");
		InfoBean bean = new InfoBean(url, dirPath, null, null, null);
		fileUpload = new FileUpload(bean, adapter);
		bean.setsFileName(sFileName);
	}
	
	/**
	 * 	上传文件文件
	 * @param url 服务器地址
	 * @param dirPath 本地路径
	 * @param tenderId 项目id	
	 * @param supplierId 供应商文件
	 * @param fileType 文件类型
	 * @param listener 进度监听
	 */
	public UploadFileUtil(String url ,String dirPath,String tenderId,String supplierId, String fileType ,UploadFileListener listener){
 		InfoBean bean = new InfoBean(url, dirPath, tenderId, supplierId, fileType);
		fileUpload = new FileUpload(bean,listener);
	}
	/**
	 * 	上传评标视频	
	 * @param dirPath 本地路径
	 * @param tenderId 评审项目id
	 * @param userId   评标人
	 * @param listener 上传信息监听
	 */
	public UploadFileUtil(String dirPath,String tenderId,String userId,UploadFileListener listener){
		String url = Constants.getServiceUrl("/videoUploader/chunk");
		InfoBean bean = new InfoBean(url, dirPath, tenderId, userId, FileConstants.VIDEO_FILE_FOLDER);
		fileUpload = new FileUpload(bean,listener);
	}
	
	
	/**
	 * 	上传文件
	 * @param dirPath 本地文件路径
	 * @param tenderId  项目id
	 * @param supplierId 供应商id
	 * @param fileType 文件类型参考 Constants
	 * @param listener 上传进度信息
	 */
	public UploadFileUtil(String dirPath,String tenderId,String supplierId, String fileType ,UploadFileListener listener){
		String url = Constants.getServiceUrl("/uploader/chunk");
		InfoBean bean = new InfoBean(url, dirPath, tenderId, supplierId, fileType);
		fileUpload = new FileUpload(bean,listener);
	}
	
	
	/**
	 * @Description: 启动文件下载
	 * @author mazc 
	 */
	public void startUploadFileFile(){
		fileUpload.start();
	}
	
	public void join(){
		try {
			fileUpload.join();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @Description: 获取文件下载进度 %
	 * @author mazc
	 * @return 
	 */
	public int getherLoadProgress(){
		return fileUpload.getherLoadProgress();
	}
	
	/**
 	 * @Description: 获取下载速度
 	 * @author mazc
 	 * @return 
 	 */
 	public String getDownloadSpeed(){
 		return fileUpload.getDownloadSpeed();
 	}
 	
	/**
 	 * @Description: 获取剩余时间
 	 * @author mazc
 	 * @return 
 	 */
 	public String getRemainingTimeCurrent(){
 		return fileUpload.getRemainingTimeCurrent();
 	}
 	
 	/**
 	 * @Description: 校验文件是否一致
 	 * @author mazc
 	 * @return 
 	 */
 	public boolean getFileMD5(){
 		return false;
 	}
 	
 	public FileUpload getFileUpload() {
 		return fileUpload;
 	}
}

