package com.gx.obe.common.file.utils;

import java.text.DecimalFormat;

/** 
 * @Description: 
 * @author mazc 
 */
public class ComputeUtill {
	
	private static DecimalFormat fmt = new DecimalFormat("##,###,###,###,##0.00");
    
	/**
	 * @Description: 计算大小
	 * @author mazc
	 * @param size
	 * @return 
	 */
	public static String computeSize(double size) {
		String unit = "KB";
		if (size < 0)
			size = 0;
		if (size > (1024 * 1024 * 1024)) {
			unit = "GB";
			size = (double) size / (1024 * 1024 * 1024);
		} else if (size > (1024 * 1024)) {
			unit = "MB";
			size = (double) size / (1024 * 1024);
		} else if (size > (1024)) {
			unit = "KB";
			size = (double) size / (1024);
		} else {
			unit = "b";
		}
		return fmt.format(size).concat(unit);
	}
    
	/**
	 * @Description: 剩余时间
	 * @author mazc
	 * @param msUsed
	 * @return 
	 */
	public static String getTime(long msUsed) {
		if (msUsed < 1000) {
			return "小于1秒";
		}
		msUsed /= 1000;
		if (msUsed < 60) {
			return msUsed  + "秒";
		}
		if (msUsed < 3600) {
			long nMinute = msUsed / 60;
			long nSecond = msUsed % 60;
			return nMinute + "分" + nSecond + "秒";
		}
 		if (msUsed < 86400) {
			long nHour = msUsed / 3600;
			long nMinute = (msUsed - nHour * 3600) / 60;
			long nSecond = (msUsed - nHour * 3600) % 60;
			return nHour  + "时" + nMinute + "分" +nSecond+"秒";
		}
		return "大于24小时";
	}
	
	 /**
     * @Description: 分块下载
     * @author mazc
     * @param contentLength 
     */
    public static int setContentLength(long contentLength) {
 		if((contentLength /(double)(1024*1024)) <= 4){
			return 1;
		}else if((contentLength/(double)(1024*1024)) > 4 && (contentLength/(double)(1024*1024)) <= 20){
			return 2;
		}else if((contentLength/(double)(1024*1024)) > 20 && (contentLength/(double)(1024*1024)) <= 60){	
			return 4;
		}else if((contentLength/(double)(1024*1024)) > 60 && (contentLength/(double)(1024*1024)) <= 120){
			return 6;
		}else if((contentLength/(double)(1024*1024)) > 120 && (contentLength/(double)(1024*1024)) <= 240){
			return 8;
		}else {
			return 10;
 		}
 	}
	
    /**
	 * @Description: 计算 下载速度
	 * @author guoyr
	 * @param length 下载文件的长度 
	 * @param time 下载用时（毫秒）
	 * @return
	 */
	public static String getSpeed(double length, long time){
		double speed = length*1024*1024/(double)(time==0?1:time*1000);
		String unit = "KB";
		if(speed < 0)speed = 0;
		if(speed > (1024*1024*1024)){
			unit = "GB";
			speed = (double)speed/(1024*1024*1024);
		}else if(speed > (1024*1024)){
			unit = "MB";
			speed = (double)speed/(1024*1024);
		}else if(speed > (1024)){
			unit = "KB";
			speed = (double)speed/(1024);
		}else {
			unit = "字节";
		}
    	return fmt.format(speed).concat(unit).concat("/S");
	}
	
	/**
	 * @Description: 获得文件大小
	 * @author guoyr
	 * @param size
	 * @return
	 */
	public static String getFileSize(double size){
		String unit = "KB";
		if(size < 0)size = 0;
		if(size > (1024*1024*1024)){
			unit = "GB";
			size = (double)size/(1024*1024*1024);
		}else if(size > (1024*1024)){
			unit = "MB";
			size = (double)size/(1024*1024);
		}else if(size > (1024)){
			unit = "KB";
			size = (double)size/(1024);
		}else {
			unit = "字节";
		}
		return fmt.format(size).concat(unit);
	}
	
	/**
	 * @Description: 获得用时
	 * @author guoyr
	 * @param msUsed
	 * @return
	 */
	public static String getUseTime(long msUsed) {
		if (msUsed < 1000) {
			return "小于1秒";
		}
		// 长于1秒的过程，毫秒不计
		msUsed /= 1000;
		if (msUsed < 60) {
			return msUsed  + "秒";
		}
		if (msUsed < 3600) {
			long nMinute = msUsed / 60;
			long nSecond = msUsed % 60;
			return nMinute + "分" + nSecond + "秒";
		}
		// 3600 * 24 = 86400
		if (msUsed < 86400) {
			long nHour = msUsed / 3600;
			long nMinute = (msUsed - nHour * 3600) / 60;
			long nSecond = (msUsed - nHour * 3600) % 60;
			return nHour  + "时" + nMinute + "分" +nSecond+"秒";
		}

		return "大于24小时";
//		long nDay = msUsed / 86400;
//		long nHour = (msUsed - nDay * 86400) / 3600;
////		long nMinute = (msUsed - nDay * 86400 - nHour * 3600) / 60;
////		long nSecond = (msUsed - nDay * 86400 - nHour * 3600) % 60;
//		return String.valueOf(nDay) + "天" + String.valueOf(nHour) + "小时";
	}
    
}
