package com.gx.obe.common.file.utils;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import java.util.function.Function;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;

import com.gx.obe.components.core.Constants;
import com.gx.obe.util.utils.StringUtils;

public class FileUtils {

	public FileUtils() {
	}
	 /** 文件大小格式化 */
	public static final String sizeFormat = "##,###,###,###,##0.00";
	
	public static final String FILE_UNIT_KB="KB";
	public static final String FILE_UNIT_GB="GB";
	public static final String	FILE_UNIT_TB="TB";
	public static final String	FILE_UNIT_MB="MB";
	public static final String	FILE_UNIT_BYTE="BYTE"; 
	private static Pattern FilePattern = Pattern.compile("[\\\\/:*?\"<>|]");
	private static Logger log = Logger.getLogger(FileUtils.class);
	
	/** 
	 * @Description:  文件大小格式化
	 * @param   
	 * @return  
	 * @Exception   
	 */
	public static String getFileSize(double size){
		String unit = "KB";
		if(size < 0)size = 0;
		if(size > (1024*1024*1024)){
			unit = "GB";
			size = (double)size/(1024*1024*1024);
		}else if(size > (1024*1024)){
			unit = "MB";
			size = (double)size/(1024*1024);
		}else if(size > (1024)){
			unit = "KB";
			size = (double)size/(1024);
		}else {
			unit = "字节";
		}
		DecimalFormat fmt = new DecimalFormat(sizeFormat);  
    	String outStr = null;
    	outStr = fmt.format(size);   
    	return outStr+unit;
	}
	
	/**
	 * @Description：根据单位类型 文件大小格式化
	 * @author guoyr
	 * @param fileSize 文件大小
	 * @param unit  文件单位 
	 * @return String  文件大小和单位
	 */
	public static String getFileSize(final double fileSize,String unit){
		double resultSize=0;
		 
		if(unit.toUpperCase().equals(FILE_UNIT_BYTE)){
			resultSize=fileSize;
			unit="字节";
		}else if(unit.toUpperCase().equals(FILE_UNIT_KB)){
			resultSize=fileSize/1024;
		}else if(unit.toUpperCase().equals(FILE_UNIT_MB)){
			resultSize=(double)fileSize/(1024*1024);
		}else if(unit.toUpperCase().equals(FILE_UNIT_GB)){
			resultSize=(double)fileSize/(1024*1024*1024);
		}else if(unit.toUpperCase().equals(FILE_UNIT_TB)){
			resultSize=(double)fileSize/(1024*1024*1024*1024);
		}
		DecimalFormat fmt = new DecimalFormat(sizeFormat);  
    	String outStr = null;
    	outStr = fmt.format(resultSize);   
		return outStr+unit;
	}
	
	public static String[] fileFilters = new String[] {".svn", "~"};

	public static boolean removeFilter(File file) {
		boolean flag = false;
		for (String filter : fileFilters) {
			if(file.getName().startsWith(filter)||file.getName().endsWith(filter)){
				flag = true;
				break;
			}
		}
		return flag;
	}
	
	/**
	 * @Description: 获得文件夹的大小
	 * @author guoyr
	 * @param path
	 * @return
	 */
	public static long getDirectorySize(String path) {
		return getDirectorySize(new File(path));
	}

	/**
	 * @Description: 获得文件夹的大小
	 * @author guoyr
	 * @param path
	 * @return
	 */
	public static long getDirectorySize(File dir) {
		if (dir == null || !dir.exists() || !dir.isDirectory()) {
			return 0;
		}
		long dirSize = 0;
		File[] files = dir.listFiles();
		for (File file : files) {
			if (removeFilter(file)) {
				continue;
			}
			if (file.isFile()) {
				dirSize += file.length();
			} else if (file.isDirectory()) {
				dirSize += file.length();
				dirSize += getDirectorySize(file.getAbsolutePath()); // 如果遇到目录则通过递归调用继续统计
			}
		}
		return dirSize;
	}

	/**
	 * @Description: 列出指定路径下的所有文件 Create Date: 2011-4-11下午06:28:02 by guoyr
	 * @author guoyr
	 * @param path
	 *            指定路径
	 * @return 返回文件组成的数组
	 * @Exception
	 */
	public static List<File> listAllFiles(String path) {
		List<File> files = new ArrayList<File>();
		File directory = getDirectory(path);
		if (directory == null)
			return files;
		for (File file : directory.listFiles()) {
			if (removeFilter(file)) {
				continue;
			}
			if (!file.isDirectory()) {
				files.add(file);
			} else {
				files.addAll(listAllFiles(file.getAbsolutePath()));
			}
		}
		return files;
	}

	/**
	 * @Description: 获得文件夹下的文件
	 * @author guoyr
	 * @param path
	 * @return 
	 */
	public static List<File> listFilesByIsFile(String path) {
		List<File> files = new ArrayList<File>();
		File directory = getDirectory(path);
		if (directory == null)
			return null;
		for (File file : directory.listFiles()) {
			if (removeFilter(file)) {
				continue;
			}
			if (!file.isDirectory()) {
				files.add(file);
			}
		}
		return files;
	}
	
	/**
	 * @Description: 列出指定路径下的所有目录 Create Date: 2011-4-11下午06:28:02 by guoyr
	 * @author guoyr
	 * @param path
	 *            指定路径
	 * @return 返回目录组成的数组
	 * @Exception
	 */
	public static List<File> listDirectory(String path) {
		List<File> files = new ArrayList<File>();
		File directory = getDirectory(path);
		if (directory == null)
			return null;
		if(!directory.isDirectory()){
			return null;
		}
		for (File file : directory.listFiles()) {
			if (removeFilter(file)) {
				continue;
			}
			if (file.isDirectory()) {
				files.add(file);
			}
		}
		return files;
	}

	/**
	 * 列出指定路径下的所有文件
	 * @param path
	 *            指定路径
	 * @return 返回文件名组成的数组
	 */
	public static final String[] listFiles(String path) {
		return list(path, false);
	}

	/**
	 * 列出指定路径下的所有目录
	 * @param path
	 *            指定路径
	 * @return 返回目录名组成的数组
	 */
	public static final String[] listDirectorys(String path) {
		return list(path, true);
	}

	/**
	 * 列出指定路径下的所有目录或文件
	 * @param path
	 *            指定路径
	 * @param isDirectory
	 *            :true为返回目录名数组,false返回文件名数组
	 * @return 返回目录名或文件名组成的数组
	 */
	private static final String[] list(String path, boolean isDirectory) {
		File directory = getDirectory(path);
		if (directory == null)
			return null;
		int count = 0;
		File files[] = directory.listFiles();
		String temp[] = new String[files.length];
		for (int i = 0; i < files.length; i++)
			if (files[i].isDirectory() == isDirectory) {
				temp[count] = files[i].getName();
				count++;
			}

		return temp;
	}

	/**
	 * 列出指定路径下的所有目录和文件
	 * @param path
	 *            指定路径
	 * @return 返回目录名和文件名组成的数组
	 */
	public static final String[] list(String path) {
		File file = getDirectory(path);
		if (file != null) {
			String list[] = file.list();
			return list;
		} else {
			return null;
		}
	}

	/**
	 * 根据传入字符串生成路径
	 * @param path
	 *            表示路径字符串
	 * @return 返回解析好的路径，如果无效返回null
	 */
	public static final File getDirectory(String path) {
		File file = new File(path);
		if (file.canRead() && file.isDirectory())
			return file;
		else
			return null;
	}

	public static void create(String path, String content) {
		try {
			rename(path, path + ".bak");
			RandomAccessFile newfile = new RandomAccessFile(path, "rw");
			newfile.writeBytes(content);
			newfile.close();
		} catch (Exception ex) {
			log.error(ex.getMessage() ,ex);
		}
	}

	public static final boolean rename(String orgPathName, String tarPathName) {
		boolean result = false;
		try {
			File file = new File(orgPathName);
			if(file.getAbsolutePath().equals(new File(tarPathName).getAbsolutePath())){
				return true;
			}
			//Java-慎用File.reNameto(),不支持跨平台，不同的分区格式也受到影响
			
			try {
				if(!file.renameTo(new File(tarPathName))){
					FileUtils.copy(orgPathName, tarPathName);
				}
			} catch (Exception e) {
				FileUtils.copy(orgPathName, tarPathName);
				file.delete();
			}
			result = true;
		} catch (Exception e) {
			log.error(e.getMessage() ,e);
		}
		return result;
	}

	public static final boolean delete(String path) {
		try {
			File file = new File(path);
			if(!file.delete()){
				file.deleteOnExit();
			}
		} catch (Exception e) {
			log.error(e.getMessage() ,e);
			return false;
		}
		return true;
	}

	public static final boolean copy(String source, String target) {
		boolean flag = true;
		byte abyte0[] = new byte[4096];
		FileOutputStream fileoutputstream = null;
		FileInputStream fileinputstream = null;
		try {
			File mvSourceFile = new File(source);
			if (mvSourceFile.exists()) {
				File targetDir = new File(target).getParentFile();
				if(!targetDir.exists()){
					targetDir.mkdirs();
				}
				targetDir = null;
				fileoutputstream = new FileOutputStream(target);
				fileinputstream = new FileInputStream(source);
				int i;
				while ((i = fileinputstream.read(abyte0)) != -1){
					fileoutputstream.write(abyte0, 0, i);
				}
				fileoutputstream.flush();
			}
			mvSourceFile = null;
			flag = true;
		} catch (Exception e) {
			flag = false;
			log.error(e.getMessage() ,e);
		}finally{
			if(null != fileinputstream){
				try {
					fileinputstream.close();
					fileinputstream = null;
				} catch (IOException e) {
				}
			}
			if(null != fileoutputstream){
				try {
					fileoutputstream.close();
					fileoutputstream = null;
				} catch (IOException e) {
				}
			}
			
		}
		return flag;
	}

	/**
	 * 复制文件
	 * @param srcPaths的形式可以单个文件也可以是多个文件
	 *            ："c:\doc\1.doc;c:\doc\...\2.txt......;c:\text\t.txt";每个路径以“;”割
	 *            开
	 * @param tarPaths为至指定的文件目录
	 *            ;
	 * @return
	 */
	public static final boolean copyOrCreateFiles(String srcPaths,
			String tarPaths) {
		boolean flag = false;
		byte abyte0[] = new byte[4096];
		String[] src = srcPaths.split(";");
		for (int i = 0; i < src.length; i++) {
			FileOutputStream fileoutputstream = null;
			FileInputStream fileinputstream = null;
			try {
				File mvSourceFile = new File(src[i]);
				File newCreateFile = new File(tarPaths.concat(File.separator).concat(mvSourceFile.getName()));
				if (!newCreateFile.exists()) {
					mkdirs(newCreateFile.getPath().substring(0,
							newCreateFile.getPath().lastIndexOf("\\")));
					newCreateFile.createNewFile();
				}
				if (mvSourceFile.exists()) {
					fileoutputstream = new FileOutputStream(newCreateFile);
					fileinputstream = new FileInputStream(src[i]);
					int j;
					while ((j = fileinputstream.read(abyte0)) != -1){
						fileoutputstream.write(abyte0, 0, j);
					}
					fileoutputstream.flush();
				} 
				mvSourceFile = null;
				flag = true;
			} catch (Exception e) {
				log.error(e.getMessage() ,e);
			}finally{
				if(null != fileinputstream){
					try {
						fileinputstream.close();
						fileinputstream = null;
					} catch (IOException e) {
					}
				}
				if(null != fileoutputstream){
					try {
						fileoutputstream.close();
						fileoutputstream = null;
					} catch (IOException e) {
					}
				}
			}
		}

		return flag;
	}

	public static boolean mkdirs(String path) {
		try {
			File dir = new File(path);
			dir.mkdirs();
		} catch (Exception e) {
			log.error(e.getMessage() ,e);
			return false;
		}
		return true;
	}

	/**
	 * @Description: 从文件流读取文件 Create Date: 2009-11-15上午02:25:55 by wangsw
	 * @author guoyr
	 * @param 文件字符串
	 * @return 返回值说明
	 * @Exception
	 */
	public static String read(InputStream inputStream){
		InputStreamReader ins = null;
		BufferedReader br = null;
		StringBuffer sb = new StringBuffer();
		try {
			ins = new InputStreamReader(inputStream, "UTF-8");
			br = new BufferedReader(ins);
			String text = "";
			
			while ((text = br.readLine()) != null) {
				sb.append(text + "\n");
			}
		} catch (UnsupportedEncodingException e) {
			log.error(e);
		} catch (IOException e) {
			log.error(e);
		}finally{
			if(null != br){
				try {
					br.close();
				} catch (IOException e) {
				}
			}
			if(null != ins){
				try {
					ins.close();
				} catch (IOException e) {
				}
			}
		}
		return sb.toString();
	}

	/**
	 * 读取文件内容
	 * @param fileName
	 *            文件名
	 */
	public static String readFileByLines(String fileName, String...encoding) {
		StringBuffer content = new StringBuffer("");
		if (fileName != null && !"".equals(fileName)) {
			File file = new File(fileName);
			if (null != file && file.exists()) {

				BufferedReader reader = null;
				try {
					reader = new BufferedReader(new InputStreamReader(
							new FileInputStream(file), (null != encoding && encoding.length == 1)?encoding[0]:"UTF-8"));
					String tempString = null;
					/* 一次读入一行，直到读入null为文件结束 */
					while ((tempString = reader.readLine()) != null) {
						content.append(tempString + "\n");
					}
				} catch (IOException e) {
					log.error(e.getMessage(), e);
				} finally {
					if (reader != null) {
						try {
							reader.close();
							reader = null;
						} catch (IOException e1) {
							log.error(e1.getMessage(), e1);
						}
					}
				}
			}
		}
		return content.toString();
	}
	
	/**
	 * @param path
	 *            文件路径
	 * @param suffix
	 *            后缀名
	 * @param isdepth
	 *            是否遍历子目录
	 * @return
	 */
	public static void getListFiles(String path, List<String> fileList,
			String suffix, boolean isdepth) {
		File file = new File(path);
		listFile(file, fileList, suffix, isdepth);
	}

	/**
	 * @Description: 递归遍历子目录，与getListFiles函数以前使用 Create Date:
	 * @author guoyr
	 * wangsw
	 * @param 输入参数按顺序进行说明
	 * @return 返回值说明
	 * @Exception
	 */
	public static List<String> listFile(File f, List<String> fileList,
			String suffix, boolean isdepth) {
		// 是目录，同时需要遍历子目录
		if (f.isDirectory() && isdepth == true) {
			File[] t = f.listFiles();
			for (int i = 0; i < t.length; i++) {
				listFile(t[i], fileList, suffix, isdepth);
			}
		} else {
			String filePath = f.getAbsolutePath();

			if (suffix != null) {
				int begIndex = filePath.lastIndexOf(".");// 最后一个.(即后缀名前面的.)的索引
				String tempsuffix = "";

				if (begIndex != -1)// 防止是文件但却没有后缀名结束的文件
				{
					tempsuffix = filePath.substring(begIndex + 1,
							filePath.length());
				}

				if (tempsuffix.equals(suffix)) {
					fileList.add(filePath);
				}
			} else {
				// 后缀名为null则为所有文件
				fileList.add(filePath);
			}
		}
		return fileList;
	}
	
	/**
	 * @Description:  获得当前路径下的文件
	 * @author guoyr
	 * @param path
	 * @param suffix 指标要获得的后缀
	 * @return
	 */
	public static List<File> getListFiles(String path, String suffix) {
		List<File> fileList = new ArrayList<File>();
		File file = new File(path);
		if(file.exists()){
			return listFile(file, fileList, suffix);
		}else {
			return fileList;
		}
	}
	
	/**
	 * @Description:  递归遍历子目录
	 * @author guoyr
	 * @param f
	 * @param fileList
	 * @param suffix
	 * @return
	 */
	public static List<File> listFile(File f, List<File> fileList,
			String suffix) {
		// 是目录，同时需要遍历子目录
		if (f.isDirectory()) {
			File[] t = f.listFiles();
			for (int i = 0; i < t.length; i++) {
				listFile(t[i], fileList, suffix);
			}
		} else {
			String filePath = f.getAbsolutePath();
			
			if (suffix != null) {
				int begIndex = filePath.lastIndexOf(".");// 最后一个.(即后缀名前面的.)的索引
				String tempsuffix = "";
				
				if (begIndex != -1)// 防止是文件但却没有后缀名结束的文件
				{
					tempsuffix = filePath.substring(begIndex + 1,
							filePath.length());
				}
				if (suffix.contains(tempsuffix)) {
					fileList.add(f);
				}
			} else {
				// 后缀名为null为所有文件
				fileList.add(f);
			}
		}
		return fileList;
	}

	/**
	 * 判断文件是否存在
	 * @param fileName
	 * @return
	 */
	public static boolean fileExist(String fileName) {
		File objFile = new File(fileName);
		if (objFile.exists()) {
			return true;
		} else {
			return false;
		}
	}

	/**
	 * 删除文件夹里面的所有文件
	 * @param path
	 *            String 文件夹路径 如 c:/Myfile
	 */
	public static void delAllFile(String path) {
		File file = new File(path);
		if (!file.exists()) {
			return;
		}
		if (!file.isDirectory()) {
			return;
		}
		System.gc();
		String[] tempList = file.list();
		File temp = null;
		for (int i = 0; i < tempList.length; i++) {
			if (path.endsWith(File.separator)) {
				temp = new File(path + tempList[i]);
			} else {
				temp = new File(path + File.separator + tempList[i]);
			}
			if (temp.isFile()) {
				System.gc();
				temp.delete();
			}
			if (temp.isDirectory()) {
				delAllFile(path + "/" + tempList[i]);// 先删除文件夹里面的文件
				delFolder(path + "/" + tempList[i]);// 再删除空文件夹
			}
		}
	}

	/**
	 * 删除文件夹
	 * @param filePathAndName
	 *            String 文件夹路径及名称 如c:/Myfile
	 * @param fileContent
	 *            String
	 * @return
	 */
	public static void delFolder(String folderPath) {
		delAllFile(folderPath); // 删除完里面所有内容
		String filePath = folderPath;
		filePath = filePath.toString();
		java.io.File myFilePath = new java.io.File(filePath);
		myFilePath.delete(); // 删除空文件夹
	}

	/**
	 * 复制整个文件夹内容
	 * @param oldPath
	 *            String 原文件路径 如：c:/Myfile
	 * @param newPath
	 *            String 复制后路径 如：f:/Myfile/file
	 * @return boolean
	 */
	public static boolean copyFolder(String oldPath, String newPath){
		(new File(newPath)).mkdirs(); // 如果文件夹不存在 则建立新文件夹
		File a = new File(oldPath);
		String[] file = a.list();
		File temp = null;
		boolean copyFlag = true;
		for (int i = 0; i < file.length; i++) {
			if(!copyFlag){
				break;
			}
			if (oldPath.endsWith(File.separator)) {
				temp = new File(oldPath + file[i]);
			} else {
				temp = new File(oldPath + File.separator + file[i]);
			}

			if (temp.isFile()) {
				FileInputStream input = null;
				FileOutputStream output = null;
				try {
					input = new FileInputStream(temp);
					output = new FileOutputStream(newPath + "/"
							+ (temp.getName()).toString());
					byte[] b = new byte[1024 * 5];
					int len;
					while ((len = input.read(b)) != -1) {
						output.write(b, 0, len);
					}
					output.flush();
				} catch (FileNotFoundException e) {
					copyFlag = false;
				} catch (IOException e) {
					copyFlag = false;
				}finally{
					if(null != output){
						try {
							output.close();
						} catch (IOException e) {
						}
					}
					if(null != input){
						try {
							input.close();
						} catch (IOException e) {
						}
					}
				}
			}
			if (temp.isDirectory()) {// 如果是子文件夹
				copyFlag = copyFolder(oldPath + "/" + file[i], newPath + "/" + file[i]);
			}
		}
		return copyFlag;
	}

	public static void main(String[] args) {
		long l=155971776;
		System.out.println(l/(1024*1024));
		System.out.println(getFileSize(l,FILE_UNIT_BYTE));
		System.out.println(getFileSize(l,FILE_UNIT_MB));
//		List<String> f=new ArrayList<String>();
//		String dir="F:\\fileReceive\\402887672fc8dd64012fc96a6c670007";
	//	getDirFiles(dir, f);
		
	/*	System.out.println(" fileSize:"+fileSize.toString()+" filetoal:"+filetotal.longValue());
		for(String str:f){
			System.out.println(str);
		}*/
		
		String parendId="402887672fc8dd64012fc96a6c670007";
		String parentFile="\\402887672fc8dd64012fc96a6c670007\\402887672fc8dd64012fc97aa29d0019\\EvalPoint.xml";
		String key=parentFile.substring(parentFile.indexOf(parendId)+parendId.length()+1,parentFile.length());
		System.out.println(key);
		key=key.substring(0,key.indexOf("\\"));
		System.out.println(key);
		
	}

	/**
	 * 把内容写到文件
	 * @param filePathName
	 *            文件名
	 * @param List
	 *            <String> 文件内容
	 */
	public static boolean writerFile(String filePathName, String content,
			String unicode) {
		boolean flag = false;
		OutputStreamWriter osw = null;
		try {
			if (filePathName != null && !"".equals(filePathName)) {
				osw = new OutputStreamWriter(
						new FileOutputStream(filePathName), unicode);
			}
		} catch (FileNotFoundException e1) {
			flag = false;
			log.error(e1.getMessage() ,e1);
		} catch (UnsupportedEncodingException ue) {
			flag = false;
			log.error(ue.getMessage() ,ue);
		}
		if (osw != null) {
			BufferedWriter bw = new BufferedWriter(osw);
			try {
				if (content != null && !"".equals(content)) {
					bw.write(content);
					bw.flush();
					flag = true;
				}
			} catch (IOException e) {
				flag = false;
				log.error(e.getMessage() ,e);
			} finally {
				try {
					bw.close();
				} catch (IOException e) {
					flag = false;
					log.error(e.getMessage() ,e);
				}
				try {
					osw.close();
				} catch (IOException e) {
					flag = false;
					log.error(e.getMessage() ,e);
				}
			}
		}
		return flag;
	}
	
	/**
	 * @Description：
	 * @author guoyr
	 * @param dir  目录
	 * @param files  文件结果集
	 * @param fileSize  文件大小
	 * @param fileTotal 文件个数
	 */
	public static void getDirFiles(String dir,List<String> files){
		File file=new File(dir);
		getDirFiles(file,files);
	}
	
	public static void getDirFiles(File file,List<String> files){
		if(file.isDirectory()){
			File[] cfile=file.listFiles();
			for(File cf:cfile){
				if(cf.isDirectory()){
					getDirFiles(file, files);
				}else{
					files.add(cf.getAbsolutePath());
				}
			} 
		}else{
			files.add(file.getAbsolutePath());
		}
	}

	/**
	 * @Description: 获得文件或文件夹的大小 Create Date: 2011-3-14下午04:04:17 by guoyr
	 * @author guoyr
	 * @param
	 * @return
	 * @Exception
	 */
	public static long getFileSize(File file) {
		long size = 0;
		if (file.exists() && file.isDirectory()) {
			size += getDirectorySize(file.getAbsolutePath());
		} else if(file.exists() && !removeFilter(file)){
			size += file.length();
		}
		return size;
	} 
	 
	 /**
     * 读取文件某块到字节数据
     * @param startPosition 起始点
     * @param endPosition 结束点
     * @param fileAbsolutePath 文件存放路径
     * @return 读取结果
     * @throws Exception
     */
    public static byte[] randomReadFile(int startPosition, int endPosition, String fileAbsolutePath) throws Exception{
    	if(endPosition < startPosition){
    		throw new Exception("起始地址不能大于结束位置.");
    	}
    	RandomAccessFile readRaf = new RandomAccessFile(fileAbsolutePath, "r");
    	
    	byte[] result = new byte[endPosition - startPosition];
    	readRaf.seek(startPosition);
    	readRaf.read(result);
    	
    	readRaf.close();
    	return result;
    }
    
    /**
	 * 获取资源根路径
	 * @return
	 */
	public static String getResourcesPath(){
		URL url=Thread.currentThread().getContextClassLoader().getResource("");
		String path=url.getPath().substring(1, url.getPath().length());
		String rootpath=path.substring(0, path.indexOf("bin/"));
		return rootpath;
	}
	
	/**
	 * 根据文件路径得到文件
	 * @param path
	 * @return
	 */
	public static File getFile(String path){
		File file=new File(path);
		if(file.exists()){
			return file;
		}
		return null;
	}
	
	/**
	 * 根据文件路径得到properties
	 * @param path
	 * @return
	 * @throws IOException
	 */
	public static Properties getProperties(String path) throws IOException{
		 InputStream in = new BufferedInputStream(new FileInputStream(path));
		 Properties p = new Properties();
		 p.load(in);
		 return p;
	}
	
	/**
	 * 验证文件是否在指定的目录下
	 * @param r
	 * @return
	 */
	public static File checkFiles(String[] paths,String fileName){
		List<File> listFile=new ArrayList<File>();
		for (String path : paths) {
			File[] files=new File(path).listFiles();
			if(null != files && files.length> 0)
			for (File file : files) {
				listFile.add(file);
			}
		}
		File res=null;
		for (File f : listFile) {
			if(fileName.equals(f.getName().substring(0, f.getName().lastIndexOf(".")))){
				res=f;
				break;
			}else{
				res=null;
			}
		}
		return res;
	}

	/**
	 * 非法文件名过滤
	 * @param fileName
	 * @return
	 */
	public static String filterFileName(String fileName){
		return fileName==null?null:FilePattern.matcher(fileName.trim()).replaceAll("");
	}
	
	public static Function<String, String> funGetSuffix = fileName -> {
		if (fileName == null) return "";
		int indexOf = fileName.lastIndexOf(".");
		if (indexOf == -1) return "";
		return fileName.substring(indexOf + 1);
	};
	/**
	 * @Description: 去除文件名.enc后缀
	 * @author chenxw
	 * @param fileName
	 * @return
	 */
	public static String removeFileNameENC(String fileName) {
		if (StringUtils.isEmpty(fileName)) {
			return fileName;
		}
		if (fileName.endsWith(Constants.ENC)) {
			return fileName.substring(0, fileName.lastIndexOf(Constants.ENC));
		}
		return fileName;
	}

	/**
	 * @Description: 获取文件名后部
	 * @author chenxw
	 * @param fileName
	 * @return
	 */
	public static String getFileNameBack(String fileName) {
		if (StringUtils.isEmpty(fileName)) {
			return fileName;
		}
		int lastIndex = fileName.lastIndexOf(".");
		if (lastIndex == -1) {
			return "";
		}
		return fileName.substring(lastIndex);
	}
	/**
	 * @Description: 获取文件名前部
	 * @author chenxw
	 * @param fileName
	 * @return
	 */
	public static String getFileNameFront(String fileName) {
		if (StringUtils.isEmpty(fileName)) {
			return fileName;
		}
		int lastIndex = fileName.lastIndexOf(".");
		if (lastIndex == -1) {
			return fileName;
		}
		return fileName.substring(0, lastIndex);
	}
	
}
