package com.gx.obe.common.file.utils;

import java.io.File;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.dom4j.Document;
import org.dom4j.Element;

import com.gx.obe.business.entity.ScoreItem;
import com.gx.obe.common.file.entity.BidFile;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.components.core.AIDataParams;
import com.gx.obe.components.core.enumeration.EvaluationFactorEnum;
import com.gx.obe.components.core.enumeration.StructEnum;
import com.gx.obe.components.core.vo.BusinessLicense;
import com.gx.obe.components.core.vo.Finance;
import com.gx.obe.components.core.vo.Performance;
import com.gx.obe.components.core.vo.PerformanceDetails;
import com.gx.obe.components.core.vo.PerformanceDetails.Detail;
import com.gx.obe.components.core.vo.Qualification;
import com.gx.obe.components.core.vo.TableXML;
import com.gx.obe.formula.AvgFloatRangeItem;
import com.gx.obe.formula.BasePriceRangeParam;
import com.gx.obe.formula.BenchmarkPriceParam;
import com.gx.obe.formula.CalculationParam;
import com.gx.obe.formula.DeviatePriceItem;
import com.gx.obe.formula.DeviateRateItem;
import com.gx.obe.formula.FilterCountConditionItem;
import com.gx.obe.formula.PriceParam;
import com.gx.obe.formula.SpecimenFilterConditionItem;
import com.gx.obe.util.utils.BigDecimalUtils;
import com.gx.obe.util.utils.CollectionUtils;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.SerializeUtils;
import com.gx.obe.util.utils.StringUtils;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.web.entity.AttachmentFile;
import com.gx.obe.web.entity.Struct;
import com.gx.obe.web.entity.evaluation.EvaluationContent;
import com.gx.obe.web.entity.evaluation.EvaluationFactor;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.price.BidPrice;
import com.gx.obe.web.entity.tender.TenderProjectEntity;

public class ParsingXmlUtils {
	
	private static int sorNo = 1;
	
	private static AnalyPackCodeElementListener analyPackCodeElementListener;
	
	public static void addAnalyPackCodeElementListener(AnalyPackCodeElementListener listener) {
		ParsingXmlUtils.analyPackCodeElementListener = listener;
	}
	
	public interface AnalyPackCodeElementListener {
		void analy(Element packElement, TenderProjectEntity tenderProjectEntity);
	}
	
	
	
	/**
	 * @Description: 解析表格xml
	 * @author chenxw
	 * @param tableXMLPath
	 * @return
	 */
	public static TableXML parsingTableXML(String tableXMLPath) {
		File xmlFile = new File(tableXMLPath);
		if(tableXMLPath.contains("4fbf7eab25f645d8b5937d7e022d2f1a.xml")) {
			System.out.println(tableXMLPath);
			
		}
		if (!xmlFile.isFile() || !xmlFile.exists()) {
			return null;
		}
		Document document = XmlUtils.getXmlDocument(xmlFile.getAbsolutePath());
		if (null == document) {
			return null;
		}
		TableXML TableXML1 = TableXML.newInstance(document.getRootElement());
//		return TableXML.newInstance(document.getRootElement());
		return TableXML1;
	}
	
	/**
	 * @Description: 解析结构化XML
	 * @author chenxw
	 * @param tenderId
	 * @param supplierId
	 * @param structType
	 * @return
	 */
	public static List<Struct> parsingStructXML(String tenderId, String supplierId, String factorId, String structType) {
		if (StringUtils.isEmpty(tenderId) || StringUtils.isEmpty(supplierId)) {
			return null;
		}
		String bidFilePath = FileConstants.getBidFilePath(tenderId, supplierId);
		String fileName = getStructFileName(bidFilePath, structType);
		if (StringUtils.isEmpty(fileName)) {
			return null;
		}
		File xmlFile = new File(bidFilePath.concat(fileName));
		if (!xmlFile.exists() || !xmlFile.isFile() || !xmlFile.getName().endsWith(FileConstants.XML)) {
			return null;
		}
		Document document = XmlUtils.getXmlDocument(xmlFile.getAbsolutePath());
		if (null == document) {
			return null;
		}
		String structName = getStructLabelName(structType);
		if (StringUtils.isEmpty(structName)) {
			return null;
		}
		List<?> objectList = document.getRootElement().elements(structName);
		if (CollectionUtils.isNull(objectList)) {
			return null;
		}
		List<Struct> structList = new ArrayList<Struct>();
		for (Object object : objectList) {
			Struct struct = getStruct((Element) object, structType);
			if (null == struct) {
				continue;
			}
			struct.setTenderId(tenderId);
			struct.setSupplierId(supplierId);
			struct.setFactorId(factorId);
			struct.setStructType(structType);
			structList.add(struct);
		}
		return structList;
	}
	
	/**
	 * @Description: 获取结构化文件名称
	 * @author chenxw
	 * @param bidFilePath
	 * @param structurizationType
	 * @return
	 */
	private static String getStructFileName(String bidFilePath, final String structurizationType) {
		if (StringUtils.isEmpty(bidFilePath) || StringUtils.isEmpty(structurizationType)) {
			return null;
		}
		File xmlFile = new File(bidFilePath + FileConstants.BID_FILE_INDEX_XML);
		if (!xmlFile.exists() || !xmlFile.isFile()) {
			return null;
		}
		Document document = XmlUtils.getXmlDocument(xmlFile.getAbsolutePath());
		if (null == document) {
			return null;
		}
		@SuppressWarnings("unchecked")
		List<Element> fileItemList = document.getRootElement().elements("fileItem");
		Element fileItem = CollectionUtils.get(fileItemList, t -> {
			return structurizationType.equals(t.elementTextTrim("chapterType"));
		});
		return BidFile.getBidFileName(fileItem);
	}
	
	/**
	 * @Description: 获取结构化标签名
	 * @author chenxw
	 * @param structType
	 * @return
	 */
	private static String getStructLabelName(String structType) {
		if (StringUtils.isEmpty(structType)) {
			return null;
		}
		String structName = null;
		if (structType.startsWith(StructEnum.EQ_BUSINESS_LICENSE)) {
			structName = "BusinessLicense";
		} else if (structType.startsWith(StructEnum.EQ_FINANCIAL_REPORT)) {
			structName = "Finance";
		} else if (structType.startsWith(StructEnum.EQ_ACHIEVEMENTS)) {
			structName = "Performance";
		} else if (structType.startsWith(StructEnum.EQ_QUALIFICATION)) {
			structName = "Qualification";
		}
		return structName;
	}
	
	/**
	 * @Description: 获取结构化数据
	 * @author chenxw
	 * @param structElement
	 * @param structType
	 * @return
	 */
	private static Struct getStruct(Element structElement, String structType) {
		if (StringUtils.isEmpty(structType)) {
			return null;
		}
		Struct struct = new Struct();
		if (structType.startsWith(StructEnum.EQ_BUSINESS_LICENSE)) {
			struct.setBusinessLicense(getBusinessLicense(structElement));
		} else if (structType.startsWith(StructEnum.EQ_FINANCIAL_REPORT)) {
			struct.setFinance(getFinance(structElement));
		} else if (structType.startsWith(StructEnum.EQ_ACHIEVEMENTS)) {
			struct.setPerformance(getPerformance(structElement));
		} else if (structType.startsWith(StructEnum.EQ_QUALIFICATION)) {
			struct.setQualification(getQualification(structElement));
		} else {
			return null;
		}
		
		struct.setQuaType(structElement.elementTextTrim("quaType"));
		struct.setAttachmentFileList(getScanCertificate(structElement));
		
		return struct;
	}
	
	/**
	 * @Description: 获取营业执照数据
	 * @author chenxw
	 * @param element
	 * @return
	 */
	private static BusinessLicense getBusinessLicense(Element element) {
		if (null == element) {
			return null;
		}
		BusinessLicense businessLicense = new BusinessLicense();
		businessLicense.setRegisterNumber(element.elementTextTrim("registerNumber"));
		businessLicense.setCompanyName(element.elementTextTrim("companyName"));
		businessLicense.setCompanyType(element.elementTextTrim("companyType"));
		businessLicense.setCompanyAddress(element.elementTextTrim("companyAddress"));
		businessLicense.setLegalRepresentative(element.elementTextTrim("legalRepresentative"));
		businessLicense.setRegisterCapital(element.elementTextTrim("registerCapital"));
		businessLicense.setEstablishDate(DateUtils.toDate(element.elementTextTrim("establishDate"), "yyyy年MM月dd日"));
		businessLicense.setBusnissTermStartDate(DateUtils.toDate(element.elementTextTrim("busnissTermStartDate"), "yyyy年MM月dd日"));
		businessLicense.setBusnissTermEndDate(DateUtils.toDate(element.elementTextTrim("busnissTermEndDate"), "yyyy年MM月dd日"));
		businessLicense.setBusinessScope(element.elementTextTrim("businessScope"));
		businessLicense.setRegisterInstitution(element.elementTextTrim("registerInstitution"));
		businessLicense.setPerpetual("true".equalsIgnoreCase(element.elementTextTrim("isPerpetual")));
		businessLicense.setInAreaEnterprises("true".equalsIgnoreCase(element.elementTextTrim("isInAreaEnterprises")));
		return businessLicense;
	}
	
	/**
	 * @Description: 获取财务报告数据
	 * @author chenxw
	 * @param element
	 * @return
	 */
	private static Finance getFinance(Element element) {
		if (null == element) {
			return null;
		}
		Finance finance = new Finance();
		finance.setAnnual(element.elementTextTrim("annual"));
		finance.setTotal(ObjectUtils.getObjBigDecimal(element.elementTextTrim("total")));
		finance.setLiabilities(ObjectUtils.getObjBigDecimal(element.elementTextTrim("liabilities")));
		finance.setIncome(ObjectUtils.getObjBigDecimal(element.elementTextTrim("income")));
		finance.setMainIncome(ObjectUtils.getObjBigDecimal(element.elementTextTrim("mainIncome")));
		finance.setProfit(ObjectUtils.getObjBigDecimal(element.elementTextTrim("profit")));
		finance.setLiabilitiesRate(ObjectUtils.getObjBigDecimal(element.elementTextTrim("liabilitiesRate")));
		finance.setCurrentAssets(ObjectUtils.getObjBigDecimal(element.elementTextTrim("currentAssets")));
		finance.setCurrentLiabilities(ObjectUtils.getObjBigDecimal(element.elementTextTrim("currentLiabilities")));
		finance.setCurrentRate(ObjectUtils.getObjBigDecimal(element.elementTextTrim("currentRate")));
		finance.setOperatingMargin(ObjectUtils.getObjBigDecimal(element.elementTextTrim("operatingMargin")));
		finance.setPaidUpCapital(ObjectUtils.getObjBigDecimal(element.elementTextTrim("paidUpCapital")));
		finance.setRetainedProfits(ObjectUtils.getObjBigDecimal(element.elementTextTrim("retainedProfits")));
		finance.setMainIncomeProfits(ObjectUtils.getObjBigDecimal(element.elementTextTrim("mainIncomeProfits")));
		return finance;
	}
	
	/**
	 * @Description: 获取供货记录数据
	 * @author chenxw
	 * @param element
	 * @return
	 */
	private static Performance getPerformance(Element element) {
		if (null == element) {
			return null;
		}
		Performance performance = new Performance();
		performance.setPurchasingUnit(element.elementTextTrim("purchasingUnit"));
		performance.setPartner(element.elementTextTrim("partner"));
		performance.setSingningDate(DateUtils.toDate(element.elementTextTrim("singningDate"), "yyyy年MM月dd日"));
		performance.setSingningTotal(ObjectUtils.getObjBigDecimal(element.elementTextTrim("singningTotal")));
		performance.setPurchaser(element.elementTextTrim("purchaser"));
		performance.setPurchaserPhone(element.elementTextTrim("purchaserPhone"));
		performance.setAuditStatus(element.elementTextTrim("auditStatus"));
		
		Element detailsListElement = element.element("detailsList");
		if (null != detailsListElement) {
			List<?> detailsList = detailsListElement.elements("details");
			if (CollectionUtils.isNotNull(detailsList)) {
				for (Object object : detailsList) {
					Element details = (Element) object;
					performance.getDetailsList().add(getPerformanceDetails(details));
				}
			}
		}
		return performance;
	}
	
	/**
	 * @Description: 获取供货记录明细数据
	 * @author chenxw
	 * @param details
	 * @return
	 */
	private static PerformanceDetails getPerformanceDetails(Element details) {
		PerformanceDetails performanceDetails = new PerformanceDetails();
		performanceDetails.setId(details.elementTextTrim("id"));
		performanceDetails.setMemo(details.elementTextTrim("memo"));
		performanceDetails.setUnit(details.elementTextTrim("unit"));
		
		Element detaillistElement = details.element("detaillist");
		if (null != detaillistElement) {
			List<?> detailList = detaillistElement.elements("detail");
			if (CollectionUtils.isNotNull(detailList)) {
				for (Object object : detailList) {
					Element detailElement = (Element) object;
					Detail detail = performanceDetails.newDetail();
					detail.setGoodsName(detailElement.elementTextTrim("goodsName"));
					detail.setGoodsUnit(detailElement.elementTextTrim("goodsUnit"));
					detail.setGoodsQuantity(ObjectUtils.getObjBigDecimal(detailElement.elementTextTrim("goodsQuantity")));
					performanceDetails.getDetailList().add(detail);
				}
			}
		}
		return performanceDetails;
	}
	
	/**
	 * @Description: 获取企业资质数据
	 * @author chenxw
	 * @param element
	 * @return
	 */
	private static Qualification getQualification(Element element) {
		if (null == element) {
			return null;
		}
		Qualification qualification = new Qualification();
		qualification.setCertificateName(element.elementTextTrim("certificateName"));
		qualification.setAwardProjectName(element.elementTextTrim("awardProjectName"));
		qualification.setCompanyName(element.elementTextTrim("companyName"));
		qualification.setAwardLevel(element.elementTextTrim("awardLevel"));
		qualification.setCertificateNumber(element.elementTextTrim("certificateNumber"));
		qualification.setIssueAuthority(element.elementTextTrim("issueAuthority"));
		qualification.setIssueDate(element.elementTextTrim("issueDate"));
		qualification.setPeriodValidity(DateUtils.toDate(element.elementTextTrim("periodValidity"), "yyyy年MM月dd日"));
		qualification.setPeriodStartValidity(DateUtils.toDate(element.elementTextTrim("periodStartValidity"), "yyyy年MM月dd日"));
		qualification.setAuditStatus(element.elementTextTrim("auditStatus"));
		qualification.setIsPerpetual(element.elementTextTrim("isPerpetual"));
		return qualification;
	}
	
	/**
	 * @Description: 获取扫描件集合
	 * @author chenxw
	 * @param element
	 * @return
	 */
	private static List<AttachmentFile> getScanCertificate(Element element) {
		Element scanCertificate = element.element("scanCertificate");
		if (null == scanCertificate) {
			scanCertificate = element.element("scanFinance");
		}
		if (null == scanCertificate) {
			scanCertificate = element.element("scanSingning");
		}
		if (null == scanCertificate) {
			return null;
		}
		List<?> objectList = scanCertificate.elements("attachmentFileWithSign");
		if (CollectionUtils.isNull(objectList)) {
			return null;
		}
		List<AttachmentFile> attachmentFileList = new ArrayList<AttachmentFile>();
		for (Object object : objectList) {
			Element attachmentFile = (Element) object;
			attachmentFileList.add(getAttachmentFile(attachmentFile));
		}
		return attachmentFileList;
	}
	
	/**
	 * @Description: 获取扫描件数据
	 * @author chenxw
	 * @param element
	 * @return
	 */
	private static AttachmentFile getAttachmentFile(Element element) {
		String downloadUrl = ObjectUtils.getObjString(element.elementTextTrim("DownloadUrl"));
		AttachmentFile attachmentFile = new AttachmentFile(downloadUrl);
		attachmentFile.setDownloadUrl(downloadUrl);
		attachmentFile.setFileName(element.elementTextTrim("FileName"));
		attachmentFile.setFullFileName(element.elementTextTrim("FullFileName"));
		attachmentFile.setFileId(element.elementTextTrim("FileId"));
		return attachmentFile;
	}
	
	/**
	 * @Description: 解析指标文件
	 * @author guoyr
	 * @param factorFilePath 指标文件路径
	 * @param tenderProjectEntity
	 * @return
	 * @throws Exception
	 */
	public static List<EvaluationFactor> parsingEvaluationFactorXML(String factorFilePath, TenderProjectEntity tenderProjectEntity) throws Exception {
		// 获得投标文件的指标Document
		Document factorDoc = XmlUtils.getXmlDocument(factorFilePath);
		if (null == factorDoc) {
			return null;
		}
		return parsingEvaluationFactorXML(factorDoc, tenderProjectEntity);
	}
	
	/**
	 * @Description: 解析指标文件
	 * @author chenxw
	 * @param factorDoc
	 * @param tenderProjectEntity
	 * @return
	 * @throws Exception
	 */
	public static List<EvaluationFactor> parsingEvaluationFactorXML(Document factorDoc, TenderProjectEntity tenderProjectEntity) throws Exception {
		List<EvaluationFactor> factorList = new ArrayList<EvaluationFactor>();
		if (null != factorDoc) {
			Element root = factorDoc.getRootElement();
			Element packEvalFactorListElement = root.element("packEvalFactorList");
			if (null != packEvalFactorListElement) {
				//分包文件解析
				List<?> packEvalFactorsElements = packEvalFactorListElement.elements();
				if (null != packEvalFactorsElements && packEvalFactorsElements.size() > 0) {
					for (Object object : packEvalFactorsElements) {
						Element packsElement = (Element) object;
						String packCode = packsElement.elementTextTrim("packCode");
						if (null == packCode || !packCode.equals(tenderProjectEntity.getTenderNo())) {
							continue;
						}
						factorList = analyELementXMl(packsElement, tenderProjectEntity);
					}
					//如果没有对应分包取第一个
					if (factorList.size() == 0) {
						factorList = analyELementXMl((Element) packEvalFactorsElements.get(0), tenderProjectEntity);
					}
				}
			} else {
				factorList = analyELementXMl(root, tenderProjectEntity);
			}
		}
		return factorList;
	}
	
	public static List<EvaluationFactor> parsingSupplierEvaluationFactorXML(Document factorDoc, TenderProjectEntity tenderProjectEntity, EvaluationStep evaluationStep) throws Exception {
		List<EvaluationFactor> factorList = new ArrayList<EvaluationFactor>();
		if (null != factorDoc) {
			Element root = factorDoc.getRootElement();
			String factorType = evaluationStep.getStepType();
			List<?> childElementList = null;
			if (com.gx.obe.components.core.enumeration.EvaluationStepEnum.COMPLIANCE.equals(factorType)) {
				Element complianceFactorRoot = root.element("complianceFactors");
				childElementList = null != complianceFactorRoot ? complianceFactorRoot.elements() : null;
			} else if (com.gx.obe.components.core.enumeration.EvaluationStepEnum.DETAIL.equals(factorType)) {
				Element scoreFactorsRoot = root.element("scoreFactors");
				childElementList = null != scoreFactorsRoot ? scoreFactorsRoot.elements() : null;
			}
			factorList = setChildList(childElementList, null, factorType, tenderProjectEntity);
		}
		return factorList;
	}
	
	/**
	 * @Description: 解析指标节点信息
	 * @author chenxw
	 * @param packElement
	 * @param tenderProjectEntity
	 * @return List<EvaluationFactor>
	 * @throws Exception
	 */
	public static List<EvaluationFactor> analyELementXMl(Element packElement, TenderProjectEntity tenderProjectEntity) throws Exception {
		
		analyPackCodeElementListener.analy(packElement, tenderProjectEntity);
		
		List<EvaluationFactor> factorList = new ArrayList<EvaluationFactor>();
		
		sorNo = 1;
		//符合性评审指标
		Element complianceFactorRoot = packElement.element("complianceFactors");
		List<?> complianceElementList = null != complianceFactorRoot ? complianceFactorRoot.elements() : null;
		List<EvaluationFactor> complianceFactorList = setChildList(complianceElementList, null, EvaluationFactorEnum.AUDIT_COMPLIANCE, tenderProjectEntity);
		if (null != complianceFactorList && complianceFactorList.size() > 0) {
			factorList.addAll(complianceFactorList);
		}
		sorNo = 1;
		//详细评审
		Element scoreFactorsRoot = packElement.element("scoreFactors");
		List<?> scoreFactorsElementList = null != scoreFactorsRoot ? scoreFactorsRoot.elements() : null;
		List<EvaluationFactor> scoreFactorList = setChildList(scoreFactorsElementList, null, EvaluationFactorEnum.AUDIT_SCORE, tenderProjectEntity);
		if (null != scoreFactorList && scoreFactorList.size() > 0) {
			factorList.addAll(scoreFactorList);
		}
		sorNo = 1;
		// 价格得分评审
		Element priceFactorsRoot = packElement.element("priceFactors");
		List<?> priceFactorsElementList = null != priceFactorsRoot ? priceFactorsRoot.elements() : null;
		List<EvaluationFactor> priceFactorList = setChildList(priceFactorsElementList, null, EvaluationFactorEnum.AUDIT_PRICE, tenderProjectEntity);
		if (null != priceFactorList && priceFactorList.size() > 0) {
			EvaluationFactor priceFactor = priceFactorList.get(0);
			if (null == priceFactor.getBidPriceCode()) {
				priceFactor.setBidPriceCode(BidPrice.BID_PRICE_TOTAL_CODE);
			}
			factorList.addAll(priceFactorList);
		}
		// 清单指标
		Element boqFactorsRoot = packElement.element("boqFactors");
		List<?> boqFactorsElementList = null != boqFactorsRoot ? boqFactorsRoot.elements() : null;
		List<EvaluationFactor> boqFactorsList = setChildList(boqFactorsElementList, null, EvaluationFactorEnum.AUDIT_BILL, tenderProjectEntity);
		if (null != boqFactorsList && boqFactorsList.size() > 0) {
			factorList.addAll(boqFactorsList);
		}
		return factorList;
	}
	
	/**
	 * @Description: 递归循环遍历子节点
	 * @author chenxw
	 * @param childElementList
	 * @param parentFactor
	 * @param factorType
	 * @param tenderProjectEntity
	 * @return List<EvaluationFactor>
	 * @throws Exception
	 */
	public static List<EvaluationFactor> setChildList(List<?> childElementList, EvaluationFactor parentFactor, String auditType, TenderProjectEntity tenderProjectEntity) throws Exception {
		if (null == childElementList || childElementList.size() == 0) {
			return null;
		}
		List<EvaluationFactor> factorList = new ArrayList<EvaluationFactor>();
		for (Object object : childElementList) {
			Element element = (Element) object;
			EvaluationFactor evaluationFactor = resolverEvaluationFactorFromElement(UuidUtils.getUUID(), parentFactor, element, auditType, tenderProjectEntity);
			if (null != evaluationFactor) {
				if(null != element.element("childFactorList")){
					List<?> childrenElementList = element.element("childFactorList").elements();
					List<EvaluationFactor> childFactorList = setChildList(childrenElementList, evaluationFactor, auditType, tenderProjectEntity);
					if (null != parentFactor) {
						if (null != parentFactor.getScoreType() && parentFactor.getScoreType().equals(EvaluationFactorEnum.SCORE_TYPE_PASS)) {
							evaluationFactor.setScoreType(EvaluationFactorEnum.SCORE_TYPE_PASS);
						}
						parentFactor.addChild(evaluationFactor);
						evaluationFactor.setSortNo(parentFactor.getChildFactorList().size());
					} else {
						evaluationFactor.setSortNo(sorNo++);
					}
					factorList.add(evaluationFactor);
					if (null != childFactorList && childFactorList.size() > 0) {
						factorList.addAll(childFactorList);
					}
				}
				if (null != auditType && auditType.equals(EvaluationFactorEnum.AUDIT_COMPLIANCE)) {
					// 符合性指标
				} else {
					if (null == evaluationFactor.getScoreType() || evaluationFactor.getScoreType().length() == 0) {
						evaluationFactor.setScoreType(EvaluationFactorEnum.SCORE_TYPE_DIRECT);
					}
					// 打分指标和价格指标
					if (null != evaluationFactor.getChildFactorList() && evaluationFactor.getChildFactorList().size() > 0) {
						if (null == evaluationFactor.getFactorScore() || evaluationFactor.getFactorScore().compareTo(BigDecimal.ZERO) == 0) {
							BigDecimal totalScore = BigDecimal.ZERO;
							for (EvaluationFactor childEvaluationFactor : evaluationFactor.getChildFactorList()) {
								totalScore = BigDecimalUtils.add(totalScore, childEvaluationFactor.getFactorScore());
							}
							evaluationFactor.setFactorScore(totalScore);
						}
					}
					if (null == evaluationFactor.getFactorMaxScore() || evaluationFactor.getFactorMaxScore().compareTo(BigDecimal.ZERO) == 0) {
						evaluationFactor.setFactorMaxScore(evaluationFactor.getFactorScore());
					}
					if(null != auditType && auditType.equals(EvaluationFactorEnum.AUDIT_BILL)){
						evaluationFactor.setFactorMaxScore(ObjectUtils.getObjBigDecimal(element.elementTextTrim("maxScore")));
						evaluationFactor.setFactorScore(ObjectUtils.getObjBigDecimal(element.elementTextTrim("score")));
//						factorList.add(evaluationFactor);
					}
				}
			}
		}
		return factorList;
	}
	
	/**
	 * @Description: 解析xml保存指标信息
	 * @author chenxw
	 * @param id
	 * @param parentEvaluationFactor
	 * @param element
	 * @param factorType
	 * @param tenderProjectEntity
	 * @return EvaluationFactor
	 */
	private static EvaluationFactor resolverEvaluationFactorFromElement(String id, EvaluationFactor parentEvaluationFactor, Element element, String auditType, TenderProjectEntity tenderProjectEntity) throws Exception {
		EvaluationFactor evaluationFactor = new EvaluationFactor();
		evaluationFactor.setId(id);
		evaluationFactor.setAuditType(auditType);
		evaluationFactor.setTenderId(tenderProjectEntity.getId());
		evaluationFactor.setFactorName(element.elementTextTrim("name"));
		evaluationFactor.setEvalContent(element.elementTextTrim("evalContent"));
		evaluationFactor.setDataType(element.elementTextTrim("relChapterType"));
		evaluationFactor.setMemo(element.elementTextTrim("memo"));
		
		String factorCode = element.elementTextTrim("code");
		if (null == factorCode || factorCode.isEmpty()) {
			factorCode = element.elementTextTrim("id");
		}
		if (null == factorCode || factorCode.isEmpty()) {
			factorCode = id;
		}
		evaluationFactor.setFactorCode(factorCode);
		
		String evalBidType = element.elementTextTrim("evalBidType");
		if (null != parentEvaluationFactor) {
			evaluationFactor.setParentId(parentEvaluationFactor.getId());
			if (null == evalBidType || evalBidType.length() == 0) {
				evalBidType = parentEvaluationFactor.getEvalBidType();
			}
		}
		evaluationFactor.setEvalBidType(evalBidType);
		
		int evalGroup = ObjectUtils.getIntValue(element.elementTextTrim("expertGroup"), 0);
		if (evalGroup != 0) evaluationFactor.setEvalGroup(evalBidType + "_" + evalGroup);
		
		if (null != auditType) {
			if (auditType.equals(EvaluationFactorEnum.AUDIT_COMPLIANCE)) {
				evaluationFactor.setObjective(element.elementTextTrim("objective"));
			} else {
				evaluationFactor.setFactorMinScore(ObjectUtils.getObjBigDecimal(element.elementTextTrim("minScore")));
				evaluationFactor.setFactorMaxScore(ObjectUtils.getObjBigDecimal(element.elementTextTrim("maxScore")));
				evaluationFactor.setScoreStatus(element.elementTextTrim("scoreStatus"));
				
				// 如果评标类型是综合信用标，则将指标评审类型设置为综合信用指标
				if (null != evalBidType && evalBidType.equals(EvaluationFactorEnum.EVAL_BID_CREDIT)) {
					evaluationFactor.setAuditType(EvaluationFactorEnum.AUDIT_CREDIT);
				}
				
				String scoreValue = element.elementTextTrim("score");
				if (StringUtils.isEmpty(scoreValue)) {
					scoreValue = "0";
				}
				evaluationFactor.setFactorScore(ObjectUtils.getObjBigDecimal(scoreValue));
				
				if (null == evaluationFactor.getFactorMinScore()) {
					evaluationFactor.setFactorMinScore(BigDecimal.ZERO);
				}
				if (null == evaluationFactor.getFactorMaxScore()) {
					evaluationFactor.setFactorMaxScore(evaluationFactor.getFactorScore());
				}
				if (null != element.elementTextTrim("weight")) {
					evaluationFactor.setFactorWeight(ObjectUtils.getObjBigDecimal(element.elementTextTrim("weight")));
				}
				
				if (auditType.equals(EvaluationFactorEnum.AUDIT_SCORE)) {
					evaluationFactor.setObjective(element.elementTextTrim("objective"));
					resolverScoreFactorParams(evaluationFactor, element);
					resolverScoreFactordataParams(evaluationFactor, element);
				} else if (auditType.equals(EvaluationFactorEnum.AUDIT_PRICE)) {
					evaluationFactor.setBidPriceCode(element.elementTextTrim("bidPriceCode"));
					resolverPriceFactorParams(evaluationFactor, element, tenderProjectEntity);
				}
			}
		}
		resolverEvaluationContent(evaluationFactor, element);
		return evaluationFactor;
	}
	/**
	 * @Description: 解析评审内容
	 * @author chenxw
	 * @param evaluationFactor
	 * @param element 
	 */
	private static void resolverEvaluationContent(EvaluationFactor evaluationFactor, Element element) {
		resolverEvaluationContent(evaluationFactor.getTenderId(), evaluationFactor.getFactorCode(), element.element("evaluationContentList"))
			.ifPresent(evaluationFactor::setEvaluationContentList);
	}

	/**
	 * @Description: 解析评审内容
	 * @author chenxw
	 * @param tenderId
	 * @param factorCode
	 * @param evaluationContentListElement
	 * @return
	 */
	public static Optional<List<EvaluationContent>> resolverEvaluationContent(String tenderId, String factorCode, Element evaluationContentListElement) {
		return Optional.ofNullable(evaluationContentListElement).map(t -> t.elements("evaluationContent")).map(l -> {
			return streamElement(l).map(ParsingXmlUtils::initEvaluationContent).peek(t -> {
				t.setTenderId(tenderId);
				t.setFactorCode(factorCode);
			}).collect(Collectors.toList());
		});
	}
	private static Stream<Element> streamElement(List<?> list) {
		return list.stream().map(t -> (Element) t);
	}
	/**
	 * @Description: 初始化评审内容
	 * @author chenxw
	 * @param evaluationContentElement
	 * @return
	 */
	private static EvaluationContent initEvaluationContent(Element evaluationContentElement) {
		EvaluationContent evaluationContent = new EvaluationContent();
		evaluationContent.setId(UuidUtils.getUUID());
		evaluationContent.setRelChapterType(evaluationContentElement.elementTextTrim("relChapterType"));
		evaluationContent.setDataCategory(evaluationContentElement.elementTextTrim("dataCategory"));
		evaluationContent.setDataCode(evaluationContentElement.elementTextTrim("dataCode"));
		evaluationContent.setEvalRule(evaluationContentElement.elementTextTrim("evalRule"));
		evaluationContent.setTenderStructName(evaluationContentElement.elementTextTrim("tenderStructName"));
		evaluationContent.setEvalPointName(evaluationContentElement.elementTextTrim("evalPointName"));
		try {
			evaluationContent.setSortNo(Integer.valueOf(evaluationContentElement.elementTextTrim("Order")));
		} catch (Exception e) {}
		return evaluationContent;
	}
	/**
	 * @Description: 解析结构化数据
	 * @author chenxw
	 * @param evaluationFactor
	 * @param element
	 */
	private static void resolverScoreFactordataParams(EvaluationFactor evaluationFactor, Element element) {
		Element dataParam = element.element("dataParam");
		if (null == dataParam) return;
		AIDataParams dataParams = new AIDataParams();
		List<?> objectList = dataParam.elements();
		if (CollectionUtils.isNotNull(objectList)) {
			for (Object object : objectList) {
				Element param = (Element) object;
				if (param.getName().equals("conditionList")) {
					continue;
				}
				if (param.getName().equals("productList")) {
					continue;
				}
				dataParams.putParam(param.getName(), param.getTextTrim());
			}
		}
		
		Element productListElement = dataParam.element("productList");
		if (null != productListElement) {
			List<?> productList = productListElement.elements("product");
			if (CollectionUtils.isNotNull(productList)) {
				for (Object object : productList) {
					AIDataParams.Product product = dataParams.newProduct();
					Element productElement = (Element) object;
					product.setProductName(productElement.elementTextTrim("productName"));
					product.setProductUnit(productElement.elementTextTrim("productUnit"));
					product.setBaseCount(ObjectUtils.getObjBigDecimal(productElement.elementTextTrim("baseCount")));
					dataParams.getProductList().add(product);
				}
			}
		}
		
		Element conditionListElement = dataParam.element("conditionList");
		if (null != conditionListElement) {
			List<?> conditionList = conditionListElement.elements("condition");
			if (CollectionUtils.isNotNull(conditionList)) {
				for (Object condition : conditionList) {
					Element conditionElement = (Element) condition;
					List<?> objectConditionList = conditionElement.elements();
					if (CollectionUtils.isNotNull(objectConditionList)) {
						AIDataParams.Condition conditionClass = dataParams.newCondition();
						for (Object object : objectConditionList) {
							Element param = (Element) object;
							conditionClass.putParam(param.getName(), param.getTextTrim());
						}
						dataParams.getConditionList().add(conditionClass);
					}
				}
			}
		}
		evaluationFactor.setAIDataParams(dataParams);
	}
	
	/**
	 * @Description: 解析打分指标参数
	 * @author chenxw
	 * @param evaluationFactor
	 * @param element
	 */
	private static void resolverScoreFactorParams(EvaluationFactor evaluationFactor, Element element) {
		evaluationFactor.setScoreType(element.elementTextTrim("scoreType"));
		Element scoreItemListElement = element.element("scoreItemList");
		if (null != scoreItemListElement) {
			List<?> scoreItemList = scoreItemListElement.elements("scoreItem");
			if (null != scoreItemList && scoreItemList.size() > 0) {
				List<ScoreItem> objectiveItemList = new ArrayList<ScoreItem>();
				for (Object object : scoreItemList) {
					Element scoreItemElement = (Element) object;
					ScoreItem scoreItem = new ScoreItem();
					scoreItem.setId(UuidUtils.getUUID());
					scoreItem.setName(scoreItemElement.elementTextTrim("name"));
					scoreItem.setScore(ObjectUtils.getObjBigDecimal(scoreItemElement.elementTextTrim("score")));
					objectiveItemList.add(scoreItem);
				}
				evaluationFactor.setObjectiveItem(SerializeUtils.serializeObject(objectiveItemList));
			}
		}
	}
	
	/**
	 * @Description: 解析价格指标参数
	 * @author chenxw
	 * @param evaluationFactor
	 * @param element
	 * @param tenderProjectEntity
	 */
	private static void resolverPriceFactorParams(EvaluationFactor evaluationFactor, Element element, TenderProjectEntity tenderProjectEntity) {
		PriceParam priceParam = new PriceParam();
		priceParam.setFormula(element.elementTextTrim("remarks"));
		Element priceFormula = element.element("priceFormula");
		if (null != priceFormula) {
			// 基准价取样范围
			resolverSpecimenRangeParams(priceFormula, priceParam);
			// 基准价计算公式
			resolverBasePriceParams(priceFormula, priceParam);
			// 得分计算公式
			resolverScoreCalculationParams(priceFormula, priceParam);
		}
		// 为价格公式设置拦标价
		if (null == priceParam.getSpecimenParam().getPriceCeiling()) {
			priceParam.getSpecimenParam().setPriceCeiling(tenderProjectEntity.getBidPriceLimit());
		}
		priceParam.setPriceFactorScore(evaluationFactor.getFactorScore());
		priceParam.setPriceFactorWeight(evaluationFactor.getFactorWeight());
		evaluationFactor.setComputerParams(priceParam.getPriceParamSerializable());
	}
	
	/**
	 * @Description: 得分计算公式
	 * @author chenxw
	 * @param priceFormula
	 * @param priceParam
	 */
	private static void resolverScoreCalculationParams(Element priceFormula, PriceParam priceParam) {
		Element scoreCalculation = priceFormula.element("scoreCalculation");
		if (null != scoreCalculation) {
			priceParam.setPriceScorePoint(ObjectUtils.getIntValue(scoreCalculation.elementTextTrim("priceScorePoint"), priceParam.getPriceScorePoint()));
			CalculationParam calculationParam = new CalculationParam();
			calculationParam.setCalculationType(scoreCalculation.elementTextTrim("calculateType"));
			calculationParam.setMinuend(scoreCalculation.elementTextTrim("minuend"));
			calculationParam.setSubtrahend(scoreCalculation.elementTextTrim("subtrahend"));
			calculationParam.setDivisor(scoreCalculation.elementTextTrim("divisor"));
			calculationParam.setDeviateRatePoint(ObjectUtils.getObjBigDecimal(scoreCalculation.elementTextTrim("deviateRatePoint")));
			
			//性价比法
			Element costPerformance = scoreCalculation.element("costPerformance");
			if (null != costPerformance) {
				calculationParam.setXjbScoreType(costPerformance.elementTextTrim("scoreBase"));
			}
			
			if (StringUtils.equals(CalculationParam.LINEAR_DEVIATE, calculationParam.getCalculationType())) {
				resolverDeviateRateParams(scoreCalculation.element("linearDeviate"), calculationParam);
			} else if (StringUtils.equals(CalculationParam.ABSOLUTE, calculationParam.getCalculationType())) {
				resolverDeviateRateParams(scoreCalculation.element("absolute"), calculationParam);
			}else if(StringUtils.equals(CalculationParam.PRICE_DEVIATEA, calculationParam.getCalculationType())){
				
				resolverPriceDeviateAParams(scoreCalculation.element("priceDeviateA"), calculationParam);
			}else if(StringUtils.equals(CalculationParam.STAGE_DEVIATE, calculationParam.getCalculationType())){
				
				resolverStageDeviateParams(scoreCalculation.element("stageDeviate"), calculationParam);
				resolverDeviateRateParams(scoreCalculation.element("absolute"), calculationParam);
			}
//			resolverPriceDeviateAParams(scoreCalculation.element("priceDeviateA"), calculationParam);
//			
//			resolverStageDeviateParams(scoreCalculation.element("stageDeviate"), calculationParam);
//			
			priceParam.setCalculationParam(calculationParam);
		}
	}
	
	/**
	 * @Description: 偏差率法
	 * @author chenxw
	 * @param element
	 * @param calculationParam 
	 */
	private static void resolverStageDeviateParams(Element stageDeviate, CalculationParam calculationParam) {
		if (null != stageDeviate) {
			calculationParam.setAbsoluteDeviate(stageDeviate.elementTextTrim("absoluteType"));
			calculationParam.setDefaultMinScore(ObjectUtils.getObjBigDecimal(stageDeviate.elementTextTrim("defaultScore")));
		}
	}

	/**
	 * @Description: 价格偏离法
	 * @author chenxw
	 * @param priceDeviateAElement
	 * @param calculationParam 
	 */
	private static void resolverPriceDeviateAParams(Element priceDeviateAElement, CalculationParam calculationParam) {
		if (null != priceDeviateAElement) {
			calculationParam.setPriceDeviateRangeMin(BigDecimalUtils.mul(ObjectUtils.getObjBigDecimal(priceDeviateAElement.elementTextTrim("deviateRangeMin")), new BigDecimal(10000)));
			calculationParam.setPriceDeviateRangeMax(BigDecimalUtils.mul(ObjectUtils.getObjBigDecimal(priceDeviateAElement.elementTextTrim("deviateRangeMax")), new BigDecimal(10000)));
			calculationParam.setInDeviateRangeScore(ObjectUtils.getObjBigDecimal(priceDeviateAElement.elementTextTrim("inDeviateRangeScore")));
			calculationParam.setOtherDeviateRangeBaseScore(ObjectUtils.getObjBigDecimal(priceDeviateAElement.elementTextTrim("otherDeviateRangeBaseScore")));
			Element deviateItemList = priceDeviateAElement.element("deviateItemList");
			if (null != deviateItemList) {
				List<?> deviateItems = deviateItemList.elements("deviateItem");
				if (null != deviateItems && deviateItems.size() > 0) {
					List<DeviatePriceItem> higherDeviatePriceList = new ArrayList<DeviatePriceItem>();
					List<DeviatePriceItem> lowerDeviatePriceList = new ArrayList<DeviatePriceItem>();
					for (Object object : deviateItems) {
						Element deviateItem = (Element) object;
						boolean isHighDeviate = "true".equalsIgnoreCase(deviateItem.elementTextTrim("isHighDeviate"));
						DeviatePriceItem deviatePriceItem = new DeviatePriceItem();
						deviatePriceItem.setDeviateRateStart(BigDecimalUtils.mul(ObjectUtils.getObjBigDecimal(deviateItem.elementTextTrim("deviateRateStart")), new BigDecimal(10000)));
						deviatePriceItem.setDeviatePercentUnit(BigDecimalUtils.mul(ObjectUtils.getObjBigDecimal(deviateItem.elementTextTrim("deviatePercentUnit")), new BigDecimal(10000)));
						deviatePriceItem.setReduceScore("02".equals(deviateItem.elementTextTrim("addOrReduceType")));
						deviatePriceItem.setDeviatePercentValue(ObjectUtils.getObjBigDecimal(deviateItem.elementTextTrim("deviatePercentValue")));
						deviatePriceItem.setDefaultScore(ObjectUtils.getObjBigDecimal(deviateItem.elementTextTrim("defaultScore")));
						deviatePriceItem.setHighDeviate(isHighDeviate);
						if (isHighDeviate) {
							higherDeviatePriceList.add(deviatePriceItem);
						} else {
							lowerDeviatePriceList.add(deviatePriceItem);
						}
					}
					calculationParam.setHigherDeviatePriceList(higherDeviatePriceList);
					calculationParam.setLowerDeviatePriceList(lowerDeviatePriceList);
				}
			}
		}
	}
	
	/**
	 * @Description: 基准价计算公式
	 * @author chenxw
	 * @param priceFormula
	 * @param priceParam
	 */
	private static void resolverBasePriceParams(Element priceFormula, PriceParam priceParam) {
		Element basePrice = priceFormula.element("basePrice");
		if (null != basePrice) {
			priceParam.setBasePricePoint(ObjectUtils.getIntValue(basePrice.elementTextTrim("basePricePoint"), priceParam.getBasePricePoint()));
			BenchmarkPriceParam benchmarkPriceParam = new BenchmarkPriceParam();
			benchmarkPriceParam.setBasePriceType(basePrice.elementTextTrim("basePriceType"));
			
			//评标价
			Element evaluationPrice = basePrice.element("evaluationPrice");
			if (null != evaluationPrice) {
				benchmarkPriceParam.setEvaluationPriceType(evaluationPrice.elementTextTrim("basePriceCategory"));
			}
			
			//算数平均法
			Element avgPrice = basePrice.element("avgPrice");
			if (null != avgPrice) {
				benchmarkPriceParam.setDownFloat("02".equals(avgPrice.elementTextTrim("floatType")));
				benchmarkPriceParam.setBasePriceFloat(ObjectUtils.getObjBigDecimal(avgPrice.elementTextTrim("floatValue")));
				benchmarkPriceParam.setUseMinPriceCondition("true".equalsIgnoreCase(avgPrice.elementTextTrim("useMinPriceCondition")));
				benchmarkPriceParam.setSupplierNum(ObjectUtils.getIntValue(avgPrice.elementTextTrim("supplierNum"), benchmarkPriceParam.getSupplierNum()));
				boolean isPriceLowSecondLowestPrice = "true".equals(avgPrice.elementTextTrim("priceLowSecondLowestPrice"));
				benchmarkPriceParam.setBasePriceUnderMinEvalPrice(isPriceLowSecondLowestPrice);
				if(isPriceLowSecondLowestPrice) {
					benchmarkPriceParam.setBelowFloat(ObjectUtils.getObjBigDecimal(avgPrice.elementTextTrim("belowRangeFloat")));
				}
				benchmarkPriceParam.setBasePriceUnderMinPrice("true".equalsIgnoreCase(avgPrice.elementTextTrim("basePriceUnderMinPrice")));
			}
			
			//算数平均值动态浮动
			Element avgFloatRange = basePrice.element("avgFloatRange");
			if(null != avgFloatRange){
	
				benchmarkPriceParam.setOutRangeFolat(ObjectUtils.getObjBigDecimal(avgFloatRange.elementTextTrim("outRangeFolat")));
				benchmarkPriceParam.setOutRangeFolatDownFloat("true".equals(avgFloatRange.elementTextTrim("outRangeFolatDownFloat")));
				Element avgFloatRangeItemListEle = avgFloatRange.element("avgFloatRangeItemList");
				List<Element> avgFloatRangeItemEleList = avgFloatRangeItemListEle.elements("avgFloatRangeItem");
				List<AvgFloatRangeItem> avgFloatRangeItemList = new ArrayList<AvgFloatRangeItem>();
				if(null != avgFloatRangeItemEleList && avgFloatRangeItemEleList.size() > 0){
					for(Element element : avgFloatRangeItemEleList){
						AvgFloatRangeItem avgFloatRangeItem  = new AvgFloatRangeItem();
						avgFloatRangeItem.setFloatRangeMin(ObjectUtils.getObjBigDecimal(element.elementTextTrim("floatRangeMin")));
						avgFloatRangeItem.setIncludeFloatRangeMin("true".equals(element.elementTextTrim("includeFloatRangeMin")));
						avgFloatRangeItem.setFloatRangeMax(ObjectUtils.getObjBigDecimal(element.elementTextTrim("floatRangeMax")));
						avgFloatRangeItem.setIncludeFloatRangeMax("true".equals(element.elementTextTrim("includeFloatRangeMax")));
						avgFloatRangeItem.setDownFloat("true".equals(element.elementTextTrim("downFloat")));
						avgFloatRangeItem.setAvgFloat(ObjectUtils.getObjBigDecimal(element.elementTextTrim("avgFloat")));
						avgFloatRangeItemList.add(avgFloatRangeItem);
					}
				}
				benchmarkPriceParam.setAvgFloatRangeItemList(avgFloatRangeItemList);
			}
			
			priceParam.setBasePriceParam(benchmarkPriceParam);
		}
	}
	
	/**
	 * @Description: 基准价取样范围
	 * @author chenxw
	 * @param priceFormula
	 * @param priceParam
	 */
	private static void resolverSpecimenRangeParams(Element priceFormula, PriceParam priceParam) {
		Element specimenRange = priceFormula.element("specimenRange");
		if (null != specimenRange) {
			BasePriceRangeParam basePriceRangeParam = new BasePriceRangeParam();
			basePriceRangeParam.setSpecimenType(specimenRange.elementTextTrim("specimenType"));
			
			if(BasePriceRangeParam.SPECIMEN_TYPE_COUNT.equals(specimenRange.elementTextTrim("specimenType"))){
				//数量
				resolverCountParams(specimenRange.element("count"), basePriceRangeParam);
			}else if(BasePriceRangeParam.SPECIMEN_TYPE_DISPERSION.equals(specimenRange.elementTextTrim("specimenType"))){
				//离散度
				resolverDispersionParams(specimenRange.element("dispersion"), basePriceRangeParam);
			}else if(BasePriceRangeParam.SPECIMEN_TYPE_DISPERSION_RANGE.equals(specimenRange.elementTextTrim("specimenType"))){
				//离散度限制范围
				resolverDispersionRangeParams(specimenRange.element("dispersionRange"), basePriceRangeParam);
			}else {
				//其他
				resolverOtherParams(specimenRange.element("other"), basePriceRangeParam);
			}
			
			
			priceParam.setSpecimenParam(basePriceRangeParam);
		}
	}
	
	/**
	 * @Description: 解析直线插值法和区间偏离法参数
	 * @author chenxw
	 * @param element
	 * @param calculationParam
	 */
	private static void resolverDeviateRateParams(Element element, CalculationParam calculationParam) {
		if (null != element) {
			calculationParam.setDeviateRangeMin(ObjectUtils.getObjBigDecimal(element.elementTextTrim("deviateRangeMin")));
			calculationParam.setDeviateRangeMax(ObjectUtils.getObjBigDecimal(element.elementTextTrim("deviateRangeMax")));
			calculationParam.setInDeviateRangeScore(ObjectUtils.getObjBigDecimal(element.elementTextTrim("inDeviateRangeScore")));
			calculationParam.setOtherDeviateRangeBaseScore(ObjectUtils.getObjBigDecimal(element.elementTextTrim("otherDeviateRangeBaseScore")));
			Element deviateItemList = element.element("deviateItemList");
			if (null != deviateItemList) {
				List<?> deviateItems = deviateItemList.elements("deviateItem");
				if (null != deviateItems && deviateItems.size() > 0) {
					List<DeviateRateItem> higherDeviateRateList = new ArrayList<DeviateRateItem>();
					List<DeviateRateItem> lowerDeviateRateList = new ArrayList<DeviateRateItem>();
					for (Object object : deviateItems) {
						Element deviateItem = (Element) object;
						boolean isHighDeviate = "true".equalsIgnoreCase(deviateItem.elementTextTrim("isHighDeviate"));
						DeviateRateItem deviateRateItem = new DeviateRateItem();
						deviateRateItem.setDeviateRateStart(ObjectUtils.getObjBigDecimal(deviateItem.elementTextTrim("deviateRateStart")));
						deviateRateItem.setDeviatePercentUnit(ObjectUtils.getObjBigDecimal(deviateItem.elementTextTrim("deviatePercentUnit")));
						deviateRateItem.setReduceScore("02".equals(deviateItem.elementTextTrim("addOrReduceType")));
						deviateRateItem.setDeviatePercentValue(ObjectUtils.getObjBigDecimal(deviateItem.elementTextTrim("deviatePercentValue")));
						deviateRateItem.setDefaultScore(ObjectUtils.getObjBigDecimal(deviateItem.elementTextTrim("defaultScore")));
						deviateRateItem.setHighDeviate(isHighDeviate);
						if (isHighDeviate) {
							higherDeviateRateList.add(deviateRateItem);
						} else {
							lowerDeviateRateList.add(deviateRateItem);
						}
					}
					calculationParam.setHigherDeviateRateList(higherDeviateRateList);
					calculationParam.setLowerDeviateRateList(lowerDeviateRateList);
				}
			}
		}
	}
	
	/**
	 * @Description: 解析其他参数
	 * @author chenxw
	 * @param other
	 * @param param
	 */
	private static void resolverOtherParams(Element other, BasePriceRangeParam param) {
		if (null != other) {
			String priceCeiling = other.elementTextTrim("priceCeiling");
			param.setPriceCeiling(ObjectUtils.getObjBigDecimal(priceCeiling));
			if (StringUtils.isNotEmpty(priceCeiling)) {
				param.setPriceCeilingType(BasePriceRangeParam.PRICE_CEILING_VALUE);
			}
			param.setCeilingType(other.elementTextTrim("priceCeilingType"));
			Element priceCeilingPercent = other.element("priceCeilingPercent");
			if (null != priceCeilingPercent) {
				param.setSupplierNum(ObjectUtils.getIntValue(priceCeilingPercent.elementTextTrim("supplierNum"), param.getSupplierNum()));
				param.setPriceCeilingPercent(ObjectUtils.getObjBigDecimal(priceCeilingPercent.elementTextTrim("priceCeilingPercent")));
				if(param.getSupplierNum() == 0){
					// 兼容以前的标签
					param.setSupplierNum(ObjectUtils.getIntValue(priceCeilingPercent.elementTextTrim("priceCeilingSupplierNum"), param.getSupplierNum()));
				}
				if(null == param.getPriceCeilingPercent()){
					// 兼容以前的标签
					param.setPriceCeilingPercent(ObjectUtils.getObjBigDecimal(priceCeilingPercent.elementTextTrim("priceCeilingLowPercent")));
				}
				
				param.setEffectiveSupplierNum(ObjectUtils.getIntValue(priceCeilingPercent.elementTextTrim("effectiveSupplierNum"), param.getEffectiveSupplierNum()));
			}
			param.setPriceScore(ObjectUtils.getObjBigDecimal(other.elementTextTrim("defaultScore")));
		}
	}
	
	/**
	 * @Description: 解析离散度参数
	 * @author chenxw
	 * @param dispersion
	 * @param param
	 */
	private static void resolverDispersionParams(Element dispersion, BasePriceRangeParam param) {
		if (null != dispersion) {
			param.setSupplierNum(ObjectUtils.getIntValue(dispersion.elementTextTrim("supplierNum"), param.getSupplierNum()));
			param.setHeigerSecondPercent(ObjectUtils.getFloatValue(dispersion.elementTextTrim("higherSecondPercent"), param.getHeigerSecondPercent()));
			param.setLessThanSecondPercent(ObjectUtils.getFloatValue(dispersion.elementTextTrim("lessThanSecondPercent"), param.getLessThanSecondPercent()));
			param.setPriceScore(ObjectUtils.getObjBigDecimal(dispersion.elementTextTrim("defaultScore")));
		}
	}
	
	/**
	 * @Description: 解析离散度限制范围参数
	 * @author wangx
	 * @param dispersionRange
	 * @param param
	 */
	private static void resolverDispersionRangeParams(Element dispersionRange, BasePriceRangeParam param) {
		if (null != dispersionRange) {
			param.setSupplierNum(ObjectUtils.getIntValue(dispersionRange.elementTextTrim("supplierNum"), param.getSupplierNum()));
			param.setDispersionFilterMaxRange(ObjectUtils.getObjBigDecimal(dispersionRange.elementTextTrim("downFloatPercent")));
			param.setIncludeDispersionFilterMinRange("true".equals(dispersionRange.elementTextTrim("includeDownFloatPercent")));
			param.setDispersionFilterMinRange(ObjectUtils.getObjBigDecimal(dispersionRange.elementTextTrim("upFloatPercent")));
			param.setIncludeDispersionFilterMaxRange("true".equals(dispersionRange.elementTextTrim("includeUpFloatPercent")));
			param.setPriceScore(ObjectUtils.getObjBigDecimal(dispersionRange.elementTextTrim("defaultScore")));
			param.setHeightSecondFirstDeviationPercent(ObjectUtils.getObjBigDecimal(dispersionRange.elementTextTrim("heightSecondFirstDeviationPercent")));
			param.setIncludeHeightSecondFirstDeviationPercent("true".equals(dispersionRange.elementTextTrim("includeHeightSecondFirstDeviationPercent")));
			param.setHeightAvgDeviationPercent(ObjectUtils.getObjBigDecimal(dispersionRange.elementTextTrim("heightAvgDeviationPercent")));
			param.setIncludeHeightAvgDeviationPercent("true".equals(dispersionRange.elementTextTrim("includeHeightAvgDeviationPercent")));
			
			Element specimenFilterConditionItemListEle = dispersionRange.element("specimenFilterConditionItemList");
			List<Element> specimenFilterConditionItemEleList = specimenFilterConditionItemListEle.elements("specimenFilterConditionItem");
			List<SpecimenFilterConditionItem> specimenFilterConditionItemList = new ArrayList<SpecimenFilterConditionItem>();
			if(null != specimenFilterConditionItemEleList && specimenFilterConditionItemEleList.size() > 0){
				for(Element element : specimenFilterConditionItemEleList){
					SpecimenFilterConditionItem specimenFilterConditionItem = new SpecimenFilterConditionItem();
					specimenFilterConditionItem.setCompareType(ObjectUtils.getObjString(element.elementTextTrim("compareType")));
					specimenFilterConditionItem.setDeviationPercent(ObjectUtils.getObjBigDecimal(element.elementTextTrim("deviationPercent")));
					specimenFilterConditionItem.setInclude("true".equals(element.elementTextTrim("include")));
					specimenFilterConditionItem.setRemoveLow("true".equals(element.elementTextTrim("isRemoveLow")));
					specimenFilterConditionItemList.add(specimenFilterConditionItem);
				}
				param.setSpecimenFilterConditionItemList(specimenFilterConditionItemList);
			}
		}
	}
	
	/**
	 * @Description: 解析数量参数
	 * @author chenxw
	 * @param count
	 * @param param
	 */
	private static void resolverCountParams(Element count, BasePriceRangeParam param) {
		if (null != count) {
			Element conditionList = count.element("conditionList");
			if (null != conditionList) {
				List<?> conditions = conditionList.elements("condition");
				if (CollectionUtils.isNotNull(conditions)) {
					param.setFilterCountByCondition(true);
					List<FilterCountConditionItem> conditionItemList = new ArrayList<FilterCountConditionItem>();
					for (Object object : conditions) {
						Element condition = (Element) object;
						FilterCountConditionItem conditionItem = new FilterCountConditionItem();
						conditionItem.setInclude("01".equals(condition.elementTextTrim("include")));
						conditionItem.setMorethanCount(ObjectUtils.getIntValue(condition.elementTextTrim("morethanCount"), conditionItem.getMorethanCount()));
						conditionItem.setFilterHighest(ObjectUtils.getIntValue(condition.elementTextTrim("filterHighest"), conditionItem.getMorethanCount()));
						conditionItem.setFilterLowest(ObjectUtils.getIntValue(condition.elementTextTrim("filterLowest"), conditionItem.getMorethanCount()));
						conditionItemList.add(conditionItem);
					}
					param.setFilterCountConditionList(conditionItemList);
				}
			}
			param.setPriceScore(ObjectUtils.getObjBigDecimal(count.elementTextTrim("defaultScore")));
		}
	}
	
}
