package com.gx.obe.common.file.utils;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.zip.CRC32;
import java.util.zip.CheckedInputStream;
import java.util.zip.CheckedOutputStream;
import java.util.zip.Deflater;
import java.util.zip.ZipException;
import java.util.zip.ZipInputStream;

import org.apache.log4j.Logger;
import org.apache.tools.zip.ZipEntry;
import org.apache.tools.zip.ZipFile;
import org.apache.tools.zip.ZipOutputStream;
import org.eclipse.swt.core.CallBack;

import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.message.Messages;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.FileUtils;

/**
 * 提供对多个文件与目录的压缩，并支持是否需要创建压缩源目录、中文路径
 */
public class ZipUtils {

	private static Logger log = Logger.getLogger(ZipUtils.class);
	
//	private static String ENCODING = "UTF-8";//PropertiesUtil.getProperty("zipEncoding", "GBK");//"UTF-8";// GBK支持中文
	private static String ENCODING = PropertiesUtils.getProperty(ConfigConstants.ZIP_ENCODING, "GBK");//"UTF-8";// GBK支持中文

	/**
	 * @Description: 压缩文件
	 * @author guoyr
	 * @param src  要进行压缩的文件源路径
	 * @param archive 指定到压缩文件夹的路径
	 * @param zipMessageReceiver 压缩文件的动态消息
	 * @param comment 描述
	 * @param isCreateSrcDir 是否创建源目录
	 * @return
	 * @throws FileNotFoundException 文件没有找到异常
	 * @throws IOException IO输入异常
	 */
	public static void zip(String[] src, String archive, String comment, boolean isCreateSrcDir, CallBack callBack)
	throws FileNotFoundException, IOException {
//		long start = Calendar.getInstance().getTimeInMillis();
		if(!new File(archive).exists()){
			new File(archive).getParentFile().mkdirs();
		}
		writeByApacheZipOutputStream(src, archive, comment,isCreateSrcDir, callBack);
//		if(null != zipMessageReceiver){
//			long end = Calendar.getInstance().getTimeInMillis();
//			zipMessageReceiver.isFinish(true, 0, end - start);
//		}
	}
	
	/**
	 * @Description：压缩文件目录 ,并且使用压文件的目录最后一个目录的名称创建压缩文件名称.保存在目录相同的地方.
	 * @author guoyr
	 * @param sourcedir  要压缩的文件目录路径
	 * @param comment  描述信息
	 * @param isCreateSrcDir 是否创建源目录
	 * @param zipMessageReceiver  压缩文件动态消息
	 * @throws FileNotFoundException  文件没有找到异常
	 * @throws IOException void  IO输入异常
	 */
	public static String zipDir(String sourcedir,String comment, boolean isCreateSrcDir, CallBack callBack) throws FileNotFoundException, IOException{
		File source=new File(sourcedir);
		if(!source.exists()){
			log.warn("目录"+sourcedir+" 不存在,压缩失败!");
			return null;
		} 
		//根据压缩的文件路径,创建文件名称
		 String zipFileName=source.getName()+".zip";
		 String archive=source.getParent()+File.separator+zipFileName;
		 zip(new String[]{sourcedir}, archive, comment,isCreateSrcDir,callBack);
		 return archive;
	}

	/**
	 * @author guoyr
	 * @param archive 压缩包路径
	 * @param decompressDir 解压路径
	 * @param unZipSubfile 解压子压缩文件
	 * @param deleteSource 删除源文件
	 * @param upZipCurrentFolder 是否解压到当前目录
	 * @param zipMessageReceiver 解压缩文件的动态消息
	 * @Description: 解压文件<BR>
	 * [使用 org.apache.tools.zip.ZipFile 解压文件，它与 java 类库中的 java.util.zip.ZipFile
	 * 使用方式是一新的，只不过多了设置编码方式的 接口。
	 * 注，apache 没有提供 ZipInputStream 类，所以只能使用它提供的ZipFile 来读取压缩文件。]<BR>
	 * @throws IOException
	 * @throws FileNotFoundException
	 * @throws ZipException
	 * @return
	 * @Exception
	 */
	public static boolean unZip(String archive, String decompressDir, boolean unZipSubfile, boolean deleteSource, boolean upZipCurrentFolder, CallBack callBack)
	 {
		callBackMsg(callBack, "");
		callBackMsg(callBack, "开始解压...");
		long start = Calendar.getInstance().getTimeInMillis();
		boolean unZipFlag = false;
		try {
			readByApacheZipFile(archive, decompressDir, unZipSubfile, deleteSource,upZipCurrentFolder, callBack);
			unZipFlag = true;
		} catch (FileNotFoundException e) {
			log.error("", e);
		} catch (ZipException e) {
			log.error("", e);
		} catch (IOException e) {
			log.error("", e);
		}
		callBackMsg(callBack, unZipFlag, "解压结束，"+Messages.Tips_timeUsed+DateUtils.timeSpan(Calendar.getInstance().getTimeInMillis() - start));
		return unZipFlag;
	}
	
	/**
	 * @Description: <BR>
	 * @author guoyr
	 * @param src  要进行压缩的文件源路径
	 * @param archive 指定到压缩文件夹的路径
	 * @param comment 描述
	 * @return
	 * @throws FileNotFoundException 文件没有找到异常
	 * @throws IOException IO输入异常
	 */
	private static void writeByApacheZipOutputStream(String[] src, String archive,
			String comment, boolean isCreateSrcDir, CallBack callBack) throws FileNotFoundException, IOException {
		// ----压缩文件：
		FileOutputStream f = new FileOutputStream(archive);
		// 使用指定校验和创建输出流
		CheckedOutputStream csum = new CheckedOutputStream(f, new CRC32());
		ZipOutputStream zos = new ZipOutputStream(csum);
		// 支持中文
		zos.setEncoding(ENCODING);
		BufferedOutputStream out = new BufferedOutputStream(zos);

		// 设置压缩包注释
		zos.setComment(comment);
		// 启用压缩
		zos.setMethod(ZipOutputStream.DEFLATED);
		// 压缩级别为最强压缩，但时间要花得多一点
		zos.setLevel(Deflater.BEST_COMPRESSION);
		
		// 如果为单个文件的压缩在这里修改
		for (int i = 0; i < src.length; i++) {
			String sourceName = src[i];
			String inZipPath = null;
			
			// 为了解决将一个文件或目录压缩到压缩文件中的目录下需要在要压缩的文件后面加|再加目录名称
			int zipPathIndex = sourceName.indexOf("|") ;
			if(zipPathIndex > 0){
				sourceName = src[i].substring(0, zipPathIndex).trim();
				inZipPath = src[i].substring(zipPathIndex + 1).trim();
			}
			File srcFile = new File(sourceName);
			if(!srcFile.exists()){
				continue;
			}
			String strSrcString = src[i];
			// 获取压缩源所在父目录
			sourceName = sourceName.replaceAll("\\\\", "/");
			String prefixDir = null;
			
			if (srcFile.isFile()) {
				prefixDir = sourceName.substring(0,
						sourceName.lastIndexOf("/") + 1);
			} else {
				prefixDir = (sourceName.replaceAll("/$", "") + "/");
				
				/**
				 * @modify by guoyr
				 * 为了解决指标压缩文件中包含目录的情况
				 * */
				prefixDir = sourceName.substring(0,
						sourceName.lastIndexOf("/") + 1);
			}
			
			// 如果不是根目录
			if (prefixDir.indexOf("/") != (prefixDir.length() - 1)
					&& isCreateSrcDir) {
				prefixDir = prefixDir.replaceAll("[^/]+/$", "");
			}
			// 开始压缩
			writeRecursive(zos, out, srcFile, prefixDir, inZipPath, callBack);
		}

		out.close();
		// 注：校验和要在流关闭后才准备，一定要放在流被关闭后使用
//		System.out.println("Checksum: " + csum.getChecksum().getValue());
		
	}
	
	/**
	 * @Description: <BR>
	 * @author guoyr
	 * [使用 org.apache.tools.zip.ZipFile 解压文件，它与 java 类库中的 java.util.zip.ZipFile
	 * 使用方式是一新的，只不过多了设置编码方式的 接口。
	 * 注，apache 没有提供 ZipInputStream 类，所以只能使用它提供的ZipFile 来读取压缩文件。]<BR>
	  * @param archive 压缩包路径
	 * @param decompressDir 解压路径
	 * @param unZipSubfile 解压子压缩文件
	 * @param deleteSource 删除源文件
	 * @param upZipCurrentFolder 是否解压到当前目录
	 * @param zipMessageReceiver 解压缩文件的动态消息
	 * @throws IOException
	 * @throws FileNotFoundException
	 * @throws ZipException
	 * @return
	 * @Exception
	 */
	private static void readByApacheZipFile(String archive, String decompressDir, boolean unZipSubfile, boolean deleteSource, boolean upZipCurrentFolder,CallBack callBack)
			throws IOException, FileNotFoundException, ZipException {
		BufferedInputStream bi = null;
		File sourceZip = new File(archive);
		archive = sourceZip.getAbsolutePath().replaceAll("\\\\", "/");
		if(null == decompressDir || "".equals(decompressDir)){
			decompressDir = sourceZip.getParent();
		}
		String zipDir = sourceZip.getName().lastIndexOf(".") > 0 ? sourceZip.getName().substring(0,sourceZip.getName().lastIndexOf(".")) : sourceZip.getName();
		if(!upZipCurrentFolder){
			decompressDir = decompressDir+"/"+zipDir;
		}
		File decompress = new File(decompressDir);
		decompressDir = decompress.getAbsolutePath().replaceAll("\\\\", "/");
		
		ZipFile zf = null;
		try {
			zf = new ZipFile(sourceZip.getPath(), ENCODING);
		} catch (Exception e2) {
			log.error("UTF-8格式解压失败", e2);
			try {
				zf = new ZipFile(sourceZip.getPath(), "GBK");// GBK支持中文
			} catch (Exception e) {
				zf = new ZipFile(sourceZip.getPath(), "UTF-8");// UTF-8支持中文
				log.error("GBK格式解压失败", e);
				
			}
		}
		if(null != zf) {
			Enumeration<?> e = zf.getEntries();
//		String rootPath = archive.substring(0,archive.lastIndexOf("/")+1);
			while(e.hasMoreElements()){
				ZipEntry ze2 = (ZipEntry) e.nextElement();
				String entryName = ze2.getName();
				if("".equals(entryName)){
					continue;
				}
				String subFilePath = decompressDir + "/" + entryName;
				if (ze2.isDirectory()) {
//				System.out.println("正在创建解压目录 - " + entryName);
					File decompressDirFile = new File(subFilePath);
					if (!decompressDirFile.exists()) {
						decompressDirFile.mkdirs();
					}
				} else {
					
					long start = Calendar.getInstance().getTimeInMillis();
//				callBackMsg(callBack, "正在解压"+(entryName.indexOf("/")>0?entryName.substring(entryName.lastIndexOf("/")+1):entryName));
//				System.out.println("正在创建解压文件 - " + entryName);
					String fileDir = subFilePath.substring(0, subFilePath.lastIndexOf("/"));
					File fileDirFile = new File(fileDir);
					if (!fileDirFile.exists()) {
						fileDirFile.mkdirs();
					}
					if(new File(subFilePath).exists()){
						new File(subFilePath).delete();
					}else {
						new File(subFilePath).getParentFile().mkdirs();
					}
					BufferedOutputStream bos = null;
					long size = 0;
					try {
						bos = new BufferedOutputStream(
								new FileOutputStream(subFilePath));
						bi = new BufferedInputStream(zf.getInputStream(ze2));
						byte[] readContent = new byte[1024];
						int readCount = bi.read(readContent);
						while (readCount != -1) {
							size += readCount;
							bos.write(readContent, 0, readCount);
							readCount = bi.read(readContent);
						}
					} catch (Exception e1) {
					}finally{
						if(null != bos){
							bos.flush();
							bos.close();
						}
						if(null != bi){
							bi.close();
						}
					}
					callBackMsg(callBack, true, entryName.indexOf("/")>0?entryName.substring(entryName.lastIndexOf("/")+1):entryName + "解压完成（大小："+FileUtils.getFileSize(size) +"，"+Messages.Tips_timeUsed+DateUtils.timeSpan(Calendar.getInstance().getTimeInMillis()-start)+"）");
					
					if(entryName.endsWith(".zip") && unZipSubfile){
						readByApacheZipFile(subFilePath, null, unZipSubfile, deleteSource, upZipCurrentFolder,callBack);
//					readByApacheZipFile(subFilePath, null, unZipSubfile, deleteSource, false,callBack);
					}
				}
			}
			zf.close();
			if(deleteSource){
				File zipFile = new File(archive);
				if(!zipFile.delete()){
					zipFile.deleteOnExit();
				}
			}
		}
	}

	/**
	 * @Description: <BR>
	 * @author guoyr
	 * [使用 java api 中的 ZipInputStream 类解压文件，但如果压缩时采用了
	 * org.apache.tools.zip.ZipOutputStream时，而不是 java 类库中的
	 * java.util.zip.ZipOutputStream时，该方法不能使用，原因就是编码方 式不一致导致，运行时会抛如下异常：
	 * java.lang.IllegalArgumentException at
	 * java.util.zip.ZipInputStream.getUTF8String(ZipInputStream.java:290)
	 * 当然，如果压缩包使用的是java类库的java.util.zip.ZipOutputStream 压缩而成是不会有问题的，但它不支持中文 ]<BR>
	 * [功能详细描述]
	 * @param archive
	 *            压缩包路径
	 * @param decompressDir
	 *            解压路径
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	public void readByZipInputStream(String archive, String decompressDir, CallBack callBack)
			throws FileNotFoundException, IOException {
		BufferedInputStream bi;
		// ----解压文件(ZIP文件的解压缩实质上就是从输入流中读取数据):
//		System.out.println("开始读压缩文件");
		FileInputStream fi = new FileInputStream(archive);
		CheckedInputStream csumi = new CheckedInputStream(fi, new CRC32());
		ZipInputStream in2 = new ZipInputStream(csumi);
		bi = new BufferedInputStream(in2);
		java.util.zip.ZipEntry ze;// 压缩文件条目
		// 遍历压缩包中的文件条目
		while ((ze = in2.getNextEntry()) != null) {
			String entryName = ze.getName();
			if (ze.isDirectory()) {
//				System.out.println("正在创建解压目录 - " + entryName);
				File decompressDirFile = new File(decompressDir + "/"
						+ entryName);
				if (!decompressDirFile.exists()) {
					decompressDirFile.mkdirs();
				}
			} else {
				long start = Calendar.getInstance().getTimeInMillis();
//				System.out.println("正在创建解压文件 - " + entryName);
				BufferedOutputStream bos = new BufferedOutputStream(
						new FileOutputStream(
								decompressDir
										+ "/"
										+ entryName.substring(
												entryName.lastIndexOf("\\"),
												entryName.length()
														- (entryName
																.lastIndexOf("\\") - 2))));
				byte[] buffer = new byte[1024];
				long size = 0;
				int readCount = bi.read(buffer);
				while (readCount != -1) {
					size += readCount;
					bos.write(buffer, 0, readCount);
					readCount = bi.read(buffer);
				}
				bos.close();
				
				callBackMsg(callBack, true, entryName.indexOf("/")>0?entryName.substring(entryName.lastIndexOf("/")+1):entryName + "解压完成（大小："+FileUtils.getFileSize(size) +"，"+Messages.Tips_timeUsed+DateUtils.timeSpan(Calendar.getInstance().getTimeInMillis()-start)+"）");
				
			}
		}
		bi.close();
		in2.close();
		csumi.close();
		fi.close();
//		System.out.println("Checksum: " + csumi.getChecksum().getValue());
	}

	/**
	 * @Description: <BR>
	 * @author guoyr
	 * [递归压缩使用 org.apache.tools.zip.ZipOutputStream 类进行压缩，它的好处就是支持中文路径， 而Java类库中的
	 * java.util.zip.ZipOutputStream 压缩中文文件名时压缩包会出现乱码。 使用 apache 中的这个类与 java
	 * 类库中的用法是一新的，只是能设置编码方式了。]<BR>
	 * [功能详细描述]
	 * @param zos
	 * @param bo
	 * @param srcFile
	 * @param prefixDir
	 * @throws IOException
	 * @throws FileNotFoundException
	 */
	private static void writeRecursive(ZipOutputStream zos, BufferedOutputStream bo,
			File srcFile, String prefixDir, String inZipPath, CallBack callBack) throws IOException,
			FileNotFoundException {
		ZipEntry zipEntry;
		String filePath = srcFile.getAbsolutePath().replaceAll("\\\\", "/")
		.replaceAll("//", "/");
		if (srcFile.isDirectory()) {
			filePath = filePath.replaceAll("/$", "") + "/";
		}
		String entryName = filePath.replace(prefixDir, "").replaceAll("/$", "");
		if (srcFile.isDirectory()) {

			/**
			 * @modify by guoyr
			 * 为了解决指标压缩文件中包含目录的情况
			 * */
			File srcFiles[] = srcFile.listFiles();
			if(null != srcFiles && srcFiles.length >0)
				for (File file : srcFiles) {
					writeRecursive(zos, bo, file, prefixDir, inZipPath, callBack);
				}
		} else {
			long start = Calendar.getInstance().getTimeInMillis();
//			callBackMsg(callBack, "正在压缩"+(entryName.indexOf("/")>0?entryName.substring(entryName.lastIndexOf("/")+1):entryName) + "（大小："+FileUtil.getFileSize(srcFile.length()));
			// System.out.println("正在写文件 - " + srcFile.getAbsolutePath()
			// + " entryName=" + entryName);
			BufferedInputStream bi = new BufferedInputStream(
					new FileInputStream(srcFile));
			// 开始写入新的ZIP文件条目并将流定位到条目数据的开始处
			// 开始写入新的ZIP文件条目并将流定位到条目数据的开始处
			if(null != inZipPath && inZipPath.length() > 0){
				
				zipEntry = new ZipEntry(inZipPath + File.separator + entryName);
			}else {
				zipEntry = new ZipEntry(entryName);
			}
			zos.putNextEntry(zipEntry);
			byte[] buffer = new byte[1024];
			int readCount = bi.read(buffer);
			while (readCount != -1) {
				bo.write(buffer, 0, readCount);
				readCount = bi.read(buffer);
			}
			// 注，在使用缓冲流写压缩文件时，一个条件完后一定要刷新一把，不
			// 然可能有的内容就会存入到后面条目中去了
			bo.flush();
			// 文件读完后关闭
			bi.close();
			callBackMsg(callBack, entryName.indexOf("/")>0?entryName.substring(entryName.lastIndexOf("/")+1):entryName + "压缩完成（大小："+FileUtils.getFileSize(srcFile.length()) +"，"+Messages.Tips_timeUsed+DateUtils.timeSpan(Calendar.getInstance().getTimeInMillis()-start)+"）");
		}
	}
	private static void callBackMsg(CallBack callBack, String msg){
		if(null != callBack){
			callBack.callBack(0,msg);
		}
	}
	private static void callBackMsg(CallBack callBack,boolean bool, String msg){
		if(null != callBack){
			callBack.callBack(bool?1:-1, msg);
		}
	}

	/**
	 * @Description: 解压文件
	 * @author chenxw
	 * @param archive
	 * @param decompressDir
	 * @return 
	 */
	public static boolean unZipFile(String archive, String decompressDir) {
		try {
			unZip(archive, decompressDir, true, true, true, null);
			return true;
		} catch (Exception e) {
			log.error("解压文件失败", e);
			return false;
		}
	}
	
	/**
	 * @param args
	 * @throws IOException
	 */
	public static void main(String[] args) throws IOException {
		String src = "D:\\work\\codetemplates.xml";// 指定压缩源，可以是目录或文件
		String src1 = "D:\\work\\范本工具.png";// 指定压缩源，可以是目录或文件
		String src2 = "D:\\work\\公司名称变更通知.jpg";
		String src3 = "D:\\work\\spark";
		String src4 = "D:\\work\\zctServer\\com\\gpcsoft";
		String decompressDir = "e:\\temp\\textUnZip";// 解压路径
		String archive = "e:\\temp\\7E6E05509B1C86D7D2EF3FBC7180E795";// 压缩包路径
		String comment = "Java Zip 测试.";// 压缩包注释
		String[] arraySrcStrings = new String[] {src, src1, src2, src3, src4};
		// ----压缩文件或目录
		String dir="D:\\test";
		
//		zipDir(dir,"D:\\test3",true, null);
//		zip(arraySrcStrings,  archive, comment, false, null);
		/*
		 * 读压缩文件，注释掉，因为使用的是apache的压缩类，所以使用java类库中 解压类时出错，这里不能运行
		 */
		// readByZipInputStream();
		// ----使用apace ZipFile读取压缩文件
		archive = "C:\\Users\\guoyr\\AppData\\Local\\Temp\\OBE_TEMP\\bidFile\\a1a24f10ef064c1d8b3ebb110d133847\\f625007229d248b6a47b8514e1703450\\00\\bidFile.zip";// 压缩包路径
		 unZip(archive, null,true,true,true,null);
	}
}
