package com.gx.obe.common.file.widget;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.widgets.EMenu;
import org.eclipse.swt.core.widgets.EMenu.ClickListener;
import org.eclipse.swt.core.widgets.ImageButton;
import org.eclipse.swt.events.MenuDetectEvent;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.bind.call.Callback;
import com.gx.obe.common.file.utils.FileUtils;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.util.utils.RuntimeUtils;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class FileButton extends Composite {
	
	protected File file;
	
	protected ImageButton imageButton;
	
	private List<ItemParam> itemParamList = new ArrayList<>();
	
	public FileButton(Composite parent, File file) {
		this(parent, file, Optional.of(file).map(File::getName).get());
	}
	
	/**
	 * @wbp.parser.constructor
	 */
	public FileButton(Composite parent, File file, String fileName) {
		super(parent, SWT.NONE);
		
		this.file = file;
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		imageButton = new ImageButton(this, SWT.BOTTOM | SWT.CENTER | SWT.WRAP);
		GridData gd_imageButton = new GridData(SWT.CENTER, SWT.FILL, false, true, 1, 1);
		gd_imageButton.widthHint = 100;
		imageButton.setLayoutData(gd_imageButton);
		
		imageButton.setForeground(SWTResourceManager.getColor(100, 100, 100));
		imageButton.setFont(SWTResourceManager.getFont("微软雅黑", 9, SWT.NORMAL));
		
		imageButton.setActiveBorderColor(SWTResourceManager.getColor(77, 163, 19));
		imageButton.setActiveForeground(SWTResourceManager.getColor(100, 100, 100));
		imageButton.setActiveBgColor(SWTResourceManager.getColor(205, 225, 205));
		
		imageButton.setGAP(0);
		imageButton.setAlpha(120);
		imageButton.setArcWidth(6);
		
		imageButton.setText(fileName);
		imageButton.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, getFileImagePath(fileName)));
		imageButton.addMouseListener(MouseListener.mouseDoubleClickAdapter(this::openFile));
		imageButton.addMenuDetectListener(this::menuDetect);
	}
	
	/**
	 * @Description:  添加菜单项
	 * @author guoyr
	 * @param menuItemText
	 * @param menuItemImage
	 * @param enabled
	 * @param callBack
	 */
	public void addMenuItem(String menuItemText, Image menuItemImage, boolean enabled, Callback clickListener) {
		itemParamList.add(new ItemParam(menuItemText, menuItemImage, enabled, clickListener));
	}
	
	private void menuDetect(MenuDetectEvent e) {
		EMenu menu = new EMenu(getShell());
		menu.addMenuItem("打开", null, true, new ClickListener() {
			public void click() {
				openFile(null);
			}
		});
		for (ItemParam itemParam : itemParamList) {
			menu.addMenuItem(itemParam.menuItemText, itemParam.menuItemImage, itemParam.enabled, new ClickListener() {
				public void click() {
					itemParam.clickListener.call();
				}
			});
		}
		menu.open();
	}
	
	
	
	protected void openFile(MouseEvent e) {
		if (!file.exists()) {
			MessageDialog.openShortMessage("文件不存在！");
			return;
		}
		RuntimeUtils.openFIle(file);
	}
	
	/**
	 * @Description:
	 * @author mazc
	 * @param fileName
	 * @return
	 */
	private String getFileImagePath(String fileName) {
		switch (FileUtils.funGetSuffix.apply(fileName)) {
			case "pdf": return "icons/pdf_64.png";
			case "xml": return "icons/xml_64.png";
			case "doc": case "docx": return "icons/word_64.png";
			case "xls": case "xlsx": return "icons/excel_64.png";
			case "png": case "bmp": case "gif": case "psd": case "jpg": case "jpeg": return "icons/image_64.png";
			default: return "icons/unknown_64.png";
		}
	}
	
	public File getFile() {
		return file;
	}

	static class ItemParam {
		
		String menuItemText;
		Image menuItemImage;
		boolean enabled;
		Callback clickListener;
		
		public ItemParam(String menuItemText, Image menuItemImage, boolean enabled, Callback clickListener) {
			this.menuItemText = menuItemText;
			this.menuItemImage = menuItemImage;
			this.enabled = enabled;
			this.clickListener = clickListener;
		}
		
	}
	
}
